@@ -67,7 +67,9 @@ def handle_password(req, meta_data):
             msg = _("Request is too large.")
             raise exc.HTTPBadRequest(explanation=msg)
 
-        instance = objects.Instance.get_by_uuid(ctxt, meta_data.uuid)
+        im = objects.InstanceMapping.get_by_instance_uuid(ctxt, meta_data.uuid)
+        with context.target_cell(ctxt, im.cell_mapping) as cctxt:
+            instance = objects.Instance.get_by_uuid(cctxt, meta_data.uuid)
         instance.system_metadata.update(convert_password(ctxt, req.body))
         instance.save()
     else:
