@@ -16,7 +16,6 @@
 
 """The Assisted volume snapshots extension."""
 
-from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import six
 from webob import exc
@@ -27,19 +26,14 @@ from nova.api.openstack import wsgi
 from nova.api import validation
 from nova import compute
 from nova import exception
-from nova.i18n import _LI
-
-
-LOG = logging.getLogger(__name__)
-ALIAS = 'os-assisted-volume-snapshots'
-authorize = extensions.os_compute_authorizer(ALIAS)
+from nova.policies import assisted_volume_snapshots as avs_policies
 
 
 class AssistedVolumeSnapshotsController(wsgi.Controller):
     """The Assisted volume snapshots API controller for the OpenStack API."""
 
     def __init__(self):
-        self.compute_api = compute.API(skip_policy_check=True)
+        self.compute_api = compute.API()
         super(AssistedVolumeSnapshotsController, self).__init__()
 
     @extensions.expected_errors(400)
@@ -47,29 +41,33 @@ class AssistedVolumeSnapshotsController(wsgi.Controller):
     def create(self, req, body):
         """Creates a new snapshot."""
         context = req.environ['nova.context']
-        authorize(context, action='create')
+        context.can(avs_policies.POLICY_ROOT % 'create')
 
         snapshot = body['snapshot']
         create_info = snapshot['create_info']
         volume_id = snapshot['volume_id']
 
-        LOG.info(_LI("Create assisted snapshot from volume %s"), volume_id,
-                  context=context)
         try:
             return self.compute_api.volume_snapshot_create(context, volume_id,
                                                            create_info)
         except (exception.VolumeBDMNotFound,
+                exception.VolumeBDMIsMultiAttach,
                 exception.InvalidVolume) as error:
             raise exc.HTTPBadRequest(explanation=error.format_message())
+        except (exception.InstanceInvalidState,
+                exception.InstanceNotReady) as e:
+            # TODO(mriedem) InstanceInvalidState and InstanceNotReady would
+            # normally result in a 409 but that would require bumping the
+            # microversion, which we should just do in a single microversion
+            # across all APIs when we fix status code wrinkles.
+            raise exc.HTTPBadRequest(explanation=e.format_message())
 
     @wsgi.response(204)
     @extensions.expected_errors((400, 404))
     def delete(self, req, id):
         """Delete a snapshot."""
         context = req.environ['nova.context']
-        authorize(context, action='delete')
-
-        LOG.info(_LI("Delete snapshot with id: %s"), id, context=context)
+        context.can(avs_policies.POLICY_ROOT % 'delete')
 
         delete_metadata = {}
         delete_metadata.update(req.GET)
@@ -88,22 +86,10 @@ class AssistedVolumeSnapshotsController(wsgi.Controller):
             raise exc.HTTPBadRequest(explanation=error.format_message())
         except exception.NotFound as e:
             return exc.HTTPNotFound(explanation=e.format_message())
-
-
-class AssistedVolumeSnapshots(extensions.V21APIExtensionBase):
-    """Assisted volume snapshots."""
-
-    name = "AssistedVolumeSnapshots"
-    alias = ALIAS
-    version = 1
-
-    def get_resources(self):
-        res = [extensions.ResourceExtension(ALIAS,
-                                    AssistedVolumeSnapshotsController())]
-        return res
-
-    def get_controller_extensions(self):
-        """It's an abstract function V21APIExtensionBase and the extension
-        will not be loaded without it.
-        """
-        return []
+        except (exception.InstanceInvalidState,
+                exception.InstanceNotReady) as e:
+            # TODO(mriedem) InstanceInvalidState and InstanceNotReady would
+            # normally result in a 409 but that would require bumping the
+            # microversion, which we should just do in a single microversion
+            # across all APIs when we fix status code wrinkles.
+            raise exc.HTTPBadRequest(explanation=e.format_message())
