@@ -24,10 +24,7 @@ from nova.api.openstack import wsgi
 from nova import compute
 from nova.i18n import _
 from nova import network
-
-
-ALIAS = 'os-virtual-interfaces'
-authorize = extensions.os_compute_authorizer(ALIAS)
+from nova.policies import virtual_interfaces as vif_policies
 
 
 def _translate_vif_summary_view(req, vif):
@@ -35,8 +32,7 @@ def _translate_vif_summary_view(req, vif):
     d = {}
     d['id'] = vif.uuid
     d['mac_address'] = vif.address
-    if (req.api_version_request >=
-        api_version_request.APIVersionRequest("2.12")):
+    if api_version_request.is_supported(req, min_version='2.12'):
         d['net_id'] = vif.net_uuid
     # NOTE(gmann): This is for v2.1 compatible mode where response should be
     # same as v2 one.
@@ -47,17 +43,19 @@ def _translate_vif_summary_view(req, vif):
 
 class ServerVirtualInterfaceController(wsgi.Controller):
     """The instance VIF API controller for the OpenStack API.
+
+       This API is deprecated from the Microversion '2.44'.
     """
 
     def __init__(self):
-        self.compute_api = compute.API(skip_policy_check=True)
-        self.network_api = network.API(skip_policy_check=True)
+        self.compute_api = compute.API()
+        self.network_api = network.API()
         super(ServerVirtualInterfaceController, self).__init__()
 
     def _items(self, req, server_id, entity_maker):
         """Returns a list of VIFs, transformed through entity_maker."""
         context = req.environ['nova.context']
-        authorize(context)
+        context.can(vif_policies.BASE_POLICY_NAME)
         instance = common.get_instance(self.compute_api, context, server_id)
 
         try:
@@ -70,30 +68,9 @@ class ServerVirtualInterfaceController(wsgi.Controller):
         res = [entity_maker(req, vif) for vif in limited_list]
         return {'virtual_interfaces': res}
 
+    @wsgi.Controller.api_version("2.1", "2.43")
     @extensions.expected_errors((400, 404))
     def index(self, req, server_id):
         """Returns the list of VIFs for a given instance."""
         return self._items(req, server_id,
                            entity_maker=_translate_vif_summary_view)
-
-
-class VirtualInterfaces(extensions.V21APIExtensionBase):
-    """Virtual interface support."""
-
-    name = "VirtualInterfaces"
-    alias = ALIAS
-    version = 1
-
-    def get_resources(self):
-        resources = []
-
-        res = extensions.ResourceExtension(
-            ALIAS,
-            controller=ServerVirtualInterfaceController(),
-            parent=dict(member_name='server', collection_name='servers'))
-        resources.append(res)
-
-        return resources
-
-    def get_controller_extensions(self):
-        return []
