@@ -12,7 +12,6 @@
 """Placement API handlers for resource providers."""
 
 import copy
-import jsonschema
 
 from oslo_db import exception as db_exc
 from oslo_serialization import jsonutils
@@ -22,6 +21,7 @@ import webob
 
 from nova.api.openstack.placement import microversion
 from nova.api.openstack.placement import util
+from nova.api.openstack.placement import wsgi_wrapper
 from nova import exception
 from nova.i18n import _
 from nova import objects
@@ -74,79 +74,23 @@ GET_RPS_SCHEMA_1_3['properties']['member_of'] = {
 # having some set of capacity for some resources. The query string is a
 # comma-delimited set of "$RESOURCE_CLASS_NAME:$AMOUNT" strings. The validation
 # of the string is left up to the helper code in the
-# _normalize_resources_qs_param() function below.
+# normalize_resources_qs_param() function.
 GET_RPS_SCHEMA_1_4 = copy.deepcopy(GET_RPS_SCHEMA_1_3)
 GET_RPS_SCHEMA_1_4['properties']['resources'] = {
     "type": "string"
 }
 
 
-def _normalize_resources_qs_param(qs):
-    """Given a query string parameter for resources, validate it meets the
-    expected format and return a dict of amounts, keyed by resource class name.
-
-    The expected format of the resources parameter looks like so:
-
-        $RESOURCE_CLASS_NAME:$AMOUNT,$RESOURCE_CLASS_NAME:$AMOUNT
-
-    So, if the user was looking for resource providers that had room for an
-    instance that will consume 2 vCPUs, 1024 MB of RAM and 50GB of disk space,
-    they would use the following query string:
-
-        ?resources=VCPU:2,MEMORY_MB:1024:DISK_GB:50
-
-    The returned value would be:
-
-        {
-            "VCPU": 2,
-            "MEMORY_MB": 1024,
-            "DISK_GB": 50,
-        }
-
-    :param qs: The value of the 'resources' query string parameter
-    :raises `webob.exc.HTTPBadRequest` if the parameter's value isn't in the
-            expected format.
-    """
-    result = {}
-    resource_tuples = qs.split(',')
-    for rt in resource_tuples:
-        try:
-            rc_name, amount = rt.split(':')
-        except ValueError:
-            msg = _('Badly formed resources parameter. Expected resources '
-                    'query string parameter in form: '
-                    '?resources=VCPU:2,MEMORY_MB:1024. Got: %s.')
-            msg = msg % rt
-            raise webob.exc.HTTPBadRequest(msg,
-                    json_formatter=util.json_error_formatter)
-        try:
-            amount = int(amount)
-        except ValueError:
-            msg = _('Requested resource %(resource_name)s expected positive '
-                    'integer amount. Got: %(amount)s.')
-            msg = msg % {
-                'resource_name': rc_name,
-                'amount': amount,
-            }
-            raise webob.exc.HTTPBadRequest(msg,
-                    json_formatter=util.json_error_formatter)
-        if amount < 1:
-            msg = _('Requested resource %(resource_name)s requires '
-                    'amount >= 1. Got: %(amount)d.')
-            msg = msg % {
-                'resource_name': rc_name,
-                'amount': amount,
-            }
-            raise webob.exc.HTTPBadRequest(msg,
-                    json_formatter=util.json_error_formatter)
-        result[rc_name] = amount
-    return result
-
-
 def _serialize_links(environ, resource_provider):
     url = util.resource_provider_url(environ, resource_provider)
     links = [{'rel': 'self', 'href': url}]
-    for rel in ('aggregates', 'inventories', 'usages'):
+    rel_types = ['inventories', 'usages']
+    want_version = environ[microversion.MICROVERSION_ENVIRON]
+    if want_version >= (1, 1):
+        rel_types.append('aggregates')
+    if want_version >= (1, 6):
+        rel_types.append('traits')
+    for rel in rel_types:
         links.append({'rel': rel, 'href': '%s/%s' % (url, rel)})
     return links
 
@@ -169,7 +113,7 @@ def _serialize_providers(environ, resource_providers):
     return {"resource_providers": output}
 
 
-@webob.dec.wsgify
+@wsgi_wrapper.PlacementWsgify
 @util.require_content('application/json')
 def create_resource_provider(req):
     """POST to create a resource provider.
@@ -187,14 +131,12 @@ def create_resource_provider(req):
         resource_provider.create()
     except db_exc.DBDuplicateEntry as exc:
         raise webob.exc.HTTPConflict(
-            _('Conflicting resource provider already exists: %(error)s') %
-            {'error': exc},
-            json_formatter=util.json_error_formatter)
+            _('Conflicting resource provider %(name)s already exists.') %
+            {'name': data['name']})
     except exception.ObjectActionError as exc:
         raise webob.exc.HTTPBadRequest(
             _('Unable to create resource provider %(rp_uuid)s: %(error)s') %
-            {'rp_uuid': uuid, 'error': exc},
-            json_formatter=util.json_error_formatter)
+            {'rp_uuid': uuid, 'error': exc})
 
     req.response.location = util.resource_provider_url(
         req.environ, resource_provider)
@@ -203,7 +145,7 @@ def create_resource_provider(req):
     return req.response
 
 
-@webob.dec.wsgify
+@wsgi_wrapper.PlacementWsgify
 def delete_resource_provider(req):
     """DELETE to destroy a single resource provider.
 
@@ -219,8 +161,7 @@ def delete_resource_provider(req):
     except exception.ResourceProviderInUse as exc:
         raise webob.exc.HTTPConflict(
             _('Unable to delete resource provider %(rp_uuid)s: %(error)s') %
-            {'rp_uuid': uuid, 'error': exc},
-            json_formatter=util.json_error_formatter)
+            {'rp_uuid': uuid, 'error': exc})
     except exception.NotFound as exc:
         raise webob.exc.HTTPNotFound(
             _("No resource provider with uuid %s found for delete") % uuid)
@@ -229,7 +170,7 @@ def delete_resource_provider(req):
     return req.response
 
 
-@webob.dec.wsgify
+@wsgi_wrapper.PlacementWsgify
 @util.check_accept('application/json')
 def get_resource_provider(req):
     """Get a single resource provider.
@@ -250,7 +191,7 @@ def get_resource_provider(req):
     return req.response
 
 
-@webob.dec.wsgify
+@wsgi_wrapper.PlacementWsgify
 @util.check_accept('application/json')
 def list_resource_providers(req):
     """GET a list of resource providers.
@@ -266,14 +207,8 @@ def list_resource_providers(req):
         schema = GET_RPS_SCHEMA_1_3
     if want_version >= (1, 4):
         schema = GET_RPS_SCHEMA_1_4
-    try:
-        jsonschema.validate(dict(req.GET), schema,
-                            format_checker=jsonschema.FormatChecker())
-    except jsonschema.ValidationError as exc:
-        raise webob.exc.HTTPBadRequest(
-            _('Invalid query string parameters: %(exc)s') %
-            {'exc': exc},
-            json_formatter=util.json_error_formatter)
+
+    util.validate_query_params(req, schema)
 
     filters = {}
     for attr in ['uuid', 'name', 'member_of']:
@@ -294,11 +229,10 @@ def list_resource_providers(req):
                     if not uuidutils.is_uuid_like(aggr_uuid):
                         raise webob.exc.HTTPBadRequest(
                             _('Invalid uuid value: %(uuid)s') %
-                            {'uuid': aggr_uuid},
-                            json_formatter=util.json_error_formatter)
+                            {'uuid': aggr_uuid})
             filters[attr] = value
     if 'resources' in req.GET:
-        resources = _normalize_resources_qs_param(req.GET['resources'])
+        resources = util.normalize_resources_qs_param(req.GET['resources'])
         filters['resources'] = resources
     try:
         resource_providers = objects.ResourceProviderList.get_all_by_filters(
@@ -306,8 +240,7 @@ def list_resource_providers(req):
     except exception.ResourceClassNotFound as exc:
         raise webob.exc.HTTPBadRequest(
             _('Invalid resource class in resources parameter: %(error)s') %
-            {'error': exc},
-            json_formatter=util.json_error_formatter)
+            {'error': exc})
 
     response = req.response
     response.body = encodeutils.to_utf8(
@@ -316,7 +249,7 @@ def list_resource_providers(req):
     return response
 
 
-@webob.dec.wsgify
+@wsgi_wrapper.PlacementWsgify
 @util.require_content('application/json')
 def update_resource_provider(req):
     """PUT to update a single resource provider.
@@ -339,14 +272,12 @@ def update_resource_provider(req):
         resource_provider.save()
     except db_exc.DBDuplicateEntry as exc:
         raise webob.exc.HTTPConflict(
-            _('Conflicting resource provider already exists: %(error)s') %
-            {'error': exc},
-            json_formatter=util.json_error_formatter)
+            _('Conflicting resource provider %(name)s already exists.') %
+            {'name': data['name']})
     except exception.ObjectActionError as exc:
         raise webob.exc.HTTPBadRequest(
             _('Unable to save resource provider %(rp_uuid)s: %(error)s') %
-            {'rp_uuid': uuid, 'error': exc},
-            json_formatter=util.json_error_formatter)
+            {'rp_uuid': uuid, 'error': exc})
 
     req.response.body = encodeutils.to_utf8(jsonutils.dumps(
         _serialize_provider(req.environ, resource_provider)))
