@@ -15,12 +15,36 @@ from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 import webob
 
+from nova.api.openstack.placement import microversion
 from nova.api.openstack.placement import util
+from nova.api.openstack.placement import wsgi_wrapper
 from nova import exception
 from nova.i18n import _
 from nova import objects
 
 
+# Represents the allowed query string parameters to GET /usages
+GET_USAGES_SCHEMA_1_9 = {
+    "type": "object",
+    "properties": {
+        "project_id": {
+            "type": "string",
+            "minLength": 1,
+            "maxLength": 255,
+        },
+        "user_id": {
+            "type": "string",
+            "minLength": 1,
+            "maxLength": 255,
+        },
+    },
+    "required": [
+        "project_id"
+     ],
+    "additionalProperties": False,
+}
+
+
 def _serialize_usages(resource_provider, usage):
     usage_dict = {resource.resource_class: resource.usage
                   for resource in usage}
@@ -28,7 +52,7 @@ def _serialize_usages(resource_provider, usage):
             'usages': usage_dict}
 
 
-@webob.dec.wsgify
+@wsgi_wrapper.PlacementWsgify
 @util.check_accept('application/json')
 def list_usages(req):
     """GET a dictionary of resource provider usage by resource class.
@@ -52,8 +76,7 @@ def list_usages(req):
     except exception.NotFound as exc:
         raise webob.exc.HTTPNotFound(
             _("No resource provider with uuid %(uuid)s found: %(error)s") %
-             {'uuid': uuid, 'error': exc},
-             json_formatter=util.json_error_formatter)
+             {'uuid': uuid, 'error': exc})
 
     usage = objects.UsageList.get_all_by_resource_provider_uuid(
         context, uuid)
@@ -63,3 +86,33 @@ def list_usages(req):
         _serialize_usages(resource_provider, usage)))
     req.response.content_type = 'application/json'
     return req.response
+
+
+@wsgi_wrapper.PlacementWsgify
+@microversion.version_handler('1.9')
+@util.check_accept('application/json')
+def get_total_usages(req):
+    """GET the sum of usages for a project or a project/user.
+
+    On success return a 200 and an application/json body representing the
+    sum/total of usages.
+    Return 404 Not Found if the wanted microversion does not match.
+    """
+    context = req.environ['placement.context']
+
+    schema = GET_USAGES_SCHEMA_1_9
+
+    util.validate_query_params(req, schema)
+
+    project_id = req.GET.get('project_id')
+    user_id = req.GET.get('user_id')
+
+    usages = objects.UsageList.get_all_by_project_user(context, project_id,
+                                                       user_id=user_id)
+
+    response = req.response
+    usages_dict = {'usages': {resource.resource_class: resource.usage
+                   for resource in usages}}
+    response.body = encodeutils.to_utf8(jsonutils.dumps(usages_dict))
+    req.response.content_type = 'application/json'
+    return req.response
