@@ -12,6 +12,7 @@
 """Utility methods for placement API."""
 
 import functools
+
 import jsonschema
 from oslo_middleware import request_id
 from oslo_serialization import jsonutils
@@ -122,11 +123,15 @@ def require_content(content_type):
             if req.content_type != content_type:
                 # webob's unset content_type is the empty string so
                 # set it the error message content to 'None' to make
-                # a useful message in that case.
+                # a useful message in that case. This also avoids a
+                # KeyError raised when webob.exc eagerly fills in a
+                # Template for output we will never use.
+                if not req.content_type:
+                    req.content_type = 'None'
                 raise webob.exc.HTTPUnsupportedMediaType(
                     _('The media type %(bad_type)s is not supported, '
                       'use %(good_type)s') %
-                    {'bad_type': req.content_type or 'None',
+                    {'bad_type': req.content_type,
                      'good_type': content_type},
                     json_formatter=json_error_formatter)
             else:
@@ -155,6 +160,26 @@ def resource_provider_url(environ, resource_provider):
     return '%s/resource_providers/%s' % (prefix, resource_provider.uuid)
 
 
+def trait_url(environ, trait):
+    """Produce the URL for a trait.
+
+    If SCRIPT_NAME is present, it is the mount point of the placement
+    WSGI app.
+    """
+    prefix = environ.get('SCRIPT_NAME', '')
+    return '%s/traits/%s' % (prefix, trait.name)
+
+
+def validate_query_params(req, schema):
+    try:
+        jsonschema.validate(dict(req.GET), schema,
+                            format_checker=jsonschema.FormatChecker())
+    except jsonschema.ValidationError as exc:
+        raise webob.exc.HTTPBadRequest(
+            _('Invalid query string parameters: %(exc)s') %
+            {'exc': exc})
+
+
 def wsgi_path_item(environ, name):
     """Extract the value of a named field in a URL.
 
@@ -167,3 +192,68 @@ def wsgi_path_item(environ, name):
         return environ['wsgiorg.routing_args'][1][name]
     except (KeyError, IndexError):
         return None
+
+
+def normalize_resources_qs_param(qs):
+    """Given a query string parameter for resources, validate it meets the
+    expected format and return a dict of amounts, keyed by resource class name.
+
+    The expected format of the resources parameter looks like so:
+
+        $RESOURCE_CLASS_NAME:$AMOUNT,$RESOURCE_CLASS_NAME:$AMOUNT
+
+    So, if the user was looking for resource providers that had room for an
+    instance that will consume 2 vCPUs, 1024 MB of RAM and 50GB of disk space,
+    they would use the following query string:
+
+        ?resources=VCPU:2,MEMORY_MB:1024,DISK_GB:50
+
+    The returned value would be:
+
+        {
+            "VCPU": 2,
+            "MEMORY_MB": 1024,
+            "DISK_GB": 50,
+        }
+
+    :param qs: The value of the 'resources' query string parameter
+    :raises `webob.exc.HTTPBadRequest` if the parameter's value isn't in the
+            expected format.
+    """
+    if qs.strip() == "":
+        msg = _('Badly formed resources parameter. Expected resources '
+                'query string parameter in form: '
+                '?resources=VCPU:2,MEMORY_MB:1024. Got: empty string.')
+        raise webob.exc.HTTPBadRequest(msg)
+
+    result = {}
+    resource_tuples = qs.split(',')
+    for rt in resource_tuples:
+        try:
+            rc_name, amount = rt.split(':')
+        except ValueError:
+            msg = _('Badly formed resources parameter. Expected resources '
+                    'query string parameter in form: '
+                    '?resources=VCPU:2,MEMORY_MB:1024. Got: %s.')
+            msg = msg % rt
+            raise webob.exc.HTTPBadRequest(msg)
+        try:
+            amount = int(amount)
+        except ValueError:
+            msg = _('Requested resource %(resource_name)s expected positive '
+                    'integer amount. Got: %(amount)s.')
+            msg = msg % {
+                'resource_name': rc_name,
+                'amount': amount,
+            }
+            raise webob.exc.HTTPBadRequest(msg)
+        if amount < 1:
+            msg = _('Requested resource %(resource_name)s requires '
+                    'amount >= 1. Got: %(amount)d.')
+            msg = msg % {
+                'resource_name': rc_name,
+                'amount': amount,
+            }
+            raise webob.exc.HTTPBadRequest(msg)
+        result[rc_name] = amount
+    return result
