@@ -52,7 +52,7 @@ from nova.consoleauth import rpcapi as consoleauth_rpcapi
 from nova import context
 from nova.db import base
 from nova import exception
-from nova.i18n import _, _LE, _LI, _LW
+from nova.i18n import _
 from nova import objects
 from nova.objects import base as objects_base
 from nova import rpc
@@ -190,7 +190,7 @@ class _BaseMessage(object):
         except Exception:
             resp_value = sys.exc_info()
             failure = True
-            LOG.exception(_LE("Error processing message locally"))
+            LOG.exception("Error processing message locally")
         return Response(self.ctxt, self.routing_path, resp_value, failure)
 
     def _setup_response_queue(self):
@@ -399,7 +399,7 @@ class _TargetedMessage(_BaseMessage):
             next_hop = self._get_next_hop()
         except Exception:
             exc_info = sys.exc_info()
-            LOG.exception(_LE("Error locating next hop for message"))
+            LOG.exception("Error locating next hop for message")
             return self._send_response_from_exception(exc_info)
 
         if next_hop.is_me:
@@ -425,7 +425,7 @@ class _TargetedMessage(_BaseMessage):
             next_hop.send_message(self)
         except Exception:
             exc_info = sys.exc_info()
-            err_str = _LE("Failed to send message to cell: %(next_hop)s")
+            err_str = "Failed to send message to cell: %(next_hop)s"
             LOG.exception(err_str, {'next_hop': next_hop})
             self._cleanup_response_queue()
             return self._send_response_from_exception(exc_info)
@@ -503,7 +503,7 @@ class _BroadcastMessage(_BaseMessage):
             next_hops = self._get_next_hops()
         except Exception:
             exc_info = sys.exc_info()
-            LOG.exception(_LE("Error locating next hops for message"))
+            LOG.exception("Error locating next hops for message")
             return self._send_response_from_exception(exc_info)
 
         # Short circuit if we don't need to respond
@@ -522,7 +522,7 @@ class _BroadcastMessage(_BaseMessage):
             # Error just trying to send to cells.  Send a single response
             # with the failure.
             exc_info = sys.exc_info()
-            LOG.exception(_LE("Error sending message to next hops."))
+            LOG.exception("Error sending message to next hops.")
             self._cleanup_response_queue()
             return self._send_response_from_exception(exc_info)
 
@@ -539,8 +539,7 @@ class _BroadcastMessage(_BaseMessage):
             # Error waiting for responses, most likely a timeout.
             # Send a single response back with the failure.
             exc_info = sys.exc_info()
-            LOG.exception(_LE("Error waiting for responses from"
-                              " neighbor cells"))
+            LOG.exception("Error waiting for responses from neighbor cells")
             return self._send_response_from_exception(exc_info)
 
         if local_response:
@@ -748,7 +747,9 @@ class _TargetedMessageMethods(_BaseMessageMethods):
             cctxt.cast(message.ctxt, method, **kwargs)
 
     def compute_node_get(self, message, compute_id):
-        """Get compute node by ID."""
+        """Get compute node by ID or UUID."""
+        if uuidutils.is_uuid_like(compute_id):
+            return objects.ComputeNode.get_by_uuid(message.ctxt, compute_id)
         return objects.ComputeNode.get_by_id(message.ctxt, compute_id)
 
     def actions_get(self, message, instance_uuid):
@@ -1100,8 +1101,8 @@ class _BroadcastMessageMethods(_BaseMessageMethods):
                        **kwargs):
         projid_str = project_id is None and "<all>" or project_id
         since_str = updated_since is None and "<all>" or updated_since
-        LOG.info(_LI("Forcing a sync of instances, project_id="
-                     "%(projid_str)s, updated_since=%(since_str)s"),
+        LOG.info("Forcing a sync of instances, project_id="
+                 "%(projid_str)s, updated_since=%(since_str)s",
                  {'projid_str': projid_str, 'since_str': since_str})
         if updated_since is not None:
             updated_since = timeutils.parse_isotime(updated_since)
@@ -1180,7 +1181,7 @@ class _BroadcastMessageMethods(_BaseMessageMethods):
             if vol_id and instance_bdm['volume_id'] == vol_id:
                 break
         else:
-            LOG.warning(_LW("No match when trying to update BDM: %(bdm)s"),
+            LOG.warning("No match when trying to update BDM: %(bdm)s",
                         dict(bdm=bdm))
             return
         LOG.debug('Calling db.block_device_mapping_update from API cell with '
@@ -1588,7 +1589,7 @@ class MessageRunner(object):
         return message.process()
 
     def compute_node_get(self, ctxt, cell_name, compute_id):
-        """Return compute node entry from a specific cell by ID."""
+        """Return compute node entry from a specific cell by ID or UUID."""
         method_kwargs = dict(compute_id=compute_id)
         message = _TargetedMessage(self, ctxt, 'compute_node_get',
                                     method_kwargs, 'down',
@@ -1674,7 +1675,7 @@ class MessageRunner(object):
         """Call instance_<method> in correct cell for instance."""
         cell_name = instance.cell_name
         if not cell_name:
-            LOG.warning(_LW("No cell_name for %(method)s() from API"),
+            LOG.warning("No cell_name for %(method)s() from API",
                         dict(method=method), instance=instance)
             return
         method_kwargs = {'instance': instance}
@@ -1691,7 +1692,7 @@ class MessageRunner(object):
         """Update an instance object in its cell."""
         cell_name = instance.cell_name
         if not cell_name:
-            LOG.warning(_LW("No cell_name for instance update from API"),
+            LOG.warning("No cell_name for instance update from API",
                         instance=instance)
             return
         method_kwargs = {'instance': instance,
@@ -1879,7 +1880,7 @@ def serialize_remote_exception(failure_info, log_failure=True):
     tb = traceback.format_exception(*failure_info)
     failure = failure_info[1]
     if log_failure:
-        LOG.error(_LE("Returning exception %s to caller"),
+        LOG.error("Returning exception %s to caller",
                   six.text_type(failure))
         LOG.error(tb)
 
