@@ -19,48 +19,27 @@
 """Built-in instance properties."""
 
 import re
-import uuid
 
-from oslo_config import cfg
-from oslo_log import log as logging
 from oslo_utils import strutils
+from oslo_utils import uuidutils
 import six
 
 from nova.api.validation import parameter_types
+import nova.conf
 from nova import context
 from nova import db
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LE
 from nova import objects
 from nova import utils
 
-flavor_opts = [
-    cfg.StrOpt('default_flavor',
-               default='m1.small',
-               help='Default flavor to use for the EC2 API only. The Nova API '
-               'does not support a default flavor.'),
-]
-
-CONF = cfg.CONF
-CONF.register_opts(flavor_opts)
-
-LOG = logging.getLogger(__name__)
+CONF = nova.conf.CONF
 
 # NOTE(luisg): Flavor names can include non-ascii characters so that users can
 # create flavor names in locales that use them, however flavor IDs are limited
 # to ascii characters.
 VALID_ID_REGEX = re.compile("^[\w\.\- ]*$")
 
-# NOTE(dosaboy): This is supposed to represent the maximum value that we can
-# place into a SQL single precision float so that we can check whether values
-# are oversize. Postgres and MySQL both define this as their max whereas Sqlite
-# uses dynamic typing so this would not apply. Different dbs react in different
-# ways to oversize values e.g. postgres will raise an exception while mysql
-# will round off the value. Nevertheless we may still want to know prior to
-# insert whether the value is oversize.
-SQL_SP_FLOAT_MAX = 3.40282e+38
-
 # Validate extra specs key names.
 VALID_EXTRASPEC_NAME_REGEX = re.compile(r"[\w\.\- :]+$", re.UNICODE)
 
@@ -93,7 +72,7 @@ def create(name, memory, vcpus, root_gb, ephemeral_gb=0, flavorid=None,
            swap=0, rxtx_factor=1.0, is_public=True):
     """Creates flavors."""
     if not flavorid:
-        flavorid = uuid.uuid4()
+        flavorid = uuidutils.generate_uuid()
 
     kwargs = {
         'memory_mb': memory,
@@ -155,11 +134,11 @@ def create(name, memory, vcpus, root_gb, ephemeral_gb=0, flavorid=None,
     try:
         kwargs['rxtx_factor'] = float(kwargs['rxtx_factor'])
         if (kwargs['rxtx_factor'] <= 0 or
-                kwargs['rxtx_factor'] > SQL_SP_FLOAT_MAX):
+                kwargs['rxtx_factor'] > db.SQL_SP_FLOAT_MAX):
             raise ValueError()
     except ValueError:
         msg = (_("'rxtx_factor' argument must be a float between 0 and %g") %
-               SQL_SP_FLOAT_MAX)
+               db.SQL_SP_FLOAT_MAX)
         raise exception.InvalidInput(reason=msg)
 
     kwargs['name'] = name
@@ -176,30 +155,6 @@ def create(name, memory, vcpus, root_gb, ephemeral_gb=0, flavorid=None,
     return flavor
 
 
-def destroy(name):
-    """Marks flavor as deleted."""
-    try:
-        if not name:
-            raise ValueError()
-        flavor = objects.Flavor(context=context.get_admin_context(), name=name)
-        flavor.destroy()
-    except (ValueError, exception.NotFound):
-        LOG.exception(_LE('Instance type %s not found for deletion'), name)
-        raise exception.FlavorNotFoundByName(flavor_name=name)
-
-
-def get_all_flavors_sorted_list(ctxt=None, filters=None, sort_key='flavorid',
-                                sort_dir='asc', limit=None, marker=None):
-    """Get all non-deleted flavors as a sorted list.
-    """
-    if ctxt is None:
-        ctxt = context.get_admin_context()
-
-    return objects.FlavorList.get_all(ctxt, filters=filters, sort_key=sort_key,
-                                      sort_dir=sort_dir, limit=limit,
-                                      marker=marker)
-
-
 def get_default_flavor():
     """Get the default flavor."""
     name = CONF.default_flavor
