@@ -18,6 +18,7 @@ import contextlib
 import functools
 import inspect
 import itertools
+import math
 import string
 import traceback
 
@@ -30,14 +31,12 @@ from nova.compute import power_state
 from nova.compute import task_states
 import nova.conf
 from nova import exception
-from nova.i18n import _LW
-from nova.network import model as network_model
 from nova import notifications
 from nova.notifications.objects import aggregate as aggregate_notification
 from nova.notifications.objects import base as notification_base
 from nova.notifications.objects import exception as notification_exception
-from nova.notifications.objects import flavor as flavor_notification
 from nova.notifications.objects import instance as instance_notification
+from nova.notifications.objects import keypair as keypair_notification
 from nova import objects
 from nova.objects import fields
 from nova import rpc
@@ -220,6 +219,15 @@ def is_volume_backed_instance(context, instance, bdms=None):
     return not instance['image_ref']
 
 
+def convert_mb_to_ceil_gb(mb_value):
+    gb_int = 0
+    if mb_value:
+        gb_float = mb_value / 1024.0
+        # ensure we reserve/allocate enough space by rounding up to nearest GB
+        gb_int = int(math.ceil(gb_float))
+    return gb_int
+
+
 def _get_unused_letter(used_letters):
     doubles = [first + second for second in string.ascii_lowercase
                for first in string.ascii_lowercase]
@@ -242,8 +250,8 @@ def get_value_from_system_metadata(instance, key, type, default):
     try:
         return type(value)
     except ValueError:
-        LOG.warning(_LW("Metadata value %(value)s for %(key)s is not of "
-                        "type %(type)s. Using default value %(default)s."),
+        LOG.warning("Metadata value %(value)s for %(key)s is not of "
+                    "type %(type)s. Using default value %(default)s.",
                     {'value': value, 'key': key, 'type': type,
                      'default': default}, instance=instance)
         return default
@@ -252,8 +260,8 @@ def get_value_from_system_metadata(instance, key, type, default):
 def notify_usage_exists(notifier, context, instance_ref, current_period=False,
                         ignore_missing_network_data=True,
                         system_metadata=None, extra_usage_info=None):
-    """Generates 'exists' notification for an instance for usage auditing
-    purposes.
+    """Generates 'exists' unversioned legacy notification for an instance for
+    usage auditing purposes.
 
     :param notifier: a messaging.Notifier
 
@@ -295,7 +303,10 @@ def notify_usage_exists(notifier, context, instance_ref, current_period=False,
 def notify_about_instance_usage(notifier, context, instance, event_suffix,
                                 network_info=None, system_metadata=None,
                                 extra_usage_info=None, fault=None):
-    """Send a notification about an instance.
+    """Send an unversioned legacy notification about an instance.
+
+    All new notifications should use notify_about_instance_action which sends
+    a versioned notification.
 
     :param notifier: a messaging.Notifier
     :param event_suffix: Event type like "delete.start" or "exists"
@@ -325,23 +336,6 @@ def notify_about_instance_usage(notifier, context, instance, event_suffix,
     method(context, 'compute.instance.%s' % event_suffix, usage_info)
 
 
-def _get_instance_ips(instance):
-    network_info = get_nw_info_for_instance(instance)
-    ips = []
-    if network_info is not None:
-        for vif in network_info:
-            for ip in vif.fixed_ips():
-                ips.append(instance_notification.IpPayload(
-                    label=vif["network"]["label"],
-                    mac=vif["address"],
-                    meta=vif["meta"],
-                    port_uuid=vif["id"],
-                    version=ip["version"],
-                    address=ip["address"],
-                    device_name=vif["devname"]))
-    return ips
-
-
 def _get_fault_and_priority_from_exc(exception):
     fault = None
     priority = fields.NotificationPriority.INFO
@@ -354,6 +348,7 @@ def _get_fault_and_priority_from_exc(exception):
     return fault, priority
 
 
+@rpc.if_notifications_enabled
 def notify_about_instance_action(context, instance, host, action, phase=None,
                                  binary='nova-compute', exception=None):
     """Send versioned notification about the action made on the instance
@@ -364,20 +359,15 @@ def notify_about_instance_action(context, instance, host, action, phase=None,
     :param binary: the binary emitting the notification
     :param exception: the thrown exception (used in error notifications)
     """
-    ips = _get_instance_ips(instance)
-
-    flavor = flavor_notification.FlavorPayload(instance.flavor)
     fault, priority = _get_fault_and_priority_from_exc(exception)
     payload = instance_notification.InstanceActionPayload(
             instance=instance,
-            fault=fault,
-            ip_addresses=ips,
-            flavor=flavor)
+            fault=fault)
     notification = instance_notification.InstanceActionNotification(
             context=context,
             priority=priority,
             publisher=notification_base.NotificationPublisher(
-                    context=context, host=host, binary=binary),
+                host=host, binary=binary),
             event_type=notification_base.EventType(
                     object='instance',
                     action=action,
@@ -386,6 +376,89 @@ def notify_about_instance_action(context, instance, host, action, phase=None,
     notification.emit(context)
 
 
+@rpc.if_notifications_enabled
+def notify_about_instance_create(context, instance, host, phase=None,
+                                 binary='nova-compute', exception=None):
+    """Send versioned notification about instance creation
+
+    :param context: the request context
+    :param instance: the instance being created
+    :param host: the host emitting the notification
+    :param phase: the phase of the creation
+    :param binary: the binary emitting the notification
+    :param exception: the thrown exception (used in error notifications)
+    """
+    fault, priority = _get_fault_and_priority_from_exc(exception)
+    payload = instance_notification.InstanceCreatePayload(
+        instance=instance,
+        fault=fault)
+    notification = instance_notification.InstanceCreateNotification(
+        context=context,
+        priority=priority,
+        publisher=notification_base.NotificationPublisher(
+            host=host, binary=binary),
+        event_type=notification_base.EventType(
+            object='instance',
+            action=fields.NotificationAction.CREATE,
+            phase=phase),
+        payload=payload)
+    notification.emit(context)
+
+
+@rpc.if_notifications_enabled
+def notify_about_volume_attach_detach(context, instance, host, action, phase,
+                                      binary='nova-compute', volume_id=None,
+                                      exception=None):
+    """Send versioned notification about the action made on the instance
+    :param instance: the instance which the action performed on
+    :param host: the host emitting the notification
+    :param action: the name of the action
+    :param phase: the phase of the action
+    :param binary: the binary emitting the notification
+    :param volume_id: id of the volume will be attached
+    :param exception: the thrown exception (used in error notifications)
+    """
+    fault, priority = _get_fault_and_priority_from_exc(exception)
+    payload = instance_notification.InstanceActionVolumePayload(
+            instance=instance,
+            fault=fault,
+            volume_id=volume_id)
+    notification = instance_notification.InstanceActionVolumeNotification(
+            context=context,
+            priority=priority,
+            publisher=notification_base.NotificationPublisher(
+                    host=host, binary=binary),
+            event_type=notification_base.EventType(
+                    object='instance',
+                    action=action,
+                    phase=phase),
+            payload=payload)
+    notification.emit(context)
+
+
+@rpc.if_notifications_enabled
+def notify_about_keypair_action(context, keypair, action, phase):
+    """Send versioned notification about the keypair action on the instance
+
+    :param context: the request context
+    :param keypair: the keypair which the action performed on
+    :param action: the name of the action
+    :param phase: the phase of the action
+    """
+    payload = keypair_notification.KeypairPayload(keypair=keypair)
+    notification = keypair_notification.KeypairNotification(
+        priority=fields.NotificationPriority.INFO,
+        publisher=notification_base.NotificationPublisher(
+            host=CONF.host, binary='nova-api'),
+        event_type=notification_base.EventType(
+            object='keypair',
+            action=action,
+            phase=phase),
+        payload=payload)
+    notification.emit(context)
+
+
+@rpc.if_notifications_enabled
 def notify_about_volume_swap(context, instance, host, action, phase,
                              old_volume_id, new_volume_id, exception=None):
     """Send versioned notification about the volume swap action
@@ -400,16 +473,10 @@ def notify_about_volume_swap(context, instance, host, action, phase,
     :param new_volume_id: the ID of the volume that is copied to and attached
     :param exception: an exception
     """
-    ips = _get_instance_ips(instance)
-
-    flavor = flavor_notification.FlavorPayload(instance.flavor)
-
     fault, priority = _get_fault_and_priority_from_exc(exception)
     payload = instance_notification.InstanceActionVolumeSwapPayload(
         instance=instance,
         fault=fault,
-        ip_addresses=ips,
-        flavor=flavor,
         old_volume_id=old_volume_id,
         new_volume_id=new_volume_id)
 
@@ -417,7 +484,7 @@ def notify_about_volume_swap(context, instance, host, action, phase,
         context=context,
         priority=priority,
         publisher=notification_base.NotificationPublisher(
-            context=context, host=host, binary='nova-compute'),
+            host=host, binary='nova-compute'),
         event_type=notification_base.EventType(
             object='instance', action=action, phase=phase),
         payload=payload).emit(context)
@@ -454,12 +521,13 @@ def notify_about_aggregate_update(context, event_suffix, aggregate_payload):
     notifier.info(context, 'aggregate.%s' % event_suffix, aggregate_payload)
 
 
+@rpc.if_notifications_enabled
 def notify_about_aggregate_action(context, aggregate, action, phase):
     payload = aggregate_notification.AggregatePayload(aggregate)
     notification = aggregate_notification.AggregateNotification(
         priority=fields.NotificationPriority.INFO,
         publisher=notification_base.NotificationPublisher(
-            context=context, host=CONF.host, binary='nova-api'),
+            host=CONF.host, binary='nova-api'),
         event_type=notification_base.EventType(
             object='aggregate',
             action=action,
@@ -478,8 +546,8 @@ def notify_about_host_update(context, event_suffix, host_payload):
     """
     host_identifier = host_payload.get('host_name')
     if not host_identifier:
-        LOG.warning(_LW("No host name specified for the notification of "
-                        "HostAPI.%s and it will be ignored"), event_suffix)
+        LOG.warning("No host name specified for the notification of "
+                    "HostAPI.%s and it will be ignored", event_suffix)
         return
 
     notifier = rpc.get_notifier(service='api', host=host_identifier)
@@ -487,12 +555,6 @@ def notify_about_host_update(context, event_suffix, host_payload):
     notifier.info(context, 'HostAPI.%s' % event_suffix, host_payload)
 
 
-def get_nw_info_for_instance(instance):
-    if instance.info_cache is None:
-        return network_model.NetworkInfo.hydrate([])
-    return instance.info_cache.network_info
-
-
 def refresh_info_cache_for_instance(context, instance):
     """Refresh the info cache for an instance.
 
@@ -635,6 +697,132 @@ def reserve_quota_delta(context, deltas, instance):
     return quotas
 
 
+def get_headroom(quotas, usages, deltas):
+    headroom = {res: quotas[res] - usages[res]
+                for res in quotas.keys()}
+    # If quota_cores is unlimited [-1]:
+    # - set cores headroom based on instances headroom:
+    if quotas.get('cores') == -1:
+        if deltas.get('cores'):
+            hc = headroom.get('instances', 1) * deltas['cores']
+            headroom['cores'] = hc / deltas.get('instances', 1)
+        else:
+            headroom['cores'] = headroom.get('instances', 1)
+
+    # If quota_ram is unlimited [-1]:
+    # - set ram headroom based on instances headroom:
+    if quotas.get('ram') == -1:
+        if deltas.get('ram'):
+            hr = headroom.get('instances', 1) * deltas['ram']
+            headroom['ram'] = hr / deltas.get('instances', 1)
+        else:
+            headroom['ram'] = headroom.get('instances', 1)
+
+    return headroom
+
+
+def check_num_instances_quota(context, instance_type, min_count,
+                              max_count, project_id=None, user_id=None,
+                              orig_num_req=None):
+    """Enforce quota limits on number of instances created."""
+    # project_id is used for the TooManyInstances error message
+    if project_id is None:
+        project_id = context.project_id
+    # Determine requested cores and ram
+    req_cores = max_count * instance_type.vcpus
+    req_ram = max_count * instance_type.memory_mb
+    deltas = {'instances': max_count, 'cores': req_cores, 'ram': req_ram}
+
+    try:
+        objects.Quotas.check_deltas(context, deltas,
+                                    project_id, user_id=user_id,
+                                    check_project_id=project_id,
+                                    check_user_id=user_id)
+    except exception.OverQuota as exc:
+        quotas = exc.kwargs['quotas']
+        overs = exc.kwargs['overs']
+        usages = exc.kwargs['usages']
+        # This is for the recheck quota case where we used a delta of zero.
+        if min_count == max_count == 0:
+            # orig_num_req is the original number of instances requested in the
+            # case of a recheck quota, for use in the over quota exception.
+            req_cores = orig_num_req * instance_type.vcpus
+            req_ram = orig_num_req * instance_type.memory_mb
+            requested = {'instances': orig_num_req, 'cores': req_cores,
+                         'ram': req_ram}
+            (overs, reqs, total_alloweds, useds) = get_over_quota_detail(
+                deltas, overs, quotas, requested)
+            msg = "Cannot run any more instances of this type."
+            params = {'overs': overs, 'pid': project_id, 'msg': msg}
+            LOG.debug("%(overs)s quota exceeded for %(pid)s. %(msg)s",
+                      params)
+            raise exception.TooManyInstances(overs=overs,
+                                             req=reqs,
+                                             used=useds,
+                                             allowed=total_alloweds)
+        # OK, we exceeded quota; let's figure out why...
+        headroom = get_headroom(quotas, usages, deltas)
+
+        allowed = headroom.get('instances', 1)
+        # Reduce 'allowed' instances in line with the cores & ram headroom
+        if instance_type.vcpus:
+            allowed = min(allowed,
+                          headroom['cores'] // instance_type.vcpus)
+        if instance_type.memory_mb:
+            allowed = min(allowed,
+                          headroom['ram'] // instance_type.memory_mb)
+
+        # Convert to the appropriate exception message
+        if allowed <= 0:
+            msg = "Cannot run any more instances of this type."
+        elif min_count <= allowed <= max_count:
+            # We're actually OK, but still need to check against allowed
+            return check_num_instances_quota(context, instance_type, min_count,
+                                             allowed, project_id=project_id,
+                                             user_id=user_id)
+        else:
+            msg = "Can only run %s more instances of this type." % allowed
+
+        num_instances = (str(min_count) if min_count == max_count else
+            "%s-%s" % (min_count, max_count))
+        requested = dict(instances=num_instances, cores=req_cores,
+                         ram=req_ram)
+        (overs, reqs, total_alloweds, useds) = get_over_quota_detail(
+            headroom, overs, quotas, requested)
+        params = {'overs': overs, 'pid': project_id,
+                  'min_count': min_count, 'max_count': max_count,
+                  'msg': msg}
+
+        if min_count == max_count:
+            LOG.debug("%(overs)s quota exceeded for %(pid)s,"
+                      " tried to run %(min_count)d instances. "
+                      "%(msg)s", params)
+        else:
+            LOG.debug("%(overs)s quota exceeded for %(pid)s,"
+                      " tried to run between %(min_count)d and"
+                      " %(max_count)d instances. %(msg)s",
+                      params)
+        raise exception.TooManyInstances(overs=overs,
+                                         req=reqs,
+                                         used=useds,
+                                         allowed=total_alloweds)
+
+    return max_count
+
+
+def get_over_quota_detail(headroom, overs, quotas, requested):
+    reqs = []
+    useds = []
+    total_alloweds = []
+    for resource in overs:
+        reqs.append(str(requested[resource]))
+        useds.append(str(quotas[resource] - headroom[resource]))
+        total_alloweds.append(str(quotas[resource]))
+    (overs, reqs, useds, total_alloweds) = map(', '.join, (
+        overs, reqs, useds, total_alloweds))
+    return overs, reqs, total_alloweds, useds
+
+
 def remove_shelved_keys_from_system_metadata(instance):
     # Delete system_metadata for a shelved instance
     for key in ['shelved_at', 'shelved_image_id', 'shelved_host']:
