@@ -190,8 +190,8 @@ class ConductorAPI(object):
 
     * 3.0  - Drop backwards compatibility
 
-    ... Liberty, Mitaka, and Newton support message version 3.0.  So, any
-    changes to existing methods in 3.x after that point should be done such
+    ... Liberty, Mitaka, Newton, and Ocata support message version 3.0.  So,
+    any changes to existing methods in 3.x after that point should be done such
     that they can handle the version_cap being set to 3.0.
 
     * Remove provider_fw_rule_get_all()
@@ -206,6 +206,7 @@ class ConductorAPI(object):
         'liberty': '3.0',
         'mitaka': '3.0',
         'newton': '3.0',
+        'ocata': '3.0',
     }
 
     def __init__(self):
@@ -272,6 +273,7 @@ class ComputeTaskAPI(object):
     1.14 - Added request_spec to unshelve_instance()
     1.15 - Added live_migrate_instance
     1.16 - Added schedule_and_build_instances
+    1.17 - Added tags to schedule_and_build_instances()
     """
 
     def __init__(self):
@@ -293,6 +295,8 @@ class ComputeTaskAPI(object):
         cctxt = self.client.prepare(version=version)
         cctxt.cast(context, 'live_migrate_instance', **kw)
 
+    # TODO(melwitt): Remove the reservations parameter in version 2.0 of the
+    # RPC API.
     def migrate_server(self, context, instance, scheduler_hint, live, rebuild,
                   flavor, block_migration, disk_over_commit,
                   reservations=None, clean_shutdown=True, request_spec=None):
@@ -352,18 +356,24 @@ class ComputeTaskAPI(object):
         cctxt.cast(context, 'build_instances', **kw)
 
     def schedule_and_build_instances(self, context, build_requests,
-                                      request_specs,
-                                      image, admin_password, injected_files,
-                                      requested_networks,
-                                      block_device_mapping):
-        version = '1.16'
+                                     request_specs,
+                                     image, admin_password, injected_files,
+                                     requested_networks,
+                                     block_device_mapping,
+                                     tags=None):
+        version = '1.17'
         kw = {'build_requests': build_requests,
               'request_specs': request_specs,
               'image': jsonutils.to_primitive(image),
               'admin_password': admin_password,
               'injected_files': injected_files,
               'requested_networks': requested_networks,
-              'block_device_mapping': block_device_mapping}
+              'block_device_mapping': block_device_mapping,
+              'tags': tags}
+
+        if not self.client.can_send_version(version):
+            version = '1.16'
+            del kw['tags']
 
         cctxt = self.client.prepare(version=version)
         cctxt.cast(context, 'schedule_and_build_instances', **kw)
