@@ -71,7 +71,11 @@ class LiveMigrationTask(base.TaskBase):
             # skip_filters=True flag so the scheduler does the work of claiming
             # resources on the destination in Placement but still bypass the
             # scheduler filters, which honors the 'force' flag in the API.
-            self._claim_resources_on_destination(source_node, dest_node)
+            # This raises NoValidHost which will be handled in
+            # ComputeTaskManager.
+            scheduler_utils.claim_resources_on_destination(
+                self.scheduler_client.reportclient, self.instance,
+                source_node, dest_node)
 
         # TODO(johngarbutt) need to move complexity out of compute manager
         # TODO(johngarbutt) disk_over_commit?
@@ -129,72 +133,6 @@ class LiveMigrationTask(base.TaskBase):
                           'across cells.') % self.instance.uuid))
         return source_node, dest_node
 
-    def _claim_resources_on_destination(self, source_node, dest_node):
-        """Copies allocations from source node to dest node in Placement
-
-        :param source_node: source ComputeNode where the instance currently
-                            lives
-        :param dest_node: destination ComputeNode where the instance is being
-                          forced to live migrate.
-        """
-        reportclient = self.scheduler_client.reportclient
-        # Get the current allocations for the source node and the instance.
-        source_node_allocations = reportclient.get_allocations_for_instance(
-            source_node.uuid, self.instance)
-        if source_node_allocations:
-            # Generate an allocation request for the destination node.
-            alloc_request = {
-                'allocations': [
-                    {
-                        'resource_provider': {
-                            'uuid': dest_node.uuid
-                        },
-                        'resources': source_node_allocations
-                    }
-                ]
-            }
-            # The claim_resources method will check for existing allocations
-            # for the instance and effectively "double up" the allocations for
-            # both the source and destination node. That's why when requesting
-            # allocations for resources on the destination node before we live
-            # migrate, we use the existing resource allocations from the
-            # source node.
-            if reportclient.claim_resources(
-                    self.instance.uuid, alloc_request,
-                    self.instance.project_id, self.instance.user_id):
-                LOG.debug('Instance allocations successfully created on '
-                          'destination node %(dest)s: %(alloc_request)s',
-                          {'dest': dest_node.uuid,
-                           'alloc_request': alloc_request},
-                          instance=self.instance)
-            else:
-                # We have to fail even though the user requested that we force
-                # the host. This is because we need Placement to have an
-                # accurate reflection of what's allocated on all nodes so the
-                # scheduler can make accurate decisions about which nodes have
-                # capacity for building an instance. We also cannot rely on the
-                # resource tracker in the compute service automatically healing
-                # the allocations since that code is going away in Queens.
-                reason = (_('Unable to migrate instance %(instance_uuid)s to '
-                            'host %(host)s. There is not enough capacity on '
-                            'the host for the instance.') %
-                          {'instance_uuid': self.instance.uuid,
-                           'host': self.destination})
-                raise exception.MigrationPreCheckError(reason=reason)
-        else:
-            # This shouldn't happen, but it could be a case where there are
-            # older (Ocata) computes still so the existing allocations are
-            # getting overwritten by the update_available_resource periodic
-            # task in the compute service.
-            # TODO(mriedem): Make this an error when the auto-heal
-            # compatibility code in the resource tracker is removed.
-            LOG.warning('No instance allocations found for source node '
-                        '%(source)s in Placement. Not creating allocations '
-                        'for destination node %(dest)s and assuming the '
-                        'compute service will heal the allocations.',
-                        {'source': source_node.uuid, 'dest': dest_node.uuid},
-                        instance=self.instance)
-
     def _check_destination_is_not_source(self):
         if self.destination == self.source:
             raise exception.UnableToMigrateToSelf(
@@ -329,8 +267,9 @@ class LiveMigrationTask(base.TaskBase):
             self._check_not_over_max_retries(attempted_hosts)
             request_spec.ignore_hosts = attempted_hosts
             try:
-                host = self.scheduler_client.select_destinations(self.context,
-                        request_spec, [self.instance.uuid])[0]['host']
+                hoststate = self.scheduler_client.select_destinations(
+                    self.context, request_spec, [self.instance.uuid])[0]
+                host = hoststate['host']
             except messaging.RemoteError as ex:
                 # TODO(ShaoHe Feng) There maybe multi-scheduler, and the
                 # scheduling algorithm is R-R, we can let other scheduler try.
@@ -346,9 +285,47 @@ class LiveMigrationTask(base.TaskBase):
                 LOG.debug("Skipping host: %(host)s because: %(e)s",
                     {"host": host, "e": e})
                 attempted_hosts.append(host)
+                # The scheduler would have created allocations against the
+                # selected destination host in Placement, so we need to remove
+                # those before moving on.
+                self._remove_host_allocations(host, hoststate['nodename'])
                 host = None
         return host
 
+    def _remove_host_allocations(self, host, node):
+        """Removes instance allocations against the given host from Placement
+
+        :param host: The name of the host.
+        :param node: The name of the node.
+        """
+        # Get the compute node object since we need the UUID.
+        # TODO(mriedem): If the result of select_destinations eventually
+        # returns the compute node uuid, we wouldn't need to look it
+        # up via host/node and we can save some time.
+        try:
+            compute_node = objects.ComputeNode.get_by_host_and_nodename(
+                self.context, host, node)
+        except exception.ComputeHostNotFound:
+            # This shouldn't happen, but we're being careful.
+            LOG.info('Unable to remove instance allocations from host %s '
+                     'and node %s since it was not found.', host, node,
+                     instance=self.instance)
+            return
+
+        # Calculate the resource class amounts to subtract from the allocations
+        # on the node based on the instance flavor.
+        resources = scheduler_utils.resources_from_flavor(
+            self.instance, self.instance.flavor)
+
+        # Now remove the allocations for our instance against that node.
+        # Note that this does not remove allocations against any other node
+        # or shared resource provider, it's just undoing what the scheduler
+        # allocated for the given (destination) node.
+        self.scheduler_client.reportclient.\
+            remove_provider_from_instance_allocation(
+                self.instance.uuid, compute_node.uuid, self.instance.user_id,
+                self.instance.project_id, resources)
+
     def _check_not_over_max_retries(self, attempted_hosts):
         if CONF.migrate_max_retries == -1:
             return
