@@ -25,7 +25,6 @@ from nova.conf import base
 from nova.conf import cache
 from nova.conf import cells
 from nova.conf import cinder
-from nova.conf import cloudpipe
 from nova.conf import compute
 from nova.conf import conductor
 from nova.conf import configdrive
@@ -35,11 +34,9 @@ from nova.conf import crypto
 from nova.conf import database
 from nova.conf import ephemeral_storage
 from nova.conf import flavors
-from nova.conf import floating_ips
 from nova.conf import glance
 from nova.conf import guestfs
 from nova.conf import hyperv
-from nova.conf import image_file_url
 from nova.conf import ipv6
 from nova.conf import ironic
 from nova.conf import key_manager
@@ -63,7 +60,6 @@ from nova.conf import service
 from nova.conf import service_token
 from nova.conf import servicegroup
 from nova.conf import spice
-from nova.conf import ssl
 from nova.conf import upgrade_levels
 from nova.conf import vendordata
 from nova.conf import vmware
@@ -81,7 +77,6 @@ base.register_opts(CONF)
 cache.register_opts(CONF)
 cells.register_opts(CONF)
 cinder.register_opts(CONF)
-cloudpipe.register_opts(CONF)
 compute.register_opts(CONF)
 conductor.register_opts(CONF)
 configdrive.register_opts(CONF)
@@ -90,13 +85,11 @@ consoleauth.register_opts(CONF)
 crypto.register_opts(CONF)
 database.register_opts(CONF)
 ephemeral_storage.register_opts(CONF)
-floating_ips.register_opts(CONF)
 flavors.register_opts(CONF)
 glance.register_opts(CONF)
 guestfs.register_opts(CONF)
 hyperv.register_opts(CONF)
 mks.register_opts(CONF)
-image_file_url.register_opts(CONF)
 ipv6.register_opts(CONF)
 ironic.register_opts(CONF)
 key_manager.register_opts(CONF)
@@ -118,7 +111,6 @@ service.register_opts(CONF)
 service_token.register_opts(CONF)
 servicegroup.register_opts(CONF)
 spice.register_opts(CONF)
-ssl.register_opts(CONF)
 upgrade_levels.register_opts(CONF)
 vendordata.register_opts(CONF)
 vmware.register_opts(CONF)
