@@ -23,6 +23,12 @@ from oslo_config import types
 
 from nova.conf import paths
 
+compute_group = cfg.OptGroup(
+    'compute',
+    title='Compute Manager Options',
+    help="""
+A collection of options specific to the nova-compute service.
+""")
 compute_opts = [
     cfg.StrOpt('compute_driver',
         help="""
@@ -43,20 +49,8 @@ Possible values:
 Allow destination machine to match source for resize. Useful when
 testing in single-host environments. By default it is not allowed
 to resize to the same host. Setting this option to true will add
-the same host to the destination options.
-"""),
-    cfg.StrOpt('default_schedule_zone',
-        help="""
-Availability zone to use when user doesn't specify one.
-
-This option is used by the scheduler to determine which availability
-zone to place a new VM instance into if the user did not specify one
-at the time of VM boot request.
-
-Possible values:
-
-* Any string representing an availability zone name
-* Default value is None.
+the same host to the destination options. Also set to true
+if you allow the ServerGroupAffinityFilter and need to resize.
 """),
     cfg.ListOpt('non_inheritable_image_properties',
         default=['cache_in_nova', 'bittorrent'],
@@ -222,7 +216,6 @@ Related options:
 * ``flat_inject``: This must be set to ``True`` to ensure nova embeds network
   configuration information in the metadata provided through the config drive.
 """),
-    # TODO(sfinucan): This should be a BooleanOpt
     cfg.StrOpt('preallocate_images',
         default='none',
         choices=('none', 'space'),
@@ -393,11 +386,24 @@ Possible values:
 
 * Any positive integer representing amount of memory in MB to reserve
   for the host.
+"""),
+    cfg.IntOpt('reserved_host_cpus',
+        default=0,
+        min=0,
+        help="""
+Number of physical CPUs to reserve for the host. The host resources usage is
+reported back to the scheduler continuously from nova-compute running on the
+compute node. To prevent the host CPU from being considered as available,
+this option is used to reserve random pCPU(s) for the host.
+
+Possible values:
+
+* Any positive integer representing number of physical CPUs to reserve
+  for the host.
 """),
 ]
 
 allocation_ratio_opts = [
-    # TODO(sfinucan): Add min parameter
     cfg.FloatOpt('cpu_allocation_ratio',
         default=0.0,
         min=0.0,
@@ -415,13 +421,15 @@ configuration value if no per-aggregate setting is found.
 
 NOTE: This can be set per-compute, or if set to 0.0, the value
 set on the scheduler node(s) or compute node(s) will be used
-and defaulted to 16.0'.
+and defaulted to 16.0.
+
+NOTE: As of the 16.0.0 Pike release, this configuration option is ignored
+for the ironic.IronicDriver compute driver and is hardcoded to 1.0.
 
 Possible values:
 
 * Any valid positive integer or float value
 """),
-    # TODO(sfinucan): Add min parameter
     cfg.FloatOpt('ram_allocation_ratio',
         default=0.0,
         min=0.0,
@@ -442,11 +450,13 @@ NOTE: This can be set per-compute, or if set to 0.0, the value
 set on the scheduler node(s) or compute node(s) will be used and
 defaulted to 1.5.
 
+NOTE: As of the 16.0.0 Pike release, this configuration option is ignored
+for the ironic.IronicDriver compute driver and is hardcoded to 1.0.
+
 Possible values:
 
 * Any valid positive integer or float value
 """),
-    # TODO(sfinucan): Add min parameter
     cfg.FloatOpt('disk_allocation_ratio',
         default=0.0,
         min=0.0,
@@ -469,7 +479,10 @@ instances.
 
 NOTE: This can be set per-compute, or if set to 0.0, the value
 set on the scheduler node(s) or compute node(s) will be used and
-defaulted to 1.0'.
+defaulted to 1.0.
+
+NOTE: As of the 16.0.0 Pike release, this configuration option is ignored
+for the ironic.IronicDriver compute driver and is hardcoded to 1.0.
 
 Possible values:
 
@@ -480,7 +493,7 @@ Possible values:
 compute_manager_opts = [
     cfg.StrOpt('console_host',
         default=socket.gethostname(),
-        sample_default="socket.gethostname()",
+        sample_default="<current_hostname>",
         help="""
 Console proxy host to be used to connect to instances on this host. It is the
 publicly visible name for the console host.
@@ -617,6 +630,27 @@ Possible values:
 """)
 ]
 
+compute_group_opts = [
+    cfg.IntOpt('consecutive_build_service_disable_threshold',
+        default=10,
+        help="""
+Number of consecutive failed builds that result in disabling a compute service.
+
+This option will cause nova-compute to set itself to a disabled state
+if a certain number of consecutive build failures occur. This will
+prevent the scheduler from continuing to send builds to a compute node that is
+consistently failing. Note that all failures qualify and count towards this
+score, including reschedules that may have been due to racy scheduler behavior.
+Since the failures must be consecutive, it is unlikely that occasional expected
+reschedules will actually disable a compute node.
+
+Possible values:
+
+* Any positive integer representing a build failure count.
+* Zero to never auto-disable.
+"""),
+]
+
 interval_opts = [
     cfg.IntOpt('image_cache_manager_interval',
         default=2400,
@@ -958,7 +992,7 @@ Possible values:
 
 Related options:
 
-* running_deleted_instance_poll
+* running_deleted_instance_poll_interval
 * running_deleted_instance_timeout
 """),
     cfg.IntOpt("running_deleted_instance_poll_interval",
@@ -1016,28 +1050,6 @@ Related options:
 """)
 ]
 
-rpcapi_opts = [
-    cfg.StrOpt("compute_topic",
-        default="compute",
-        deprecated_for_removal=True,
-        deprecated_since="15.0.0",
-        deprecated_reason="""
-There is no need to let users choose the RPC topic for all services - there
-is little gain from this. Furthermore, it makes it really easy to break Nova
-by using this option.
-""",
-        help="""
-This is the message queue topic that the compute service 'listens' on. It is
-used when the compute service is started up to configure the queue, and
-whenever an RPC call to the compute service is made.
-
-Possible values:
-
-* Any string, but there is almost never any reason to ever change this value
-  from its default of 'compute'.
-"""),
-]
-
 db_opts = [
     cfg.StrOpt('osapi_compute_unique_server_name_scope',
         default='',
@@ -1064,19 +1076,21 @@ Possible values:
     cfg.BoolOpt('enable_new_services',
         default=True,
         help="""
-Enable new services on this host automatically.
+Enable new nova-compute services on this host automatically.
 
-When a new service (for example "nova-compute") starts up, it gets
+When a new nova-compute service starts up, it gets
 registered in the database as an enabled service. Sometimes it can be useful
-to register new services in disabled state and then enabled them at a later
-point in time. This option can set this behavior for all services per host.
+to register new compute services in disabled state and then enabled them at a
+later point in time. This option only sets this behavior for nova-compute
+services, it does not auto-disable other services like nova-conductor,
+nova-scheduler, nova-consoleauth, or nova-osapi_compute.
 
 Possible values:
 
-* ``True``: Each new service is enabled as soon as it registers itself.
-* ``False``: Services must be enabled via a REST API call or with the CLI
-  with ``nova service-enable <hostname> <binary>``, otherwise they are not
-  ready to use.
+* ``True``: Each new compute service is enabled as soon as it registers itself.
+* ``False``: Compute services must be enabled via an os-services REST API call
+  or with the CLI with ``nova service-enable <hostname> <binary>``, otherwise
+  they are not ready to use.
 """),
     cfg.StrOpt('instance_name_template',
          default='instance-%08x',
@@ -1113,13 +1127,15 @@ ALL_OPTS = (compute_opts +
             timeout_opts +
             running_deleted_opts +
             instance_cleaning_opts +
-            rpcapi_opts +
             db_opts)
 
 
 def register_opts(conf):
     conf.register_opts(ALL_OPTS)
+    conf.register_group(compute_group)
+    conf.register_opts(compute_group_opts, group=compute_group)
 
 
 def list_opts():
-    return {'DEFAULT': ALL_OPTS}
+    return {'DEFAULT': ALL_OPTS,
+            'compute': compute_group_opts}
