@@ -62,6 +62,8 @@ vmware_utils_opts = [
 Set this value if affected by an increased network latency causing
 repeated characters when typing in a remote console.
 """),
+    # NOTE(takashin): 'serial_port_service_uri' can be non URI format.
+    # See https://github.com/openstack/vmware-vspc/blob/master/README.rst
     cfg.StrOpt('serial_port_service_uri',
                help="""
 Identifies the remote system where the serial port traffic will
@@ -76,14 +78,15 @@ Possible values:
 
 * Any valid URI
 """),
-    cfg.StrOpt('serial_port_proxy_uri',
+    cfg.URIOpt('serial_port_proxy_uri',
+               schemes=['telnet', 'telnets'],
                help="""
 Identifies a proxy service that provides network access to the
 serial_port_service_uri.
 
 Possible values:
 
-* Any valid URI
+* Any valid URI (The scheme is 'telnet' or 'telnets'.)
 
 Related options:
 This option is ignored if serial_port_service_uri is not specified.
@@ -92,8 +95,8 @@ This option is ignored if serial_port_service_uri is not specified.
 ]
 
 vmwareapi_opts = [
-    cfg.StrOpt('host_ip',
-               help="""
+    cfg.HostAddressOpt('host_ip',
+                       help="""
 Hostname or IP address for connection to VMware vCenter host."""),
     cfg.PortOpt('host_port',
                 default=443,
@@ -188,26 +191,6 @@ operation from the cache location to the hypervisor file directory
 in the shared datastore. If set to true, the above copy operation
 is avoided as it creates copy of the virtual machine that shares
 virtual disks with its parent VM.
-"""),
-    cfg.StrOpt('wsdl_location',
-               deprecated_for_removal=True,
-               deprecated_reason='Only vCenter versions earlier than 5.1 '
-                                 'require this option and the current '
-                                 'minimum version is 5.1.',
-               deprecated_since='15.0.0',
-               help="""
-This option specifies VIM Service WSDL Location
-
-If vSphere API versions 5.1 and later is being used, this section can
-be ignored. If version is less than 5.1, WSDL files must be hosted
-locally and their location must be specified in the above section.
-
-Optional over-ride to default location for bug work-arounds.
-
-Possible values:
-
-* http://<server>/vimService.wsdl
-* file:///opt/stack/vmware/SDK/wsdl/vim25/vimService.wsdl
 """)
 ]
 
