@@ -31,7 +31,6 @@ from nova.consoleauth import rpcapi as consoleauth_rpcapi
 from nova import context
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LW
 
 LOG = logging.getLogger(__name__)
 
@@ -86,7 +85,7 @@ class NovaProxyRequestHandlerBase(object):
             # NoVNC uses it's own convention that forward token
             # from the request to a cookie header, we should check
             # also for this behavior
-            hcookie = self.headers.getheader('cookie')
+            hcookie = self.headers.get('cookie')
             if hcookie:
                 cookie = Cookie.SimpleCookie()
                 for hcookie_part in hcookie.split(';'):
@@ -96,7 +95,7 @@ class NovaProxyRequestHandlerBase(object):
                     except Cookie.CookieError:
                         # NOTE(stgleb): Do not print out cookie content
                         # for security reasons.
-                        LOG.warning(_LW('Found malformed cookie'))
+                        LOG.warning('Found malformed cookie')
                     else:
                         if 'token' in cookie:
                             token = cookie['token'].value
@@ -109,7 +108,7 @@ class NovaProxyRequestHandlerBase(object):
             raise exception.InvalidToken(token=token)
 
         # Verify Origin
-        expected_origin_hostname = self.headers.getheader('Host')
+        expected_origin_hostname = self.headers.get('Host')
         if ':' in expected_origin_hostname:
             e = expected_origin_hostname
             if '[' in e and ']' in e:
@@ -118,7 +117,7 @@ class NovaProxyRequestHandlerBase(object):
                 expected_origin_hostname = e.split(':')[0]
         expected_origin_hostnames = CONF.console.allowed_origins
         expected_origin_hostnames.append(expected_origin_hostname)
-        origin_url = self.headers.getheader('Origin')
+        origin_url = self.headers.get('Origin')
         # missing origin header indicates non-browser client which is OK
         if origin_url is not None:
             origin = urlparse.urlparse(origin_url)
@@ -147,12 +146,15 @@ class NovaProxyRequestHandlerBase(object):
         if connect_info.get('internal_access_path'):
             tsock.send("CONNECT %s HTTP/1.1\r\n\r\n" %
                         connect_info['internal_access_path'])
+            end_token = "\r\n\r\n"
             while True:
                 data = tsock.recv(4096, socket.MSG_PEEK)
-                if data.find("\r\n\r\n") != -1:
+                token_loc = data.find(end_token)
+                if token_loc != -1:
                     if data.split("\r\n")[0].find("200") == -1:
                         raise exception.InvalidConnectionInfo()
-                    tsock.recv(len(data))
+                    # remove the response from recv buffer
+                    tsock.recv(token_loc + len(end_token))
                     break
 
         # Start proxying
