@@ -21,8 +21,6 @@ from sqlalchemy import ForeignKey, Index, Integer, MetaData, String, Table
 from sqlalchemy import Text
 from sqlalchemy.types import NullType
 
-from nova.i18n import _LE
-
 LOG = logging.getLogger(__name__)
 
 
@@ -81,37 +79,10 @@ def _create_shadow_tables(migrate_engine):
             shadow_table.create()
         except Exception:
             LOG.info(repr(shadow_table))
-            LOG.exception(_LE('Exception while creating table.'))
+            LOG.exception('Exception while creating table.')
             raise
 
 
-def _populate_instance_types(instance_types_table):
-    default_inst_types = {
-        'm1.tiny': dict(mem=512, vcpus=1, root_gb=1, eph_gb=0, flavid=1),
-        'm1.small': dict(mem=2048, vcpus=1, root_gb=20, eph_gb=0, flavid=2),
-        'm1.medium': dict(mem=4096, vcpus=2, root_gb=40, eph_gb=0, flavid=3),
-        'm1.large': dict(mem=8192, vcpus=4, root_gb=80, eph_gb=0, flavid=4),
-        'm1.xlarge': dict(mem=16384, vcpus=8, root_gb=160, eph_gb=0, flavid=5)
-        }
-
-    try:
-        i = instance_types_table.insert()
-        for name, values in default_inst_types.items():
-            i.execute({'name': name, 'memory_mb': values["mem"],
-                        'vcpus': values["vcpus"], 'deleted': 0,
-                        'root_gb': values["root_gb"],
-                        'ephemeral_gb': values["eph_gb"],
-                        'rxtx_factor': 1,
-                        'swap': 0,
-                        'flavorid': values["flavid"],
-                        'disabled': False,
-                        'is_public': True})
-    except Exception:
-        LOG.info(repr(instance_types_table))
-        LOG.exception(_LE('Exception while seeding instance_types table'))
-        raise
-
-
 # NOTE(dprince): we add these here so our schema contains dump tables
 # which were added in migration 209 (in Havana). We can drop these in
 # Icehouse: https://bugs.launchpad.net/nova/+bug/1266538
@@ -328,13 +299,9 @@ def upgrade(migrate_engine):
         mysql_charset='utf8'
     )
 
-    # NOTE(mriedem): DB2 can't create the FK since we don't have the unique
-    # constraint on instances.uuid because it's nullable (so a unique
-    # constraint isn't created for instances.uuid, only a unique index).
     consoles_instance_uuid_column_args = ['instance_uuid', String(length=36)]
-    if migrate_engine.name != 'ibm_db_sa':
-        consoles_instance_uuid_column_args.append(
-            ForeignKey('instances.uuid', name='consoles_instance_uuid_fkey'))
+    consoles_instance_uuid_column_args.append(
+        ForeignKey('instances.uuid', name='consoles_instance_uuid_fkey'))
 
     consoles = Table('consoles', meta,
         Column('created_at', DateTime),
@@ -1106,7 +1073,7 @@ def upgrade(migrate_engine):
             table.create()
         except Exception:
             LOG.info(repr(table))
-            LOG.exception(_LE('Exception while creating table.'))
+            LOG.exception('Exception while creating table.')
             raise
 
     # task log unique constraint
@@ -1127,7 +1094,7 @@ def upgrade(migrate_engine):
     UniqueConstraint('flavorid', 'deleted', table=instance_types,
                      name='uniq_instance_types0flavorid0deleted').create()
 
-    # keypair contraint
+    # keypair constraint
     UniqueConstraint('user_id', 'name', 'deleted', table=key_pairs,
                      name='uniq_key_pairs0user_id0name0deleted').create()
 
@@ -1461,9 +1428,6 @@ def upgrade(migrate_engine):
         'block_device_mapping_instance_uuid_virtual_name_device_name_idx'
     ]
 
-    # NOTE(mriedem): DB2 doesn't allow duplicate indexes either.
-    DB2_INDEX_SKIPS = POSTGRES_INDEX_SKIPS
-
     MYSQL_INDEX_SKIPS = [
         # we create this one manually for MySQL above
         'migrations_by_host_nodes_and_status_idx'
@@ -1473,9 +1437,7 @@ def upgrade(migrate_engine):
         if ((migrate_engine.name == 'postgresql' and
                 index.name in POSTGRES_INDEX_SKIPS) or
             (migrate_engine.name == 'mysql' and
-                index.name in MYSQL_INDEX_SKIPS) or
-            (migrate_engine.name == 'ibm_db_sa' and
-                index.name in DB2_INDEX_SKIPS)):
+                index.name in MYSQL_INDEX_SKIPS)):
             continue
         else:
             index.create(migrate_engine)
@@ -1506,52 +1468,47 @@ def upgrade(migrate_engine):
 
             ]
 
-    # NOTE(mriedem): DB2 doesn't support unique constraints on columns that
-    # are nullable so we can only create foreign keys on unique constraints
-    # that actually exist, which excludes any FK on instances.uuid.
-    if migrate_engine.name != 'ibm_db_sa':
+    secgroup_instance_association_instance_uuid_fkey = (
+                'security_group_instance_association_instance_uuid_fkey')
+    fkeys.extend(
+            [
 
-        secgroup_instance_association_instance_uuid_fkey = (
-                    'security_group_instance_association_instance_uuid_fkey')
-        fkeys.extend(
-                [
+              [[fixed_ips.c.instance_uuid],
+                  [instances.c.uuid],
+                  'fixed_ips_instance_uuid_fkey'],
+              [[block_device_mapping.c.instance_uuid],
+                  [instances.c.uuid],
+                  'block_device_mapping_instance_uuid_fkey'],
+              [[instance_info_caches.c.instance_uuid],
+                  [instances.c.uuid],
+                  'instance_info_caches_instance_uuid_fkey'],
+              [[instance_metadata.c.instance_uuid],
+                  [instances.c.uuid],
+                  'instance_metadata_instance_uuid_fkey'],
+              [[instance_system_metadata.c.instance_uuid],
+                  [instances.c.uuid],
+                  'instance_system_metadata_ibfk_1'],
+              [[security_group_instance_association.c.instance_uuid],
+                  [instances.c.uuid],
+                  secgroup_instance_association_instance_uuid_fkey],
+              [[virtual_interfaces.c.instance_uuid],
+                  [instances.c.uuid],
+                  'virtual_interfaces_instance_uuid_fkey'],
+              [[instance_actions.c.instance_uuid],
+                  [instances.c.uuid],
+                  'fk_instance_actions_instance_uuid'],
+              [[instance_faults.c.instance_uuid],
+                  [instances.c.uuid],
+                  'fk_instance_faults_instance_uuid'],
+              [[migrations.c.instance_uuid],
+                  [instances.c.uuid],
+                  'fk_migrations_instance_uuid']
 
-                  [[fixed_ips.c.instance_uuid],
-                      [instances.c.uuid],
-                      'fixed_ips_instance_uuid_fkey'],
-                  [[block_device_mapping.c.instance_uuid],
-                      [instances.c.uuid],
-                      'block_device_mapping_instance_uuid_fkey'],
-                  [[instance_info_caches.c.instance_uuid],
-                      [instances.c.uuid],
-                      'instance_info_caches_instance_uuid_fkey'],
-                  [[instance_metadata.c.instance_uuid],
-                      [instances.c.uuid],
-                      'instance_metadata_instance_uuid_fkey'],
-                  [[instance_system_metadata.c.instance_uuid],
-                      [instances.c.uuid],
-                      'instance_system_metadata_ibfk_1'],
-                  [[security_group_instance_association.c.instance_uuid],
-                      [instances.c.uuid],
-                      secgroup_instance_association_instance_uuid_fkey],
-                  [[virtual_interfaces.c.instance_uuid],
-                      [instances.c.uuid],
-                      'virtual_interfaces_instance_uuid_fkey'],
-                  [[instance_actions.c.instance_uuid],
-                      [instances.c.uuid],
-                      'fk_instance_actions_instance_uuid'],
-                  [[instance_faults.c.instance_uuid],
-                      [instances.c.uuid],
-                      'fk_instance_faults_instance_uuid'],
-                  [[migrations.c.instance_uuid],
-                      [instances.c.uuid],
-                      'fk_migrations_instance_uuid']
-
-                ])
+            ])
 
     for fkey_pair in fkeys:
-        if migrate_engine.name in ('mysql', 'ibm_db_sa'):
-            # For MySQL and DB2 we name our fkeys explicitly
+        if migrate_engine.name in ('mysql'):
+            # For MySQL we name our fkeys explicitly
             # so they match Havana
             fkey = ForeignKeyConstraint(columns=fkey_pair[0],
                                    refcolumns=fkey_pair[1],
@@ -1574,7 +1531,4 @@ def upgrade(migrate_engine):
 
     _create_shadow_tables(migrate_engine)
 
-    # populate initial instance types
-    _populate_instance_types(instance_types)
-
     _create_dump_tables(migrate_engine)
