@@ -27,15 +27,11 @@ from oslo_log import log as logging
 import webob.exc
 from webob import util as woutil
 
-import nova.conf
 from nova.i18n import _, _LE
 
 LOG = logging.getLogger(__name__)
 
 
-CONF = nova.conf.CONF
-
-
 class ConvertedException(webob.exc.WSGIHTTPException):
     def __init__(self, code, title="", explanation=""):
         self.code = code
@@ -86,17 +82,21 @@ class NovaException(Exception):
                 message = self.msg_fmt % kwargs
 
             except Exception:
-                # kwargs doesn't match a variable in the message
-                # log the issue and the kwargs
-                LOG.exception(_LE('Exception in string format operation'))
-                for name, value in kwargs.items():
-                    LOG.error("%s: %s" % (name, value))  # noqa
-
+                # NOTE(melwitt): This is done in a separate method so it can be
+                # monkey-patched during testing to make it a hard failure.
+                self._log_exception()
                 message = self.msg_fmt
 
         self.message = message
         super(NovaException, self).__init__(message)
 
+    def _log_exception(self):
+        # kwargs doesn't match a variable in the message
+        # log the issue and the kwargs
+        LOG.exception(_LE('Exception in string format operation'))
+        for name, value in self.kwargs.items():
+            LOG.error("%s: %s" % (name, value))  # noqa
+
     def format_message(self):
         # NOTE(mrodden): use the first argument to the python Exception object
         # which should be our full NovaException message, (see __init__)
@@ -145,6 +145,13 @@ class UnsupportedCinderAPIVersion(NovaException):
     msg_fmt = _('Nova does not support Cinder API version %(version)s')
 
 
+class CinderAPIVersionNotAvailable(NovaException):
+    """Used to indicate that a requested Cinder API version, generally a
+    microversion, is not available.
+    """
+    msg_fmt = _('Cinder API version %(version)s is not available.')
+
+
 class Forbidden(NovaException):
     msg_fmt = _("Forbidden")
     code = 403
@@ -158,10 +165,6 @@ class PolicyNotAuthorized(Forbidden):
     msg_fmt = _("Policy doesn't allow %(action)s to be performed.")
 
 
-class VolumeLimitExceeded(Forbidden):
-    msg_fmt = _("Volume resource quota exceeded")
-
-
 class ImageNotActive(NovaException):
     # NOTE(jruzicka): IncorrectState is used for volumes only in EC2,
     # but it still seems like the most appropriate option.
@@ -247,21 +250,40 @@ class VolumeAttachFailed(Invalid):
                 "Reason: %(reason)s")
 
 
-class VolumeUnattached(Invalid):
-    msg_fmt = _("Volume %(volume_id)s is not attached to anything")
-
-
 class VolumeNotCreated(NovaException):
     msg_fmt = _("Volume %(volume_id)s did not finish being created"
                 " even after we waited %(seconds)s seconds or %(attempts)s"
                 " attempts. And its status is %(volume_status)s.")
 
 
+class ExtendVolumeNotSupported(Invalid):
+    msg_fmt = _("Volume size extension is not supported by the hypervisor.")
+
+
 class VolumeEncryptionNotSupported(Invalid):
     msg_fmt = _("Volume encryption is not supported for %(volume_type)s "
                 "volume %(volume_id)s")
 
 
+class TaggedAttachmentNotSupported(Invalid):
+    msg_fmt = _("Tagged device attachment is not yet available.")
+
+
+class VolumeTaggedAttachNotSupported(TaggedAttachmentNotSupported):
+    msg_fmt = _("Tagged volume attachment is not supported for this server "
+                "instance.")
+
+
+class VolumeTaggedAttachToShelvedNotSupported(TaggedAttachmentNotSupported):
+    msg_fmt = _("Tagged volume attachment is not supported for "
+                "shelved-offloaded instances.")
+
+
+class NetworkInterfaceTaggedAttachNotSupported(TaggedAttachmentNotSupported):
+    msg_fmt = _("Tagged network interface attachment is not supported for "
+                "this server instance.")
+
+
 class InvalidKeypair(Invalid):
     msg_fmt = _("Keypair data is invalid: %(reason)s")
 
@@ -352,10 +374,6 @@ class InvalidAggregateActionUpdateMeta(InvalidAggregateAction):
                 "%(aggregate_id)s. Reason: %(reason)s.")
 
 
-class InvalidGroup(Invalid):
-    msg_fmt = _("Group not valid. Reason: %(reason)s")
-
-
 class InvalidSortKey(Invalid):
     msg_fmt = _("Sort key supplied was not valid.")
 
@@ -435,6 +453,10 @@ class ServiceUnavailable(Invalid):
     msg_fmt = _("Service is unavailable at this time.")
 
 
+class ServiceNotUnique(Invalid):
+    msg_fmt = _("More than one possible service found.")
+
+
 class ComputeResourcesUnavailable(ServiceUnavailable):
     msg_fmt = _("Insufficient compute resources: %(reason)s.")
 
@@ -490,10 +512,6 @@ class DevicePathInUse(Invalid):
     code = 409
 
 
-class DeviceIsBusy(Invalid):
-    msg_fmt = _("The supplied device (%(device)s) is busy.")
-
-
 class InvalidCPUInfo(Invalid):
     msg_fmt = _("Unacceptable CPU info: %(reason)s")
 
@@ -571,6 +589,10 @@ class AgentBuildExists(NovaException):
                 "architecture %(architecture)s exists.")
 
 
+class VolumeAttachmentNotFound(NotFound):
+    msg_fmt = _("Volume attachment %(attachment_id)s could not be found.")
+
+
 class VolumeNotFound(NotFound):
     msg_fmt = _("Volume %(volume_id)s could not be found.")
 
@@ -962,11 +984,6 @@ class ComputeHostNotFound(HostNotFound):
     msg_fmt = _("Compute host %(host)s could not be found.")
 
 
-class ComputeHostNotCreated(HostNotFound):
-    msg_fmt = _("Compute host %(name)s needs to be created first"
-                " before updating.")
-
-
 class HostBinaryNotFound(NotFound):
     msg_fmt = _("Could not find binary %(binary)s on host %(host)s.")
 
@@ -1010,6 +1027,10 @@ class QuotaClassNotFound(QuotaNotFound):
     msg_fmt = _("Quota class %(class_name)s could not be found.")
 
 
+class QuotaClassExists(NovaException):
+    msg_fmt = _("Quota class %(class_name)s exists for resource %(resource)s")
+
+
 class QuotaUsageNotFound(QuotaNotFound):
     msg_fmt = _("Quota usage for project %(project_id)s could not be found.")
 
@@ -1065,10 +1086,6 @@ class SecurityGroupCannotBeApplied(Invalid):
                 " in order to apply security groups.")
 
 
-class SecurityGroupRuleExists(Invalid):
-    msg_fmt = _("Rule already exists in group: %(rule)s")
-
-
 class NoUniqueMatch(NovaException):
     msg_fmt = _("No Unique Match Found.")
     code = 409
@@ -1103,10 +1120,6 @@ class ConsoleLogOutputException(NovaException):
                 "%(instance_id)s. Reason: %(reason)s")
 
 
-class ConsolePoolNotFound(NotFound):
-    msg_fmt = _("Console pool %(pool_id)s could not be found.")
-
-
 class ConsolePoolExists(NovaException):
     msg_fmt = _("Console pool with host %(host)s, console_type "
                 "%(console_type)s and compute_host %(compute_host)s "
@@ -1238,10 +1251,6 @@ class InstanceTagNotFound(NotFound):
     msg_fmt = _("Instance %(instance_id)s has no tag '%(tag)s'")
 
 
-class RotationRequiredForBackup(NovaException):
-    msg_fmt = _("Rotation param is required for backup image_type")
-
-
 class KeyPairExists(NovaException):
     msg_fmt = _("Key pair '%(key_name)s' already exists.")
 
@@ -1346,11 +1355,6 @@ class VolumeSmallerThanMinDisk(FlavorDiskTooSmall):
                 "size is %(image_min_disk)i bytes.")
 
 
-class BootFromVolumeRequiredForZeroDiskFlavor(Forbidden):
-    msg_fmt = _("Only volume-backed servers are allowed for flavors with "
-                "zero disk.")
-
-
 class InsufficientFreeMemory(NovaException):
     msg_fmt = _("Insufficient free memory on compute node to start %(uuid)s.")
 
@@ -1454,22 +1458,10 @@ class InstanceInfoCacheNotFound(NotFound):
                 "found.")
 
 
-class InvalidAssociation(NotFound):
-    msg_fmt = _("Invalid association.")
-
-
 class MarkerNotFound(NotFound):
     msg_fmt = _("Marker %(marker)s could not be found.")
 
 
-class InvalidInstanceIDMalformed(Invalid):
-    msg_fmt = _("Invalid id: %(instance_id)s (expecting \"i-...\")")
-
-
-class InvalidVolumeIDMalformed(Invalid):
-    msg_fmt = _("Invalid id: %(volume_id)s (expecting \"i-...\")")
-
-
 class CouldNotFetchImage(NovaException):
     msg_fmt = _("Could not fetch image %(image_id)s")
 
@@ -1624,10 +1616,6 @@ class ObjectActionError(NovaException):
     msg_fmt = _('Object action %(action)s failed because: %(reason)s')
 
 
-class CoreAPIMissing(NovaException):
-    msg_fmt = _("Core API extensions are missing: %(missing_apis)s")
-
-
 class AgentError(NovaException):
     msg_fmt = _('Error during following call to agent: %(method)s')
 
@@ -1658,33 +1646,6 @@ class InstanceGroupSaveException(NovaException):
     msg_fmt = _("%(field)s should not be part of the updates.")
 
 
-class PluginRetriesExceeded(NovaException):
-    msg_fmt = _("Number of retries to plugin (%(num_retries)d) exceeded.")
-
-
-class ImageDownloadModuleError(NovaException):
-    msg_fmt = _("There was an error with the download module %(module)s. "
-                "%(reason)s")
-
-
-class ImageDownloadModuleMetaDataError(ImageDownloadModuleError):
-    msg_fmt = _("The metadata for this location will not work with this "
-                "module %(module)s.  %(reason)s.")
-
-
-class ImageDownloadModuleNotImplementedError(ImageDownloadModuleError):
-    msg_fmt = _("The method %(method_name)s is not implemented.")
-
-
-class ImageDownloadModuleConfigurationError(ImageDownloadModuleError):
-    msg_fmt = _("The module %(module)s is misconfigured: %(reason)s.")
-
-
-class SignatureVerificationError(NovaException):
-    msg_fmt = _("Signature verification for the image "
-                "failed: %(reason)s.")
-
-
 class ResourceMonitorError(NovaException):
     msg_fmt = _("Error when creating resource monitor: %(monitor)s")
 
@@ -1693,11 +1654,6 @@ class PciDeviceWrongAddressFormat(NovaException):
     msg_fmt = _("The PCI address %(address)s has an incorrect format.")
 
 
-class PciDeviceInvalidAddressField(NovaException):
-    msg_fmt = _("Invalid PCI Whitelist: "
-                "The PCI address %(address)s has an invalid %(field)s.")
-
-
 class PciDeviceInvalidDeviceName(NovaException):
     msg_fmt = _("Invalid PCI Whitelist: "
                 "The PCI whitelist can specify devname or address,"
@@ -1756,11 +1712,6 @@ class PciRequestAliasNotDefined(NovaException):
     msg_fmt = _("PCI alias %(alias)s is not defined")
 
 
-class MissingParameter(NovaException):
-    msg_fmt = _("Not enough parameters: %(reason)s")
-    code = 400
-
-
 class PciConfigInvalidWhitelist(Invalid):
     msg_fmt = _("Invalid PCI devices Whitelist config %(reason)s")
 
@@ -1817,12 +1768,6 @@ class InvalidWatchdogAction(Invalid):
     msg_fmt = _("Provided watchdog action (%(action)s) is not supported.")
 
 
-class NoLiveMigrationForConfigDriveInLibVirt(NovaException):
-    msg_fmt = _("Live migration of instances with config drives is not "
-                "supported in libvirt unless libvirt instance path and "
-                "drive data is shared across compute nodes.")
-
-
 class LiveMigrationWithOldNovaNotSupported(NovaException):
     msg_fmt = _("Live migration with API v2.25 requires all the Mitaka "
                 "upgrade to be complete before it is available.")
@@ -1867,8 +1812,9 @@ class ImageNUMATopologyForbidden(Forbidden):
 
 
 class ImageNUMATopologyAsymmetric(Invalid):
-    msg_fmt = _("Asymmetric NUMA topologies require explicit assignment "
-                "of CPUs and memory to nodes in image or flavor")
+    msg_fmt = _("Instance CPUs and/or memory cannot be evenly distributed "
+                "across instance NUMA nodes. Explicit assignment of CPUs "
+                "and memory to nodes is required")
 
 
 class ImageNUMATopologyCPUOutOfRange(Invalid):
@@ -2026,14 +1972,6 @@ class MemoryPagesUnsupported(Invalid):
     msg_fmt = _("Host does not support guests with custom memory page sizes")
 
 
-class EnumFieldInvalid(Invalid):
-    msg_fmt = _('%(typename)s in %(fieldname)s is not an instance of Enum')
-
-
-class EnumFieldUnset(Invalid):
-    msg_fmt = _('%(fieldname)s missing field type')
-
-
 class InvalidImageFormat(Invalid):
     msg_fmt = _("Invalid image format '%(format)s'")
 
@@ -2076,6 +2014,11 @@ class LibguestfsCannotReadKernel(Invalid):
     msg_fmt = _("Libguestfs does not have permission to read host kernel.")
 
 
+class MaxDBRetriesExceeded(NovaException):
+    msg_fmt = _("Max retries of DB transaction exceeded attempting to "
+                "perform %(action)s.")
+
+
 class RealtimePolicyNotSupported(Invalid):
     msg_fmt = _("Realtime policy not supported by hypervisor")
 
@@ -2097,7 +2040,11 @@ class BuildRequestNotFound(NotFound):
 
 class AttachInterfaceNotSupported(Invalid):
     msg_fmt = _("Attaching interfaces is not supported for "
-                "instance %(instance)s.")
+                "instance %(instance_uuid)s.")
+
+
+class InstanceDiagnosticsNotSupported(Invalid):
+    msg_fmt = _("Instance diagnostics are not supported by compute node.")
 
 
 class InvalidReservedMemoryPagesOption(Invalid):
@@ -2123,6 +2070,10 @@ class InventoryWithResourceClassNotFound(NotFound):
     msg_fmt = _("No inventory of class %(resource_class)s found.")
 
 
+class InvalidResourceClass(Invalid):
+    msg_fmt = _("Resource class '%(resource_class)s' invalid.")
+
+
 class ResourceClassExists(NovaException):
     msg_fmt = _("Resource class %(resource_class)s already exists.")
 
@@ -2140,12 +2091,18 @@ class ResourceClassCannotUpdateStandard(Invalid):
     msg_fmt = _("Cannot update standard resource class %(resource_class)s.")
 
 
+class InvalidResourceAmount(Invalid):
+    msg_fmt = _("Resource amounts must be integers. Received '%(amount)s'.")
+
+
 class InvalidInventory(Invalid):
     msg_fmt = _("Inventory for '%(resource_class)s' on "
                 "resource provider '%(resource_provider)s' invalid.")
 
 
 class InventoryInUse(InvalidInventory):
+    # NOTE(mriedem): This message cannot change without impacting the
+    # nova.scheduler.client.report._RE_INV_IN_USE regex.
     msg_fmt = _("Inventory for '%(resource_classes)s' on "
                 "resource provider '%(resource_provider)s' in use.")
 
@@ -2192,3 +2149,34 @@ class PlacementNotConfigured(NovaException):
     msg_fmt = _("This compute is not configured to talk to the placement "
                 "service. Configure the [placement] section of nova.conf "
                 "and restart the service.")
+
+
+class InvalidEmulatorThreadsPolicy(Invalid):
+    msg_fmt = _("CPU emulator threads option requested is invalid, "
+                "given: '%(requested)s', available: '%(available)s'.")
+
+
+class BadRequirementEmulatorThreadsPolicy(Invalid):
+    msg_fmt = _("An isolated CPU emulator threads option requires a dedicated "
+                "CPU policy option.")
+
+
+class PowerVMAPIFailed(NovaException):
+    msg_fmt = _("PowerVM API failed to complete for instance=%(inst_name)s.  "
+                "%(reason)s")
+
+
+class TraitNotFound(NotFound):
+    msg_fmt = _("No such trait %(name)s.")
+
+
+class TraitExists(NovaException):
+    msg_fmt = _("The Trait %(name)s already exists")
+
+
+class TraitCannotDeleteStandard(Invalid):
+    msg_fmt = _("Cannot delete standard trait %(name)s.")
+
+
+class TraitInUse(Invalid):
+    msg_fmt = _("The trait %(name)s is in use by a resource provider.")
