@@ -24,10 +24,6 @@ from nova.i18n import _
 from nova import utils
 
 
-def ensure_string_keys(d):
-    # http://bugs.python.org/issue4978
-    return {str(k): v for k, v in d.items()}
-
 # Constants for the 'vif_type' field in VIF class
 VIF_TYPE_OVS = 'ovs'
 VIF_TYPE_IVS = 'ivs'
@@ -46,6 +42,7 @@ VIF_TYPE_VROUTER = 'vrouter'
 VIF_TYPE_OTHER = 'other'
 VIF_TYPE_TAP = 'tap'
 VIF_TYPE_MACVTAP = 'macvtap'
+VIF_TYPE_AGILIO_OVS = 'agilio_ovs'
 VIF_TYPE_BINDING_FAILED = 'binding_failed'
 VIF_TYPE_VIF = 'vif'
 
@@ -83,6 +80,10 @@ VIF_DETAILS_VHOSTUSER_OVS_PLUG = 'vhostuser_ovs_plug'
 # Specifies whether vhost-user socket should be used to
 # create a fp netdevice interface.
 VIF_DETAILS_VHOSTUSER_FP_PLUG = 'vhostuser_fp_plug'
+# Specifies whether vhost-user socket should be used to
+# create a vrouter netdevice interface
+# TODO(mhenkel): Consider renaming this to be contrail-specific.
+VIF_DETAILS_VHOSTUSER_VROUTER_PLUG = 'vhostuser_vrouter_plug'
 # ovs vhost user interface type name
 OVS_VHOSTUSER_INTERFACE_TYPE = 'dpdkvhostuser'
 
@@ -97,12 +98,15 @@ VNIC_TYPE_DIRECT = 'direct'
 VNIC_TYPE_MACVTAP = 'macvtap'
 VNIC_TYPE_DIRECT_PHYSICAL = 'direct-physical'
 VNIC_TYPE_BAREMETAL = 'baremetal'
+VNIC_TYPE_VIRTIO_FORWARDER = 'virtio-forwarder'
 
 # Define list of ports which needs pci request.
 # Note: The macvtap port needs a PCI request as it is a tap interface
 # with VF as the lower physical interface.
+# Note: Currently, VNIC_TYPE_VIRTIO_FORWARDER assumes a 1:1
+# relationship with a VF. This is expected to change in the future.
 VNIC_TYPES_SRIOV = (VNIC_TYPE_DIRECT, VNIC_TYPE_MACVTAP,
-                    VNIC_TYPE_DIRECT_PHYSICAL)
+                    VNIC_TYPE_DIRECT_PHYSICAL, VNIC_TYPE_VIRTIO_FORWARDER)
 
 # Define list of ports which are passthrough to the guest
 # and need a special treatment on snapshot and suspend/resume
@@ -118,6 +122,7 @@ VIF_MODEL_E1000 = 'e1000'
 VIF_MODEL_E1000E = 'e1000e'
 VIF_MODEL_NETFRONT = 'netfront'
 VIF_MODEL_SPAPR_VLAN = 'spapr-vlan'
+VIF_MODEL_LAN9118 = 'lan9118'
 
 VIF_MODEL_SRIOV = 'sriov'
 VIF_MODEL_VMXNET = 'vmxnet'
@@ -132,6 +137,7 @@ VIF_MODEL_ALL = (
     VIF_MODEL_E1000E,
     VIF_MODEL_NETFRONT,
     VIF_MODEL_SPAPR_VLAN,
+    VIF_MODEL_LAN9118,
     VIF_MODEL_SRIOV,
     VIF_MODEL_VMXNET,
     VIF_MODEL_VMXNET3,
@@ -207,7 +213,7 @@ class IP(Model):
     @classmethod
     def hydrate(cls, ip):
         if ip:
-            return cls(**ensure_string_keys(ip))
+            return cls(**ip)
         return None
 
 
@@ -229,7 +235,7 @@ class FixedIP(IP):
 
     @staticmethod
     def hydrate(fixed_ip):
-        fixed_ip = FixedIP(**ensure_string_keys(fixed_ip))
+        fixed_ip = FixedIP(**fixed_ip)
         fixed_ip['floating_ips'] = [IP.hydrate(floating_ip)
                                    for floating_ip in fixed_ip['floating_ips']]
         return fixed_ip
@@ -256,7 +262,7 @@ class Route(Model):
 
     @classmethod
     def hydrate(cls, route):
-        route = cls(**ensure_string_keys(route))
+        route = cls(**route)
         route['gateway'] = IP.hydrate(route['gateway'])
         return route
 
@@ -304,7 +310,7 @@ class Subnet(Model):
 
     @classmethod
     def hydrate(cls, subnet):
-        subnet = cls(**ensure_string_keys(subnet))
+        subnet = cls(**subnet)
         subnet['dns'] = [IP.hydrate(dns) for dns in subnet['dns']]
         subnet['ips'] = [FixedIP.hydrate(ip) for ip in subnet['ips']]
         subnet['routes'] = [Route.hydrate(route) for route in subnet['routes']]
@@ -332,7 +338,7 @@ class Network(Model):
     @classmethod
     def hydrate(cls, network):
         if network:
-            network = cls(**ensure_string_keys(network))
+            network = cls(**network)
             network['subnets'] = [Subnet.hydrate(subnet)
                                   for subnet in network['subnets']]
         return network
@@ -403,11 +409,8 @@ class VIF(Model):
         return not self.__eq__(other)
 
     def fixed_ips(self):
-        if self['network']:
-            return [fixed_ip for subnet in self['network']['subnets']
-                             for fixed_ip in subnet['ips']]
-        else:
-            return []
+        return [fixed_ip for subnet in self['network']['subnets']
+                         for fixed_ip in subnet['ips']]
 
     def floating_ips(self):
         return [floating_ip for fixed_ip in self.fixed_ips()
@@ -435,7 +438,7 @@ class VIF(Model):
         """
         if self['network']:
             # remove unnecessary fields on fixed_ips
-            ips = [IP(**ensure_string_keys(ip)) for ip in self.fixed_ips()]
+            ips = [IP(**ip) for ip in self.fixed_ips()]
             for ip in ips:
                 # remove floating ips from IP, since this is a flat structure
                 # of all IPs
@@ -461,7 +464,7 @@ class VIF(Model):
 
     @classmethod
     def hydrate(cls, vif):
-        vif = cls(**ensure_string_keys(vif))
+        vif = cls(**vif)
         vif['network'] = Network.hydrate(vif['network'])
         return vif
 
