@@ -27,6 +27,7 @@ from nova import rpc
 
 
 CONF = nova.conf.CONF
+RPC_TOPIC = 'network'
 
 
 @profiler.trace_cls("rpc")
@@ -109,7 +110,7 @@ class NetworkAPI(object):
 
         * 1.17 - Add method release_dhcp()
 
-        ... Newton supports message version 1.17.  So, any changes to
+        ... Newton and Ocata support message version 1.17.  So, any changes to
         existing methods in 1.x after that point should be done such that they
         can handle the version_cap being set to 1.17.
     '''
@@ -123,11 +124,12 @@ class NetworkAPI(object):
         'liberty': '1.15',
         'mitaka': '1.16',
         'newton': '1.17',
+        'ocata': '1.17',
     }
 
     def __init__(self, topic=None):
         super(NetworkAPI, self).__init__()
-        topic = topic or CONF.network_topic
+        topic = topic or RPC_TOPIC
         target = messaging.Target(topic=topic, version='1.0')
         version_cap = self.VERSION_ALIASES.get(CONF.upgrade_levels.network,
                                                CONF.upgrade_levels.network)
