@@ -34,7 +34,6 @@ from nova.image import glance
 from nova import network
 from nova.network import model as network_model
 from nova.notifications.objects import base as notification_base
-from nova.notifications.objects import flavor as flavor_notification
 from nova.notifications.objects import instance as instance_notification
 from nova import objects
 from nova.objects import base as obj_base
@@ -135,7 +134,7 @@ def send_update(context, old_instance, new_instance, service="compute",
             old_display_name = None
             if new_instance["display_name"] != old_instance["display_name"]:
                 old_display_name = old_instance["display_name"]
-            _send_instance_update_notification(context, new_instance,
+            send_instance_update_notification(context, new_instance,
                     service=service, host=host,
                     old_display_name=old_display_name)
         except exception.InstanceNotFound:
@@ -177,7 +176,7 @@ def send_update_with_states(context, instance, old_vm_state, new_vm_state,
     if fire_update:
         # send either a state change or a regular notification
         try:
-            _send_instance_update_notification(context, instance,
+            send_instance_update_notification(context, instance,
                     old_vm_state=old_vm_state, old_task_state=old_task_state,
                     new_vm_state=new_vm_state, new_task_state=new_task_state,
                     service=service, host=host)
@@ -218,7 +217,8 @@ def _compute_states_payload(instance, old_vm_state=None,
     return states_payload
 
 
-def _send_instance_update_notification(context, instance, old_vm_state=None,
+@rpc.if_notifications_enabled
+def send_instance_update_notification(context, instance, old_vm_state=None,
             old_task_state=None, new_vm_state=None, new_task_state=None,
             service="compute", host=None, old_display_name=None):
     """Send 'compute.instance.update' notification to inform observers
@@ -235,8 +235,8 @@ def _send_instance_update_notification(context, instance, old_vm_state=None,
 
     # add audit fields:
     (audit_start, audit_end) = audit_period_bounds(current_period=True)
-    payload["audit_period_beginning"] = audit_start
-    payload["audit_period_ending"] = audit_end
+    payload["audit_period_beginning"] = null_safe_isotime(audit_start)
+    payload["audit_period_ending"] = null_safe_isotime(audit_end)
 
     # add bw usage info:
     bw = bandwidth_usage(instance, audit_start)
@@ -252,18 +252,15 @@ def _send_instance_update_notification(context, instance, old_vm_state=None,
     _send_versioned_instance_update(context, instance, payload, host, service)
 
 
-def _map_service_to_binary(service):
-    if service == 'api':
-        binary = 'nova-api'
-    elif service == 'compute':
-        binary = 'nova-compute'
-    else:
-        binary = service
-    return binary
-
-
+@rpc.if_notifications_enabled
 def _send_versioned_instance_update(context, instance, payload, host, service):
 
+    def _map_legacy_service_to_binary(legacy_service):
+        if not legacy_service.startswith('nova-'):
+            return 'nova-' + service
+        else:
+            return service
+
     state_update = instance_notification.InstanceStateUpdatePayload(
         old_state=payload.get('old_state'),
         state=payload.get('state'),
@@ -280,17 +277,11 @@ def _send_versioned_instance_update(context, instance, payload, host, service):
                     out_bytes=bw['bw_out'])
                  for label, bw in payload['bandwidth'].items()]
 
-    network_info = instance.info_cache.network_info
-    flavor = flavor_notification.FlavorPayload(instance.flavor)
-
     versioned_payload = instance_notification.InstanceUpdatePayload(
         instance=instance,
         state_update=state_update,
         audit_period=audit_period,
         bandwidth=bandwidth,
-        ip_addresses=instance_notification.IpPayload.from_network_info(
-            network_info),
-        flavor=flavor,
         old_display_name=payload.get('old_display_name'))
 
     notification = instance_notification.InstanceUpdateNotification(
@@ -300,7 +291,7 @@ def _send_versioned_instance_update(context, instance, payload, host, service):
             action=fields.NotificationAction.UPDATE),
         publisher=notification_base.NotificationPublisher(
                 host=host or CONF.host,
-                binary=_map_service_to_binary(service)),
+                binary=_map_legacy_service_to_binary(service)),
         payload=versioned_payload)
     notification.emit(context)
 
@@ -392,6 +383,17 @@ def image_meta(system_metadata):
     return image_meta
 
 
+def null_safe_str(s):
+    return str(s) if s else ''
+
+
+def null_safe_isotime(s):
+    if isinstance(s, datetime.datetime):
+        return utils.strtime(s)
+    else:
+        return str(s) if s else ''
+
+
 def info_from_instance(context, instance, network_info,
                 system_metadata, **kw):
     """Get detailed instance information for an instance which is common to all
@@ -408,19 +410,6 @@ def info_from_instance(context, instance, network_info,
         modifications.
 
     """
-
-    def null_safe_str(s):
-        return str(s) if s else ''
-
-    def null_safe_int(s):
-        return int(s) if s else ''
-
-    def null_safe_isotime(s):
-        if isinstance(s, datetime.datetime):
-            return utils.strtime(s)
-        else:
-            return str(s) if s else ''
-
     image_ref_url = glance.generate_image_url(instance.image_ref)
 
     instance_type = instance.get_flavor()
@@ -478,7 +467,11 @@ def info_from_instance(context, instance, network_info,
         # Status properties
         state=instance.vm_state,
         state_description=null_safe_str(instance.task_state),
-        progress=null_safe_int(instance.progress),
+        # NOTE(gibi): It might seems wrong to default the progress to an empty
+        # string but this is how legacy work and this code only used by the
+        # legacy notification so try to keep the compatibility here but also
+        # keep it contained.
+        progress=int(instance.progress) if instance.progress else '',
 
         # accessIPs
         access_ip_v4=instance.access_ip_v4,
