@@ -11,11 +11,16 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from oslo_log import log as logging
+from oslo_versionedobjects import exception as ovo_exception
 
+from nova import exception
 from nova.objects import base
 from nova.objects import fields
 from nova import rpc
 
+LOG = logging.getLogger(__name__)
+
 
 @base.NovaObjectRegistry.register_if(False)
 class NotificationObject(base.NovaObject):
@@ -51,10 +56,16 @@ class EventType(NotificationObject):
         'phase': fields.NotificationPhaseField(nullable=True),
     }
 
+    def __init__(self, object, action, phase=None):
+        super(EventType, self).__init__()
+        self.object = object
+        self.action = action
+        self.phase = phase
+
     def to_notification_event_type_field(self):
         """Serialize the object to the wire format."""
         s = '%s.%s' % (self.object, self.action)
-        if self.obj_attr_is_set('phase'):
+        if self.phase:
             s += '.%s' % self.phase
         return s
 
@@ -84,10 +95,11 @@ class NotificationPayloadBase(NotificationObject):
     # Version 1.0: Initial version
     VERSION = '1.0'
 
-    def __init__(self, **kwargs):
-        super(NotificationPayloadBase, self).__init__(**kwargs)
+    def __init__(self):
+        super(NotificationPayloadBase, self).__init__()
         self.populated = not self.SCHEMA
 
+    @rpc.if_notifications_enabled
     def populate_schema(self, **kwargs):
         """Populate the object based on the SCHEMA and the source objects
 
@@ -96,13 +108,31 @@ class NotificationPayloadBase(NotificationObject):
         """
         for key, (obj, field) in self.SCHEMA.items():
             source = kwargs[obj]
-            if source.obj_attr_is_set(field):
+            # trigger lazy-load if possible
+            try:
                 setattr(self, key, getattr(source, field))
+            # ObjectActionError - not lazy loadable field
+            # NotImplementedError - obj_load_attr() is not even defined
+            # OrphanedObjectError - lazy loadable field but context is None
+            except (exception.ObjectActionError,
+                    NotImplementedError,
+                    exception.OrphanedObjectError,
+                    ovo_exception.OrphanedObjectError) as e:
+                LOG.debug(("Defaulting the value of the field '%(field)s' "
+                           "to None in %(payload)s due to '%(exception)s'"),
+                          {'field': key,
+                           'payload': self.__class__.__name__,
+                           'exception': e})
+                # NOTE(gibi): This will fail if the payload field is not
+                # nullable, but that means that either the source object is not
+                # properly initialized or the payload field needs to be defined
+                # as nullable
+                setattr(self, key, None)
         self.populated = True
 
         # the schema population will create changed fields but we don't need
         # this information in the notification
-        self.obj_reset_changes(recursive=False)
+        self.obj_reset_changes(recursive=True)
 
 
 @base.NovaObjectRegistry.register_notification
@@ -115,6 +145,11 @@ class NotificationPublisher(NotificationObject):
         'binary': fields.StringField(nullable=False),
     }
 
+    def __init__(self, host, binary):
+        super(NotificationPublisher, self).__init__()
+        self.host = host
+        self.binary = binary
+
     @classmethod
     def from_service_obj(cls, service):
         return cls(host=service.host, binary=service.binary)
@@ -140,6 +175,7 @@ class NotificationBase(NotificationObject):
         notify = getattr(notifier, self.priority)
         notify(context, event_type=event_type, payload=payload)
 
+    @rpc.if_notifications_enabled
     def emit(self, context):
         """Send the notification."""
         assert self.payload.populated
@@ -147,7 +183,7 @@ class NotificationBase(NotificationObject):
         # Note(gibi): notification payload will be a newly populated object
         # therefore every field of it will look changed so this does not carry
         # any extra information so we drop this from the payload.
-        self.payload.obj_reset_changes(recursive=False)
+        self.payload.obj_reset_changes(recursive=True)
 
         self._emit(context,
                    event_type=
