@@ -55,7 +55,13 @@ class RequestSpec(base.NovaObject):
         'flavor': fields.ObjectField('Flavor', nullable=False),
         'num_instances': fields.IntegerField(default=1),
         'ignore_hosts': fields.ListOfStringsField(nullable=True),
+        # NOTE(mriedem): In reality, you can only ever have one
+        # host in the force_hosts list. The fact this is a list
+        # is a mistake perpetuated over time.
         'force_hosts': fields.ListOfStringsField(nullable=True),
+        # NOTE(mriedem): In reality, you can only ever have one
+        # node in the force_nodes list. The fact this is a list
+        # is a mistake perpetuated over time.
         'force_nodes': fields.ListOfStringsField(nullable=True),
         'requested_destination': fields.ObjectField('Destination',
                                                     nullable=True,
@@ -258,8 +264,6 @@ class RequestSpec(base.NovaObject):
         spec._populate_group_info(filter_properties)
         scheduler_hints = filter_properties.get('scheduler_hints', {})
         spec._from_hints(scheduler_hints)
-        spec.requested_destination = filter_properties.get(
-            'requested_destination')
 
         # NOTE(sbauza): Default the other fields that are not part of the
         # original contract
@@ -316,8 +320,7 @@ class RequestSpec(base.NovaObject):
         # the existing dictionary as a primitive.
         return {'group_updated': True,
                 'group_hosts': set(self.instance_group.hosts),
-                'group_policies': set(self.instance_group.policies),
-                'group_members': set(self.instance_group.members)}
+                'group_policies': set(self.instance_group.policies)}
 
     def to_legacy_request_spec_dict(self):
         """Returns a legacy request_spec dict from the RequestSpec object.
@@ -368,15 +371,12 @@ class RequestSpec(base.NovaObject):
             # we had to hydrate the field by putting a single item into a list.
             filt_props['scheduler_hints'] = {hint: self.get_scheduler_hint(
                 hint) for hint in self.scheduler_hints}
-        if self.obj_attr_is_set('requested_destination'
-                                ) and self.requested_destination:
-            filt_props['requested_destination'] = self.requested_destination
         return filt_props
 
     @classmethod
     def from_components(cls, context, instance_uuid, image, flavor,
             numa_topology, pci_requests, filter_properties, instance_group,
-            availability_zone, security_groups=None, project_id=None):
+            availability_zone, security_groups=None):
         """Returns a new RequestSpec object hydrated by various components.
 
         This helper is useful in creating the RequestSpec from the various
@@ -395,8 +395,6 @@ class RequestSpec(base.NovaObject):
         :param availability_zone: an availability_zone string
         :param security_groups: A SecurityGroupList object. If None, don't
                                 set security_groups on the resulting object.
-        :param project_id: The project_id for the requestspec (should match
-                           the instance project_id).
         """
         spec_obj = cls(context)
         spec_obj.num_instances = 1
@@ -404,7 +402,7 @@ class RequestSpec(base.NovaObject):
         spec_obj.instance_group = instance_group
         if spec_obj.instance_group is None and filter_properties:
             spec_obj._populate_group_info(filter_properties)
-        spec_obj.project_id = project_id or context.project_id
+        spec_obj.project_id = context.project_id
         spec_obj._image_meta_from_image(image)
         spec_obj._from_flavor(flavor)
         spec_obj._from_instance_pci_requests(pci_requests)
@@ -418,8 +416,6 @@ class RequestSpec(base.NovaObject):
         spec_obj.availability_zone = availability_zone
         if security_groups is not None:
             spec_obj.security_groups = security_groups
-        spec_obj.requested_destination = filter_properties.get(
-            'requested_destination')
 
         # NOTE(sbauza): Default the other fields that are not part of the
         # original contract
@@ -437,18 +433,6 @@ class RequestSpec(base.NovaObject):
             else:
                 setattr(spec, key, getattr(spec_obj, key))
         spec._context = context
-
-        if 'instance_group' in spec and spec.instance_group:
-            # NOTE(danms): We don't store the full instance group in
-            # the reqspec since it would be stale almost immediately.
-            # Instead, load it by uuid here so it's up-to-date.
-            try:
-                spec.instance_group = objects.InstanceGroup.get_by_uuid(
-                    context, spec.instance_group.uuid)
-            except exception.InstanceGroupNotFound:
-                # NOTE(danms): Instance group may have been deleted
-                spec.instance_group = None
-
         spec.obj_reset_changes()
         return spec
 
@@ -488,17 +472,7 @@ class RequestSpec(base.NovaObject):
         # NOTE(alaski): The db schema is the full serialized object in a
         # 'spec' column.  If anything has changed we rewrite the full thing.
         if updates:
-            # NOTE(danms): Don't persist the could-be-large and could-be-stale
-            # properties of InstanceGroup
-            spec = self.obj_clone()
-            if 'instance_group' in spec and spec.instance_group:
-                spec.instance_group.members = None
-                spec.instance_group.hosts = None
-            # NOTE(mriedem): Don't persist retries since those are per-request
-            if 'retry' in spec and spec.retry:
-                spec.retry = None
-
-            db_updates = {'spec': jsonutils.dumps(spec.obj_to_primitive())}
+            db_updates = {'spec': jsonutils.dumps(self.obj_to_primitive())}
             if 'instance_uuid' in updates:
                 db_updates['instance_uuid'] = updates['instance_uuid']
         return db_updates
@@ -594,21 +568,15 @@ def _set_or_delete_marker_for_migrate_instances(context, marker=None):
 
 def _create_minimal_request_spec(context, instance):
     image = instance.image_meta
-    # TODO(sbauza): Modify that once setup_instance_group() accepts a
-    # RequestSpec object
-    request_spec = {'instance_properties': {'uuid': instance.uuid}}
-    filter_properties = {}
-    scheduler_utils.setup_instance_group(context, request_spec,
-                                         filter_properties)
     # This is an old instance. Let's try to populate a RequestSpec
     # object using the existing information we have previously saved.
     request_spec = objects.RequestSpec.from_components(
         context, instance.uuid, image,
         instance.flavor, instance.numa_topology,
         instance.pci_requests,
-        filter_properties, None, instance.availability_zone,
-        project_id=instance.project_id
+        {}, None, instance.availability_zone
     )
+    scheduler_utils.setup_instance_group(context, request_spec)
     request_spec.create()
 
 
@@ -645,7 +613,8 @@ def migrate_instances_add_request_spec(context, max_count):
 @base.NovaObjectRegistry.register
 class Destination(base.NovaObject):
     # Version 1.0: Initial version
-    VERSION = '1.0'
+    # Version 1.1: Add cell field
+    VERSION = '1.1'
 
     fields = {
         'host': fields.StringField(),
@@ -653,8 +622,16 @@ class Destination(base.NovaObject):
         # and also remove the possibility to have multiple nodes per service,
         # let's provide a possible nullable node here.
         'node': fields.StringField(nullable=True),
+        'cell': fields.ObjectField('CellMapping', nullable=True),
     }
 
+    def obj_make_compatible(self, primitive, target_version):
+        super(Destination, self).obj_make_compatible(primitive, target_version)
+        target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 1):
+            if 'cell' in primitive:
+                del primitive['cell']
+
 
 @base.NovaObjectRegistry.register
 class SchedulerRetries(base.NovaObject):
