@@ -23,23 +23,21 @@ __all__ = [
     'get_client',
     'get_server',
     'get_notifier',
-    'TRANSPORT_ALIASES',
 ]
 
 import functools
 
 from oslo_log import log as logging
 import oslo_messaging as messaging
+from oslo_messaging.rpc import dispatcher
 from oslo_serialization import jsonutils
 from oslo_service import periodic_task
 from oslo_utils import importutils
-from oslo_utils import timeutils
 
 import nova.conf
 import nova.context
 import nova.exception
 from nova.i18n import _
-from nova import objects
 
 profiler = importutils.try_import("osprofiler.profiler")
 
@@ -58,25 +56,13 @@ ALLOWED_EXMODS = [
 ]
 EXTRA_EXMODS = []
 
-# NOTE(markmc): The nova.openstack.common.rpc entries are for backwards compat
-# with Havana rpc_backend configuration values. The nova.rpc entries are for
-# compat with Essex values.
-TRANSPORT_ALIASES = {
-    'nova.openstack.common.rpc.impl_kombu': 'rabbit',
-    'nova.openstack.common.rpc.impl_qpid': 'qpid',
-    'nova.openstack.common.rpc.impl_zmq': 'zmq',
-    'nova.rpc.impl_kombu': 'rabbit',
-    'nova.rpc.impl_qpid': 'qpid',
-    'nova.rpc.impl_zmq': 'zmq',
-}
-
 
 def init(conf):
     global TRANSPORT, NOTIFICATION_TRANSPORT, LEGACY_NOTIFIER, NOTIFIER
     exmods = get_allowed_exmods()
     TRANSPORT = create_transport(get_transport_url())
     NOTIFICATION_TRANSPORT = messaging.get_notification_transport(
-        conf, allowed_remote_exmods=exmods, aliases=TRANSPORT_ALIASES)
+        conf, allowed_remote_exmods=exmods)
     serializer = RequestContextSerializer(JsonPayloadSerializer())
     if conf.notifications.notification_format == 'unversioned':
         LEGACY_NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
@@ -86,16 +72,18 @@ def init(conf):
     elif conf.notifications.notification_format == 'both':
         LEGACY_NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
                                              serializer=serializer)
-        NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
-                                      serializer=serializer,
-                                      topics=['versioned_notifications'])
+        NOTIFIER = messaging.Notifier(
+            NOTIFICATION_TRANSPORT,
+            serializer=serializer,
+            topics=conf.notifications.versioned_notifications_topics)
     else:
         LEGACY_NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
                                              serializer=serializer,
                                              driver='noop')
-        NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
-                                      serializer=serializer,
-                                      topics=['versioned_notifications'])
+        NOTIFIER = messaging.Notifier(
+            NOTIFICATION_TRANSPORT,
+            serializer=serializer,
+            topics=conf.notifications.versioned_notifications_topics)
 
 
 def cleanup():
@@ -181,7 +169,7 @@ class ProfilerRequestContextSerializer(RequestContextSerializer):
 
 
 def get_transport_url(url_str=None):
-    return messaging.TransportURL.parse(CONF, url_str, TRANSPORT_ALIASES)
+    return messaging.TransportURL.parse(CONF, url_str)
 
 
 def get_client(target, version_cap=None, serializer=None):
@@ -205,12 +193,13 @@ def get_server(target, endpoints, serializer=None):
         serializer = ProfilerRequestContextSerializer(serializer)
     else:
         serializer = RequestContextSerializer(serializer)
-
+    access_policy = dispatcher.DefaultRPCAccessPolicy
     return messaging.get_rpc_server(TRANSPORT,
                                     target,
                                     endpoints,
                                     executor='eventlet',
-                                    serializer=serializer)
+                                    serializer=serializer,
+                                    access_policy=access_policy)
 
 
 def get_notifier(service, host=None, publisher_id=None):
@@ -226,12 +215,24 @@ def get_versioned_notifier(publisher_id):
     return NOTIFIER.prepare(publisher_id=publisher_id)
 
 
+def if_notifications_enabled(f):
+    """Calls decorated method only if versioned notifications are enabled."""
+    @functools.wraps(f)
+    def wrapped(*args, **kwargs):
+        if (NOTIFIER.is_enabled() and
+                CONF.notifications.notification_format in ('both',
+                                                           'versioned')):
+            return f(*args, **kwargs)
+        else:
+            return None
+    return wrapped
+
+
 def create_transport(url):
     exmods = get_allowed_exmods()
-    return messaging.get_transport(CONF,
-                                   url=url,
-                                   allowed_remote_exmods=exmods,
-                                   aliases=TRANSPORT_ALIASES)
+    return messaging.get_rpc_transport(CONF,
+                                       url=url,
+                                       allowed_remote_exmods=exmods)
 
 
 class LegacyValidatingNotifier(object):
@@ -275,8 +276,6 @@ class LegacyValidatingNotifier(object):
         'compute.instance.finish_resize.start',
         'compute.instance.live.migration.abort.start',
         'compute.instance.live.migration.abort.end',
-        'compute.instance.live.migration.force.complete.start',
-        'compute.instance.live.migration.force.complete.end',
         'compute.instance.live_migration.post.dest.end',
         'compute.instance.live_migration.post.dest.start',
         'compute.instance.live_migration._post.end',
@@ -339,7 +338,6 @@ class LegacyValidatingNotifier(object):
         'compute.instance.volume.attach',
         'compute.instance.volume.detach',
         'compute.libvirt.error',
-        'compute.metrics.update',
         'compute_task.build_instances',
         'compute_task.migrate_server',
         'compute_task.rebuild_server',
@@ -396,27 +394,14 @@ class LegacyValidatingNotifier(object):
         getattr(self.notifier, priority)(ctxt, event_type, payload)
 
 
-class ClientWrapper(object):
-    def __init__(self, client):
-        self._client = client
-        self.last_access_time = timeutils.utcnow()
-
-    @property
-    def client(self):
-        self.last_access_time = timeutils.utcnow()
-        return self._client
-
-
 class ClientRouter(periodic_task.PeriodicTasks):
-    """Creates and caches RPC clients that route to cells or the default.
-
-    The default client connects to the API cell message queue. The rest of the
-    clients connect to compute cell message queues.
+    """Creates RPC clients that honor the context's RPC transport
+    or provides a default.
     """
+
     def __init__(self, default_client):
         super(ClientRouter, self).__init__(CONF)
-        self.clients = {}
-        self.clients['default'] = ClientWrapper(default_client)
+        self.default_client = default_client
         self.target = default_client.target
         self.version_cap = default_client.version_cap
         # NOTE(melwitt): Cells v1 does its own serialization and won't
@@ -425,55 +410,11 @@ class ClientRouter(periodic_task.PeriodicTasks):
         # Prevent this empty context from overwriting the thread local copy
         self.run_periodic_tasks(nova.context.RequestContext(overwrite=False))
 
-    def _client(self, context, cell_mapping=None):
-        if cell_mapping:
-            client_id = cell_mapping.uuid
+    def client(self, context):
+        transport = context.mq_connection
+        if transport:
+            return messaging.RPCClient(transport, self.target,
+                                       version_cap=self.version_cap,
+                                       serializer=self.serializer)
         else:
-            client_id = 'default'
-
-        try:
-            client = self.clients[client_id].client
-        except KeyError:
-            transport = create_transport(cell_mapping.transport_url)
-            client = messaging.RPCClient(transport, self.target,
-                                         version_cap=self.version_cap,
-                                         serializer=self.serializer)
-            self.clients[client_id] = ClientWrapper(client)
-
-        return client
-
-    @periodic_task.periodic_task
-    def _remove_stale_clients(self, context):
-        timeout = 60
-
-        def stale(client_id, last_access_time):
-            if timeutils.is_older_than(last_access_time, timeout):
-                LOG.debug('Removing stale RPC client: %s as it was last '
-                          'accessed at %s', client_id, last_access_time)
-                return True
-            return False
-
-        # Never expire the default client
-        items_copy = list(self.clients.items())
-        for client_id, client_wrapper in items_copy:
-            if (client_id != 'default' and
-                    stale(client_id, client_wrapper.last_access_time)):
-                del self.clients[client_id]
-
-    def by_instance(self, context, instance):
-        try:
-            cell_mapping = objects.InstanceMapping.get_by_instance_uuid(
-                               context, instance.uuid).cell_mapping
-        except nova.exception.InstanceMappingNotFound:
-            # Not a cells v2 deployment
-            cell_mapping = None
-        return self._client(context, cell_mapping=cell_mapping)
-
-    def by_host(self, context, host):
-        try:
-            cell_mapping = objects.HostMapping.get_by_host(
-                               context, host).cell_mapping
-        except nova.exception.HostMappingNotFound:
-            # Not a cells v2 deployment
-            cell_mapping = None
-        return self._client(context, cell_mapping=cell_mapping)
+            return self.default_client
