@@ -23,7 +23,10 @@ LOG = logging.getLogger(__name__)
 class ExactDiskFilter(filters.BaseHostFilter):
     """Exact Disk Filter."""
 
-    RUN_ON_REBUILD = False
+    def __init__(self, *args, **kwargs):
+        super(ExactDiskFilter, self).__init__(*args, **kwargs)
+        LOG.warning('ExactDiskFilter is deprecated in Pike and will be '
+                    'removed in a subsequent release.')
 
     def host_passes(self, host_state, spec_obj):
         """Return True if host has the exact amount of disk available."""
