@@ -23,6 +23,11 @@ LOG = logging.getLogger(__name__)
 class ExactRamFilter(filters.BaseHostFilter):
     """Exact RAM Filter."""
 
+    def __init__(self, *args, **kwargs):
+        super(ExactRamFilter, self).__init__(*args, **kwargs)
+        LOG.warning('ExactRamFilter is deprecated in Pike and will be '
+                    'removed in a subsequent release.')
+
     def host_passes(self, host_state, spec_obj):
         """Return True if host has the exact amount of RAM available."""
         requested_ram = spec_obj.memory_mb
