@@ -14,18 +14,32 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
+
 from nova.scheduler import filters
 from nova.scheduler.filters import utils
 
+LOG = logging.getLogger(__name__)
+
 
 class TypeAffinityFilter(filters.BaseHostFilter):
-    """TypeAffinityFilter doesn't allow more than one VM type per host.
+    """DEPRECATED: TypeAffinityFilter doesn't allow more than one VM type
+    per host.
 
     Note: this works best with ram_weight_multiplier
     (spread) set to 1 (default).
     """
 
-    RUN_ON_REBUILD = False
+    def __init__(self):
+        super(TypeAffinityFilter, self).__init__()
+        LOG.warning('TypeAffinityFilter is deprecated for removal in the '
+                    '17.0.0 Queens release. There is no replacement planned '
+                    'for this filter. It is fundamentally flawed in that it '
+                    'relies on the flavors.id primary key and if a flavor '
+                    '\"changed\" (deleted and re-created with new values) '
+                    'it will result in this filter thinking it is a '
+                    'different flavor, thus breaking the usefulness of this '
+                    'filter.')
 
     def host_passes(self, host_state, spec_obj):
         """Dynamically limits hosts to one instance type
@@ -50,8 +64,6 @@ class AggregateTypeAffinityFilter(filters.BaseHostFilter):
     # Aggregate data does not change within a request
     run_filter_once_per_request = True
 
-    RUN_ON_REBUILD = False
-
     def host_passes(self, host_state, spec_obj):
         instance_type = spec_obj.flavor
 
