@@ -55,7 +55,6 @@ SERVICE_MANAGERS = {
     'nova-compute': 'nova.compute.manager.ComputeManager',
     'nova-console': 'nova.console.manager.ConsoleProxyManager',
     'nova-consoleauth': 'nova.consoleauth.manager.ConsoleAuthManager',
-    'nova-cert': 'nova.cert.manager.CertManager',
     'nova-conductor': 'nova.conductor.manager.ConductorManager',
     'nova-metadata': 'nova.api.manager.MetadataManager',
     'nova-scheduler': 'nova.scheduler.manager.SchedulerManager',
@@ -302,7 +301,7 @@ class WSGIService(service.Service):
 
         """
         self.name = name
-        # NOTE(danms): Name can be metadata, os_compute, or ec2, per
+        # NOTE(danms): Name can be metadata, osapi_compute, or ec2, per
         # nova.service's enabled_apis
         self.binary = 'nova-%s' % name
         self.topic = None
@@ -428,7 +427,8 @@ def serve(server, workers=None):
     if _launcher:
         raise RuntimeError(_('serve() can only be called once'))
 
-    _launcher = service.launch(CONF, server, workers=workers)
+    _launcher = service.launch(CONF, server, workers=workers,
+                               restart_method='mutate')
 
 
 def wait():
