@@ -28,20 +28,24 @@ from keystoneauth1 import session as ks
 import mock
 from oslo_concurrency import lockutils
 from oslo_config import cfg
-from oslo_db.sqlalchemy import enginefacade
 import oslo_messaging as messaging
 from oslo_messaging import conffixture as messaging_conffixture
-import six
+from requests import adapters
+from wsgi_intercept import interceptor
 
+from nova.api.openstack.compute import tenant_networks
 from nova.api.openstack.placement import deploy as placement_deploy
+from nova.api.openstack import wsgi_app
 from nova.compute import rpcapi as compute_rpcapi
 from nova import context
 from nova.db import migration
 from nova.db.sqlalchemy import api as session
 from nova import exception
+from nova.network import model as network_model
 from nova import objects
 from nova.objects import base as obj_base
 from nova.objects import service as service_obj
+from nova import quota as nova_quota
 from nova import rpc
 from nova import service
 from nova.tests.functional.api import client
@@ -287,12 +291,18 @@ class SingleCellSimple(fixtures.Fixture):
         self.useFixture(fixtures.MonkeyPatch(
             'nova.objects.InstanceMapping._get_by_instance_uuid_from_db',
             self._fake_instancemapping_get))
+        self.useFixture(fixtures.MonkeyPatch(
+            'nova.objects.InstanceMappingList._get_by_instance_uuids_from_db',
+            self._fake_instancemapping_get_uuids))
         self.useFixture(fixtures.MonkeyPatch(
             'nova.objects.InstanceMapping._save_in_db',
             self._fake_instancemapping_get))
         self.useFixture(fixtures.MonkeyPatch(
             'nova.context.target_cell',
             self._fake_target_cell))
+        self.useFixture(fixtures.MonkeyPatch(
+            'nova.context.set_target_cell',
+            lambda c, m: None))
 
     def _fake_hostmapping_get(self, *args):
         return {'id': 1,
@@ -306,13 +316,17 @@ class SingleCellSimple(fixtures.Fixture):
             'id': 1,
             'updated_at': None,
             'created_at': None,
-            'instance_uuid': uuidsentinel.instance,
+            'instance_uuid': args[-1],
             'cell_id': (self.instances_created and 1 or None),
             'project_id': 'project',
             'cell_mapping': (
                 self.instances_created and self._fake_cell_get() or None),
         }
 
+    def _fake_instancemapping_get_uuids(self, *args):
+        return [self._fake_instancemapping_get(uuid)
+                for uuid in args[-1]]
+
     def _fake_cell_get(self, *args):
         return self._fake_cell_list()[0]
 
@@ -332,6 +346,32 @@ class SingleCellSimple(fixtures.Fixture):
         yield context
 
 
+class CheatingSerializer(rpc.RequestContextSerializer):
+    """A messaging.RequestContextSerializer that helps with cells.
+
+    Our normal serializer does not pass in the context like db_connection
+    and mq_connection, for good reason. We don't really want/need to
+    force a remote RPC server to use our values for this. However,
+    during unit and functional tests, since we're all in the same
+    process, we want cell-targeted RPC calls to preserve these values.
+    Unless we had per-service config and database layer state for
+    the fake services we start, this is a reasonable cheat.
+    """
+    def serialize_context(self, context):
+        """Serialize context with the db_connection inside."""
+        values = super(CheatingSerializer, self).serialize_context(context)
+        values['db_connection'] = context.db_connection
+        values['mq_connection'] = context.mq_connection
+        return values
+
+    def deserialize_context(self, values):
+        """Deserialize context and honor db_connection if present."""
+        ctxt = super(CheatingSerializer, self).deserialize_context(values)
+        ctxt.db_connection = values.pop('db_connection', None)
+        ctxt.mq_connection = values.pop('mq_connection', None)
+        return ctxt
+
+
 class CellDatabases(fixtures.Fixture):
     """Create per-cell databases for testing.
 
@@ -373,6 +413,16 @@ class CellDatabases(fixtures.Fixture):
     @contextmanager
     def _wrap_target_cell(self, context, cell_mapping):
         with self._cell_lock.write_lock():
+            if cell_mapping is None:
+                # NOTE(danms): The real target_cell untargets with a
+                # cell_mapping of None. Since we're controlling our
+                # own targeting in this fixture, we need to call this out
+                # specifically and avoid switching global database state
+                try:
+                    with self._real_target_cell(context, cell_mapping) as c:
+                        yield c
+                finally:
+                    return
             ctxt_mgr = self._ctxt_mgrs[cell_mapping.database_connection]
             # This assumes the next local DB access is the same cell that
             # was targeted last time.
@@ -391,11 +441,40 @@ class CellDatabases(fixtures.Fixture):
         return ctxt_mgr
 
     def _wrap_get_context_manager(self, context):
+        try:
+            # If already targeted, we can proceed without a lock
+            if context.db_connection:
+                return context.db_connection
+        except AttributeError:
+            # Unit tests with None, FakeContext, etc
+            pass
+
         # NOTE(melwitt): This is a hack to try to deal with
         # local accesses i.e. non target_cell accesses.
         with self._cell_lock.read_lock():
+            # FIXME(mriedem): This is actually misleading and means we don't
+            # catch things like bug 1717000 where a context should be targeted
+            # to a cell but it's not, and the fixture here just returns the
+            # last targeted context that was used.
             return self._last_ctxt_mgr
 
+    def _wrap_get_server(self, target, endpoints, serializer=None):
+        """Mirror rpc.get_server() but with our special sauce."""
+        serializer = CheatingSerializer(serializer)
+        return messaging.get_rpc_server(rpc.TRANSPORT,
+                                        target,
+                                        endpoints,
+                                        executor='eventlet',
+                                        serializer=serializer)
+
+    def _wrap_get_client(self, target, version_cap=None, serializer=None):
+        """Mirror rpc.get_client() but with our special sauce."""
+        serializer = CheatingSerializer(serializer)
+        return messaging.RPCClient(rpc.TRANSPORT,
+                                   target,
+                                   version_cap=version_cap,
+                                   serializer=serializer)
+
     def add_cell_database(self, connection_str, default=False):
         """Add a cell database to the fixture.
 
@@ -456,6 +535,13 @@ class CellDatabases(fixtures.Fixture):
             'nova.context.target_cell',
             self._wrap_target_cell))
 
+        self.useFixture(fixtures.MonkeyPatch(
+            'nova.rpc.get_server',
+            self._wrap_get_server))
+        self.useFixture(fixtures.MonkeyPatch(
+            'nova.rpc.get_client',
+            self._wrap_get_client))
+
     def cleanup(self):
         for ctxt_mgr in self._ctxt_mgrs.values():
             engine = ctxt_mgr.get_legacy_facade().get_engine()
@@ -552,6 +638,11 @@ class DefaultFlavorsFixture(fixtures.Fixture):
         ctxt = context.get_admin_context()
         defaults = {'rxtx_factor': 1.0, 'disabled': False, 'is_public': True,
                     'ephemeral_gb': 0, 'swap': 0}
+        extra_specs = {
+            "hw:cpu_model": "SandyBridge",
+            "hw:mem_page_size": "2048",
+            "hw:cpu_policy": "dedicated"
+        }
         default_flavors = [
             objects.Flavor(context=ctxt, memory_mb=512, vcpus=1,
                            root_gb=1, flavorid='1', name='m1.tiny',
@@ -568,6 +659,9 @@ class DefaultFlavorsFixture(fixtures.Fixture):
             objects.Flavor(context=ctxt, memory_mb=16384, vcpus=8,
                            root_gb=160, flavorid='5', name='m1.xlarge',
                            **defaults),
+            objects.Flavor(context=ctxt, memory_mb=512, vcpus=1,
+                           root_gb=1, flavorid='6', name='m1.tiny.specs',
+                           extra_specs=extra_specs, **defaults),
             ]
         for flavor in default_flavors:
             flavor.create()
@@ -593,11 +687,10 @@ class RPCFixture(fixtures.Fixture):
         # one of the many transports we're multplexing here.
         if url not in self._buses:
             exmods = rpc.get_allowed_exmods()
-            self._buses[url] = messaging.get_transport(
+            self._buses[url] = messaging.get_rpc_transport(
                 CONF,
                 url=url,
-                allowed_remote_exmods=exmods,
-                aliases=rpc.TRANSPORT_ALIASES)
+                allowed_remote_exmods=exmods)
         return self._buses[url]
 
     def setUp(self):
@@ -631,6 +724,21 @@ class WarningsFixture(fixtures.Fixture):
         warnings.filterwarnings('ignore',
                                 message='With-statements now directly support'
                                         ' multiple context managers')
+        # NOTE(sdague): nova does not use pkg_resources directly, this
+        # is all very long standing deprecations about other tools
+        # using it. None of this is useful to Nova development.
+        warnings.filterwarnings('ignore',
+            module='pkg_resources')
+        # NOTE(sdague): this remains an unresolved item around the way
+        # forward on is_admin, the deprecation is definitely really premature.
+        warnings.filterwarnings('ignore',
+            message='Policy enforcement is depending on the value of is_admin.'
+                    ' This key is deprecated. Please update your policy '
+                    'file to use the standard policy values.')
+        # NOTE(sdague): mox3 is on life support, don't really care
+        # about any deprecations coming from it
+        warnings.filterwarnings('ignore',
+            module='mox3.mox')
 
         self.addCleanup(warnings.resetwarnings)
 
@@ -658,7 +766,7 @@ class ConfPatcher(fixtures.Fixture):
 
     def setUp(self):
         super(ConfPatcher, self).setUp()
-        for k, v in six.iteritems(self.args):
+        for k, v in self.args.items():
             self.addCleanup(CONF.clear_override, k, self.group)
             CONF.set_override(k, v, self.group)
 
@@ -667,7 +775,7 @@ class OSAPIFixture(fixtures.Fixture):
     """Create an OS API server as a fixture.
 
     This spawns an OS API server as a fixture in a new greenthread in
-    the current test. The fixture has a .api paramenter with is a
+    the current test. The fixture has a .api parameter with is a
     simple rest client that can communicate with it.
 
     This fixture is extremely useful for testing REST responses
@@ -705,27 +813,44 @@ class OSAPIFixture(fixtures.Fixture):
 
     def setUp(self):
         super(OSAPIFixture, self).setUp()
-        # in order to run these in tests we need to bind only to local
-        # host, and dynamically allocate ports
+        # A unique hostname for the wsgi-intercept.
+        hostname = uuidsentinel.osapi_host
+        port = 80
+        service_name = 'osapi_compute'
+        endpoint = 'http://%s:%s/' % (hostname, port)
         conf_overrides = {
-            'osapi_compute_listen': '127.0.0.1',
-            'osapi_compute_listen_port': 0,
-            'verbose': True,
+            'osapi_compute_listen': hostname,
+            'osapi_compute_listen_port': port,
             'debug': True,
         }
         self.useFixture(ConfPatcher(**conf_overrides))
 
-        self.osapi = service.WSGIService("osapi_compute")
-        self.osapi.start()
-        self.addCleanup(self.osapi.stop)
+        # Turn off manipulation of socket_options in TCPKeepAliveAdapter
+        # to keep wsgi-intercept happy. Replace it with the method
+        # from its superclass.
+        self.useFixture(fixtures.MonkeyPatch(
+                'keystoneauth1.session.TCPKeepAliveAdapter.init_poolmanager',
+                adapters.HTTPAdapter.init_poolmanager))
+
+        loader = wsgi.Loader().load_app(service_name)
+        app = lambda: loader
+
+        # re-use service setup code from wsgi_app to register
+        # service, which is looked for in some tests
+        wsgi_app._setup_service(CONF.host, service_name)
+        intercept = interceptor.RequestsInterceptor(app, url=endpoint)
+        intercept.install_intercept()
+        self.addCleanup(intercept.uninstall_intercept)
 
         self.auth_url = 'http://%(host)s:%(port)s/%(api_version)s' % ({
-            'host': self.osapi.host, 'port': self.osapi.port,
-            'api_version': self.api_version})
+            'host': hostname, 'port': port, 'api_version': self.api_version})
         self.api = client.TestOpenStackClient('fake', 'fake', self.auth_url,
                                               self.project_id)
         self.admin_api = client.TestOpenStackClient(
             'admin', 'admin', self.auth_url, self.project_id)
+        # Provide a way to access the wsgi application to tests using
+        # the fixture.
+        self.app = app
 
 
 class OSMetadataServer(fixtures.Fixture):
@@ -746,7 +871,6 @@ class OSMetadataServer(fixtures.Fixture):
         conf_overrides = {
             'metadata_listen': '127.0.0.1',
             'metadata_listen_port': 0,
-            'verbose': True,
             'debug': True
         }
         self.useFixture(ConfPatcher(**conf_overrides))
@@ -879,42 +1003,6 @@ class BannedDBSchemaOperations(fixtures.Fixture):
                 lambda *a, **k: self._explode(thing, 'alter')))
 
 
-class EngineFacadeFixture(fixtures.Fixture):
-    """Fixture to isolation EngineFacade during tests.
-
-    Because many elements of EngineFacade are based on globals, once
-    an engine facade has been initialized, all future code goes
-    through it. This means that the initialization of sqlite in
-    databases in our Database fixture will drive all connections to
-    sqlite. While that's fine in a production environment, during
-    testing this means we can't test against multiple backends in the
-    same test run.
-
-    oslo.db does not yet support a reset mechanism here. This builds a
-    custom in tree engine facade fixture to handle this. Eventually
-    this will be added to oslo.db and this can be removed. Tracked by
-    https://bugs.launchpad.net/oslo.db/+bug/1548960
-
-    """
-    def __init__(self, ctx_manager, engine, sessionmaker):
-        super(EngineFacadeFixture, self).__init__()
-        self._ctx_manager = ctx_manager
-        self._engine = engine
-        self._sessionmaker = sessionmaker
-
-    def setUp(self):
-        super(EngineFacadeFixture, self).setUp()
-
-        self._existing_factory = self._ctx_manager._root_factory
-        self._ctx_manager._root_factory = enginefacade._TestTransactionFactory(
-            self._engine, self._sessionmaker, apply_global=False,
-            synchronous_reader=True)
-        self.addCleanup(self.cleanup)
-
-    def cleanup(self):
-        self._ctx_manager._root_factory = self._existing_factory
-
-
 class ForbidNewLegacyNotificationFixture(fixtures.Fixture):
     """Make sure the test fails if new legacy notification is added"""
     def __init__(self):
@@ -950,6 +1038,18 @@ class AllServicesCurrent(fixtures.Fixture):
         return service_obj.SERVICE_VERSION
 
 
+class RegisterNetworkQuota(fixtures.Fixture):
+    def setUp(self):
+        super(RegisterNetworkQuota, self).setUp()
+        # Quota resource registration modifies the global QUOTAS engine, so
+        # this fixture registers and unregisters network quota for a test.
+        tenant_networks._register_network_quota()
+        self.addCleanup(self.cleanup)
+
+    def cleanup(self):
+        nova_quota.QUOTAS._resources.pop('networks', None)
+
+
 class NeutronFixture(fixtures.Fixture):
     """A fixture to boot instances with neutron ports"""
 
@@ -991,6 +1091,8 @@ class NeutronFixture(fixtures.Fixture):
         'mac_address': 'fa:16:3e:4c:2c:30',
         'fixed_ips': [
             {
+                # The IP on this port must be a prefix of the IP on port_2 to
+                # test listing servers with an ip filter regex.
                 'ip_address': '192.168.1.3',
                 'subnet_id': subnet_1['id']
             }
@@ -998,9 +1100,75 @@ class NeutronFixture(fixtures.Fixture):
         'tenant_id': tenant_id
     }
 
-    def __init__(self, test):
+    # This port is only used if the fixture is created with multiple_ports=True
+    port_2 = {
+        'id': '88dae9fa-0dc6-49e3-8c29-3abc41e99ac9',
+        'network_id': network_1['id'],
+        'admin_state_up': True,
+        'status': 'ACTIVE',
+        'mac_address': '00:0c:29:0d:11:74',
+        'fixed_ips': [
+            {
+                'ip_address': '192.168.1.30',
+                'subnet_id': subnet_1['id']
+            }
+        ],
+        'tenant_id': tenant_id
+    }
+
+    nw_info = [{
+        "profile": {},
+        "ovs_interfaceid": "b71f1699-42be-4515-930a-f3ef01f94aa7",
+        "preserve_on_delete": False,
+        "network": {
+            "bridge": "br-int",
+            "subnets": [{
+                "ips": [{
+                    "meta": {},
+                    "version": 4,
+                    "type": "fixed",
+                    "floating_ips": [],
+                    "address": "10.0.0.4"
+                }],
+                "version": 4,
+                "meta": {},
+                "dns": [],
+                "routes": [],
+                "cidr": "10.0.0.0/26",
+                "gateway": {
+                    "meta": {},
+                    "version": 4,
+                    "type": "gateway",
+                    "address": "10.0.0.1"
+                }
+            }],
+            "meta": {
+                "injected": False,
+                "tenant_id": tenant_id,
+                "mtu": 1500
+            },
+            "id": "e1882e38-38c2-4239-ade7-35d644cb963a",
+            "label": "public"
+        },
+        "devname": "tapb71f1699-42",
+        "vnic_type": "normal",
+        "qbh_params": None,
+        "meta": {},
+        "details": {
+            "port_filter": True,
+            "ovs_hybrid_plug": True
+        },
+        "address": "fa:16:3e:47:94:4a",
+        "active": True,
+        "type": "ovs",
+        "id": "b71f1699-42be-4515-930a-f3ef01f94aa7",
+        "qbg_params": None
+    }]
+
+    def __init__(self, test, multiple_ports=False):
         super(NeutronFixture, self).__init__()
         self.test = test
+        self.multiple_ports = multiple_ports
 
     def setUp(self):
         super(NeutronFixture, self).setUp()
@@ -1019,6 +1187,14 @@ class NeutronFixture(fixtures.Fixture):
         self.test.stub_out(
             'nova.network.neutronv2.api.API.migrate_instance_start',
             lambda *args, **kwargs: None)
+        self.test.stub_out(
+            'nova.network.neutronv2.api.API.add_fixed_ip_to_instance',
+            lambda *args, **kwargs: network_model.NetworkInfo.hydrate(
+                NeutronFixture.nw_info))
+        self.test.stub_out(
+            'nova.network.neutronv2.api.API.remove_fixed_ip_from_instance',
+            lambda *args, **kwargs: network_model.NetworkInfo.hydrate(
+                NeutronFixture.nw_info))
         self.test.stub_out(
             'nova.network.neutronv2.api.API.migrate_instance_finish',
             lambda *args, **kwargs: None)
@@ -1029,17 +1205,29 @@ class NeutronFixture(fixtures.Fixture):
 
         mock_neutron_client = mock.Mock()
         mock_neutron_client.list_extensions.return_value = {'extensions': []}
-        mock_neutron_client.show_port.return_value = {
-            'port': NeutronFixture.port_1}
+
+        def stub_show_port(port_id, *args, **kwargs):
+            if port_id == NeutronFixture.port_1['id']:
+                return {'port': NeutronFixture.port_1}
+            if port_id == NeutronFixture.port_2['id']:
+                return {'port': NeutronFixture.port_2}
+            raise exception.PortNotFound(port_id=port_id)
+
+        mock_neutron_client.show_port.side_effect = stub_show_port
         mock_neutron_client.list_networks.return_value = {
             'networks': [NeutronFixture.network_1]}
-        mock_neutron_client.list_ports.return_value = {
-            'ports': [NeutronFixture.port_1]}
+
+        def stub_list_ports(*args, **kwargs):
+            ports = {'ports': [NeutronFixture.port_1]}
+            if self.multiple_ports:
+                ports['ports'].append(NeutronFixture.port_2)
+            return ports
+
+        mock_neutron_client.list_ports.side_effect = stub_list_ports
         mock_neutron_client.list_subnets.return_value = {
             'subnets': [NeutronFixture.subnet_1]}
         mock_neutron_client.list_floatingips.return_value = {'floatingips': []}
-        mock_neutron_client.update_port.return_value = {
-            'port': NeutronFixture.port_1}
+        mock_neutron_client.update_port.side_effect = stub_show_port
 
         self.test.stub_out(
             'nova.network.neutronv2.api.get_client',
@@ -1084,10 +1272,6 @@ class CinderFixture(fixtures.Fixture):
     SWAP_ERR_OLD_VOL = '828419fa-3efb-4533-b458-4267ca5fe9b1'
     SWAP_ERR_NEW_VOL = '9c6d9c2d-7a8f-4c80-938d-3bf062b8d489'
 
-    # This represents a bootable image-backed volume to test
-    # boot-from-volume scenarios.
-    IMAGE_BACKED_VOL = '6ca404f3-d844-4169-bb96-bc792f37de98'
-
     def __init__(self, test):
         super(CinderFixture, self).__init__()
         self.test = test
@@ -1130,7 +1314,7 @@ class CinderFixture(fixtures.Fixture):
                         },
                         'attach_status': 'attached'
                     })
-                return volume
+                    return volume
 
             # Check to see if the volume is attached.
             for instance_uuid, volumes in self.attachments.items():
@@ -1165,17 +1349,6 @@ class CinderFixture(fixtures.Fixture):
                 [volume['id'] in attachments
                  for attachments in self.attachments.values()])
             volume['status'] = 'attached' if has_attachment else 'detached'
-
-            # Check for our special image-backed volume.
-            if volume_id == self.IMAGE_BACKED_VOL:
-                # Make it a bootable volume.
-                volume['bootable'] = True
-                # Add the image_id metadata.
-                volume['volume_image_metadata'] = {
-                    # There would normally be more image metadata in here...
-                    'image_id': '155d900f-4e14-4e4c-a73d-069cbf4541e6'
-                }
-
             return volume
 
         def fake_initialize_connection(self, context, volume_id, connector):
@@ -1225,10 +1398,6 @@ class CinderFixture(fixtures.Fixture):
 
         self.test.stub_out('nova.volume.cinder.API.begin_detaching',
                            lambda *args, **kwargs: None)
-        self.test.stub_out('nova.volume.cinder.API.check_attach',
-                           lambda *args, **kwargs: None)
-        self.test.stub_out('nova.volume.cinder.API.check_detach',
-                           lambda *args, **kwargs: None)
         self.test.stub_out('nova.volume.cinder.API.get',
                            fake_get)
         self.test.stub_out('nova.volume.cinder.API.initialize_connection',
@@ -1246,6 +1415,14 @@ class CinderFixture(fixtures.Fixture):
                            fake_unreserve_volume)
 
 
+class PlacementApiClient(object):
+    def __init__(self, placement_fixture):
+        self.fixture = placement_fixture
+
+    def get(self, url):
+        return client.APIResponse(self.fixture._fake_get(None, url))
+
+
 class PlacementFixture(fixtures.Fixture):
     """A fixture to placement operations.
 
@@ -1265,12 +1442,20 @@ class PlacementFixture(fixtures.Fixture):
         super(PlacementFixture, self).setUp()
 
         self.useFixture(ConfPatcher(group='api', auth_strategy='noauth2'))
-        app = placement_deploy.loadapp(CONF)
-        # in order to run these in tests we need to bind only to local
-        # host, and dynamically allocate ports
-        self.service = wsgi.Server('placement', app, host='127.0.0.1')
-        self.service.start()
-        self.addCleanup(self.service.stop)
+        loader = placement_deploy.loadapp(CONF)
+        app = lambda: loader
+        host = uuidsentinel.placement_host
+        self.endpoint = 'http://%s/placement' % host
+        intercept = interceptor.RequestsInterceptor(app, url=self.endpoint)
+        intercept.install_intercept()
+        self.addCleanup(intercept.uninstall_intercept)
+
+        # Turn off manipulation of socket_options in TCPKeepAliveAdapter
+        # to keep wsgi-intercept happy. Replace it with the method
+        # from its superclass.
+        self.useFixture(fixtures.MonkeyPatch(
+            'keystoneauth1.session.TCPKeepAliveAdapter.init_poolmanager',
+            adapters.HTTPAdapter.init_poolmanager))
 
         self._client = ks.Session(auth=None)
         # NOTE(sbauza): We need to mock the scheduler report client because
@@ -1289,25 +1474,31 @@ class PlacementFixture(fixtures.Fixture):
             'nova.scheduler.client.report.SchedulerReportClient.delete',
             self._fake_delete))
 
-    def _fake_get(self, *args, **kwargs):
-        (url,) = args[1:]
+        self.api = PlacementApiClient(self)
+
+    @staticmethod
+    def _update_headers_with_version(headers, **kwargs):
         version = kwargs.get("version")
-        # TODO(sbauza): The current placement NoAuthMiddleware returns a 401
-        # in case a token is not provided. We should change that by creating
-        # a fake token so we could remove adding the header below.
-        headers = {'x-auth-token': self.token}
         if version is not None:
             # TODO(mriedem): Perform some version discovery at some point.
             headers.update({
                 'OpenStack-API-Version': 'placement %s' % version
             })
+
+    def _fake_get(self, *args, **kwargs):
+        (url,) = args[1:]
+        # TODO(sbauza): The current placement NoAuthMiddleware returns a 401
+        # in case a token is not provided. We should change that by creating
+        # a fake token so we could remove adding the header below.
+        headers = {'x-auth-token': self.token}
+        self._update_headers_with_version(headers, **kwargs)
         return self._client.get(
             url,
-            endpoint_override="http://127.0.0.1:%s" % self.service.port,
+            endpoint_override=self.endpoint,
             headers=headers,
             raise_exc=False)
 
-    def _fake_post(self, *args):
+    def _fake_post(self, *args, **kwargs):
         (url, data) = args[1:]
         # NOTE(sdague): using json= instead of data= sets the
         # media type to application/json for us. Placement API is
@@ -1316,13 +1507,15 @@ class PlacementFixture(fixtures.Fixture):
         # TODO(sbauza): The current placement NoAuthMiddleware returns a 401
         # in case a token is not provided. We should change that by creating
         # a fake token so we could remove adding the header below.
+        headers = {'x-auth-token': self.token}
+        self._update_headers_with_version(headers, **kwargs)
         return self._client.post(
             url, json=data,
-            endpoint_override="http://127.0.0.1:%s" % self.service.port,
-            headers={'x-auth-token': self.token},
+            endpoint_override=self.endpoint,
+            headers=headers,
             raise_exc=False)
 
-    def _fake_put(self, *args):
+    def _fake_put(self, *args, **kwargs):
         (url, data) = args[1:]
         # NOTE(sdague): using json= instead of data= sets the
         # media type to application/json for us. Placement API is
@@ -1331,10 +1524,12 @@ class PlacementFixture(fixtures.Fixture):
         # TODO(sbauza): The current placement NoAuthMiddleware returns a 401
         # in case a token is not provided. We should change that by creating
         # a fake token so we could remove adding the header below.
+        headers = {'x-auth-token': self.token}
+        self._update_headers_with_version(headers, **kwargs)
         return self._client.put(
             url, json=data,
-            endpoint_override="http://127.0.0.1:%s" % self.service.port,
-            headers={'x-auth-token': self.token},
+            endpoint_override=self.endpoint,
+            headers=headers,
             raise_exc=False)
 
     def _fake_delete(self, *args):
@@ -1344,6 +1539,6 @@ class PlacementFixture(fixtures.Fixture):
         # a fake token so we could remove adding the header below.
         return self._client.delete(
             url,
-            endpoint_override="http://127.0.0.1:%s" % self.service.port,
+            endpoint_override=self.endpoint,
             headers={'x-auth-token': self.token},
             raise_exc=False)
