@@ -15,7 +15,6 @@
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import requests
-import six
 from six.moves.urllib import parse
 
 
@@ -57,17 +56,7 @@ class APIResponse(object):
         self.status = response.status_code
         self.content = response.content
         if self.content:
-            # The Compute API and Placement API handle error responses a bit
-            # differently so we need to check the content-type header to
-            # figure out what to do.
-            content_type = response.headers.get('content-type')
-            if 'application/json' in content_type:
-                self.body = response.json()
-            elif 'text/html' in content_type:
-                self.body = response.text
-            else:
-                raise ValueError('Unexpected response content-type: %s' %
-                                 content_type)
+            self.body = jsonutils.loads(self.content)
         self.headers = response.headers
 
     def __str__(self):
@@ -92,10 +81,6 @@ class OpenStackApiException(Exception):
                         '_body': _body})
 
         super(OpenStackApiException, self).__init__(message)
-        # py35 does not give special meaning to the first arg and store it
-        # as the message variable.
-        if not hasattr(self, 'message'):
-            self.message = message
 
 
 class OpenStackApiAuthenticationException(OpenStackApiException):
@@ -149,7 +134,7 @@ class TestOpenStackClient(object):
         response = requests.request(method, url, data=body, headers=_headers)
         return response
 
-    def _authenticate(self):
+    def _authenticate(self, retry_count=0):
         if self.auth_result:
             return self.auth_result
 
@@ -164,8 +149,15 @@ class TestOpenStackClient(object):
         LOG.debug("%(auth_uri)s => code %(http_status)s",
                   {'auth_uri': auth_uri, 'http_status': http_status})
 
+        # NOTE(cdent): This is a workaround for an issue where the placement
+        # API fixture may respond when a request was supposed to go to the
+        # compute API fixture. Retry a few times, hoping to hit the right
+        # fixture.
         if http_status == 401:
-            raise OpenStackApiAuthenticationException(response=response)
+            if retry_count <= 3:
+                return self._authenticate(retry_count=retry_count + 1)
+            else:
+                raise OpenStackApiAuthenticationException(response=response)
 
         self.auth_result = response.headers
         return self.auth_result
@@ -228,7 +220,7 @@ class TestOpenStackClient(object):
             headers['Content-Type'] = 'application/json'
             kwargs['body'] = jsonutils.dumps(body)
 
-        kwargs.setdefault('check_response_status', [200, 202])
+        kwargs.setdefault('check_response_status', [200, 201, 202])
         return APIResponse(self.api_request(relative_uri, **kwargs))
 
     def api_put(self, relative_uri, body, **kwargs):
@@ -269,7 +261,7 @@ class TestOpenStackClient(object):
 
         if search_opts is not None:
             qparams = {}
-            for opt, val in six.iteritems(search_opts):
+            for opt, val in search_opts.items():
                 qparams[opt] = val
             if qparams:
                 query_string = "?%s" % parse.urlencode(qparams)
@@ -286,20 +278,13 @@ class TestOpenStackClient(object):
     def put_server(self, server_id, server):
         return self.api_put('/servers/%s' % server_id, server).body
 
-    def post_server_action(self, server_id, data):
-        return self.api_post('/servers/%s/action' % server_id, data).body
+    def post_server_action(self, server_id, data, **kwargs):
+        return self.api_post(
+            '/servers/%s/action' % server_id, data, **kwargs).body
 
     def delete_server(self, server_id):
         return self.api_delete('/servers/%s' % server_id)
 
-    def force_down_service(self, host, binary, forced_down):
-        req = {
-            "host": host,
-            "binary": binary,
-            "forced_down": forced_down
-        }
-        return self.api_put('/os-services/force-down', req).body['service']
-
     def get_image(self, image_id):
         return self.api_get('/images/%s' % image_id).body['image']
 
@@ -313,16 +298,6 @@ class TestOpenStackClient(object):
     def delete_image(self, image_id):
         return self.api_delete('/images/%s' % image_id)
 
-    def put_image_meta_key(self, image_id, key, value):
-        """Creates or updates a given image metadata key/value pair."""
-        req_body = {
-            'meta': {
-                key: value
-            }
-        }
-        return self.api_put('/images/%s/metadata/%s' % (image_id, key),
-                            req_body)
-
     def get_flavor(self, flavor_id):
         return self.api_get('/flavors/%s' % flavor_id).body['flavor']
 
@@ -427,6 +402,10 @@ class TestOpenStackClient(object):
     def delete_aggregate(self, aggregate_id):
         self.api_delete('/os-aggregates/%s' % aggregate_id)
 
+    def add_host_to_aggregate(self, aggregate_id, host):
+        return self.api_post('/os-aggregates/%s/action' % aggregate_id,
+                             {'add_host': {'host': host}})
+
     def get_limits(self):
         return self.api_get('/limits').body['limits']
 
@@ -438,11 +417,32 @@ class TestOpenStackClient(object):
         return self.api_put('/servers/%s/tags' % server_id,
                             {'tags': tags}).body['tags']
 
+    def get_port_interfaces(self, server_id):
+        return self.api_get('/servers/%s/os-interface' %
+                            (server_id)).body['interfaceAttachments']
+
+    def detach_interface(self, server_id, port_id):
+        return self.api_delete('/servers/%s/os-interface/%s' %
+                               (server_id, port_id))
+
+    def get_services(self, binary=None, host=None):
+        url = '/os-services?'
+        if binary:
+            url += 'binary=%s&' % binary
+        if host:
+            url += 'host=%s&' % host
+        return self.api_get(url).body['services']
+
+    def put_service(self, service_id, req):
+        return self.api_put(
+            '/os-services/%s' % service_id, req).body['service']
+
+    def post_keypair(self, keypair):
+        return self.api_post('/os-keypairs', keypair).body['keypair']
+
+    def get_active_migrations(self, server_id):
+        return self.api_get('/servers/%s/migrations' %
+                            server_id).body['migrations']
+
     def get_migrations(self):
         return self.api_get('os-migrations').body['migrations']
-
-    def forced_down_service(self, binary, host, force_down=True):
-        return self.api_put('os-services/force-down',
-                            {"host": host,
-                             "binary": binary,
-                             "forced_down": force_down}).body['service']
