@@ -35,7 +35,6 @@ import mock
 from oslo_db.sqlalchemy import test_base
 from oslo_db.sqlalchemy import test_migrations
 from oslo_db.sqlalchemy import utils as db_utils
-from oslo_serialization import jsonutils
 import sqlalchemy
 from sqlalchemy.engine import reflection
 
@@ -44,7 +43,6 @@ from nova.db.sqlalchemy.api_migrations import migrate_repo
 from nova.db.sqlalchemy import api_models
 from nova.db.sqlalchemy import migration as sa_migration
 from nova import test
-from nova.test import uuids
 from nova.tests import fixtures as nova_fixtures
 
 
@@ -92,7 +90,8 @@ class NovaAPIModelsSync(test_migrations.ModelsMigrationsSync):
                     'display_name', 'access_ip_v6', 'access_ip_v4', 'key_name',
                     'locked_by', 'image_ref', 'progress', 'request_spec_id',
                     'info_cache', 'user_id', 'task_state', 'security_groups',
-                    'config_drive']
+                    'config_drive'],
+                'resource_providers': ['can_host'],
         }
 
         for element in diff:
@@ -141,7 +140,7 @@ class NovaAPIMigrationsWalk(test_migrations.WalkVersionsMixin):
     def setUp(self):
         # NOTE(sdague): the oslo_db base test case completely
         # invalidates our logging setup, we actually have to do that
-        # before it is called to keep this from vomitting all over our
+        # before it is called to keep this from vomiting all over our
         # test output.
         self.useFixture(nova_fixtures.StandardLogging())
         super(NovaAPIMigrationsWalk, self).setUp()
@@ -166,10 +165,14 @@ class NovaAPIMigrationsWalk(test_migrations.WalkVersionsMixin):
     def _skippable_migrations(self):
         mitaka_placeholders = list(range(8, 13))
         newton_placeholders = list(range(21, 26))
+        ocata_placeholders = list(range(31, 41))
         special_cases = [
             30,  # Enforcement migration, no changes to test
         ]
-        return mitaka_placeholders + newton_placeholders + special_cases
+        return (mitaka_placeholders +
+                newton_placeholders +
+                ocata_placeholders +
+                special_cases)
 
     def migrate_up(self, version, with_data=False):
         if with_data:
@@ -580,33 +583,63 @@ class NovaAPIMigrationsWalk(test_migrations.WalkVersionsMixin):
         for column in ['created_at', 'updated_at', 'id', 'uuid']:
             self.assertColumnExists(engine, 'placement_aggregates', column)
 
-    def _pre_upgrade_031(self, engine):
-        request_specs = db_utils.get_table(engine, 'request_specs')
-        # The spec value is a serialized json blob.
-        spec = jsonutils.dumps(
-            {"instance_group": {"id": 42,
-                                "members": ["uuid1",
-                                            "uuid2",
-                                            "uuid3"]}})
-        fake_request_spec = {
-            'id': 42, 'spec': spec, 'instance_uuid': uuids.instance}
-        request_specs.insert().execute(fake_request_spec)
+    def _check_041(self, engine, data):
+        self.assertColumnExists(engine, 'traits', 'id')
+        self.assertUniqueConstraintExists(engine, 'traits', ['name'])
 
-    def _check_031(self, engine, data):
-        request_specs = db_utils.get_table(engine, 'request_specs')
-        if engine.name == 'mysql':
-            self.assertIsInstance(request_specs.c.spec.type,
-                                  sqlalchemy.dialects.mysql.MEDIUMTEXT)
+        self.assertColumnExists(engine, 'resource_provider_traits', 'trait_id')
+        self.assertColumnExists(engine, 'resource_provider_traits',
+                                'resource_provider_id')
+        self.assertIndexExists(
+            engine, 'resource_provider_traits',
+            'resource_provider_traits_resource_provider_trait_idx')
 
-        expected_spec = {"instance_group": {"id": 42,
-                                            "members": ["uuid1",
-                                                        "uuid2",
-                                                        "uuid3"]}}
-        from_db_request_spec = request_specs.select(
-            request_specs.c.id == 42).execute().first()
-        self.assertEqual(uuids.instance, from_db_request_spec['instance_uuid'])
-        db_spec = jsonutils.loads(from_db_request_spec['spec'])
-        self.assertDictEqual(expected_spec, db_spec)
+        inspector = reflection.Inspector.from_engine(engine)
+        self.assertEqual(
+            2, len(inspector.get_foreign_keys('resource_provider_traits')))
+        for fk in inspector.get_foreign_keys('resource_provider_traits'):
+            if 'traits' == fk['referred_table']:
+                self.assertEqual(['id'], fk['referred_columns'])
+                self.assertEqual(['trait_id'], fk['constrained_columns'])
+            elif 'resource_providers' == fk['referred_table']:
+                self.assertEqual(['id'], fk['referred_columns'])
+                self.assertEqual(['resource_provider_id'],
+                                 fk['constrained_columns'])
+
+    def _check_042(self, engine, data):
+        self.assertColumnExists(engine, 'build_requests', 'tags')
+
+    def _check_043(self, engine, data):
+        for column in ['created_at', 'updated_at', 'id', 'uuid', 'project_id',
+                       'user_id']:
+            self.assertColumnExists(engine, 'consumers', column)
+
+        self.assertIndexExists(engine, 'consumers',
+                               'consumers_project_id_uuid_idx')
+        self.assertIndexExists(engine, 'consumers',
+                               'consumers_project_id_user_id_uuid_idx')
+        self.assertUniqueConstraintExists(engine, 'consumers', ['uuid'])
+
+    def _check_044(self, engine, data):
+        for column in ['created_at', 'updated_at', 'id', 'external_id']:
+            self.assertColumnExists(engine, 'projects', column)
+            self.assertColumnExists(engine, 'users', column)
+
+        self.assertUniqueConstraintExists(engine, 'projects', ['external_id'])
+        self.assertUniqueConstraintExists(engine, 'users', ['external_id'])
+
+        # We needed to drop and recreate columns and indexes on consumers, so
+        # check that worked out properly
+        self.assertColumnExists(engine, 'consumers', 'project_id')
+        self.assertColumnExists(engine, 'consumers', 'user_id')
+        self.assertIndexExists(
+            engine, 'consumers',
+            'consumers_project_id_uuid_idx',
+        )
+        self.assertIndexExists(
+            engine, 'consumers',
+            'consumers_project_id_user_id_uuid_idx',
+        )
 
 
 class TestNovaAPIMigrationsWalkSQLite(NovaAPIMigrationsWalk,
