@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 import time
 
+import mock
 from oslo_config import cfg
 
 from nova import context
@@ -24,35 +24,33 @@ from nova.db.sqlalchemy import api as db_api
 from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.functional.api import client
-from nova.tests.functional import api_paste_fixture
+from nova.tests.functional import integrated_helpers
 from nova.tests.unit import fake_network
 from nova.tests.unit import policy_fixture
+from nova.virt import fake
 
 import nova.scheduler.utils
 import nova.servicegroup
 import nova.tests.unit.image.fake
 
-CONF = cfg.CONF
-
 # An alternate project id
 PROJECT_ID_ALT = "616c6c796f7572626173656172656f73"
 
+CONF = cfg.CONF
 
-class ServerGroupTestBase(test.TestCase):
+
+class ServerGroupTestBase(test.TestCase,
+                          integrated_helpers.InstanceHelperMixin):
     REQUIRES_LOCKING = True
-    api_major_version = 'v2'
+    api_major_version = 'v2.1'
     microversion = None
-    _image_ref_parameter = 'imageRef'
-    _flavor_ref_parameter = 'flavorRef'
 
-    # Note(gibi): RamFilter is needed to ensure that
-    # test_boot_servers_with_affinity_no_valid_host behaves as expected
-    _scheduler_default_filters = ('ServerGroupAntiAffinityFilter',
-                                  'ServerGroupAffinityFilter',
-                                  'RamFilter')
+    _enabled_filters = (CONF.filter_scheduler.enabled_filters
+                        + ['ServerGroupAntiAffinityFilter',
+                           'ServerGroupAffinityFilter'])
 
     # Override servicegroup parameters to make the tests run faster
-    _service_down_time = 2
+    _service_down_time = 10
     _report_interval = 1
 
     anti_affinity = {'name': 'fake-name-1', 'policies': ['anti-affinity']}
@@ -63,20 +61,22 @@ class ServerGroupTestBase(test.TestCase):
 
     def setUp(self):
         super(ServerGroupTestBase, self).setUp()
-        self.flags(scheduler_default_filters=self._scheduler_default_filters)
-        self.flags(scheduler_weight_classes=self._get_weight_classes())
+        self.flags(enabled_filters=self._enabled_filters,
+                   group='filter_scheduler')
+        # NOTE(sbauza): Don't verify VCPUS and disks given the current nodes.
+        self.flags(cpu_allocation_ratio=9999.0)
+        self.flags(disk_allocation_ratio=9999.0)
+        self.flags(weight_classes=self._get_weight_classes(),
+                   group='filter_scheduler')
         self.flags(service_down_time=self._service_down_time)
         self.flags(report_interval=self._report_interval)
 
         self.useFixture(policy_fixture.RealPolicyFixture())
+        self.useFixture(nova_fixtures.NeutronFixture(self))
 
-        if self.api_major_version == 'v2.1':
-            api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
-                api_version='v2.1'))
-        else:
-            self.useFixture(api_paste_fixture.ApiPasteLegacyV2Fixture())
-            api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
-                api_version='v2'))
+        self.useFixture(nova_fixtures.PlacementFixture())
+        api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
+            api_version='v2.1'))
 
         self.api = api_fixture.api
         self.api.microversion = self.microversion
@@ -86,31 +86,17 @@ class ServerGroupTestBase(test.TestCase):
         # the image fake backend needed for image discovery
         nova.tests.unit.image.fake.stub_out_image_service(self)
 
-        self.start_service('conductor', manager=CONF.conductor.manager)
+        self.start_service('conductor')
         self.start_service('scheduler')
 
         self.addCleanup(nova.tests.unit.image.fake.FakeImageService_reset)
 
-    def _wait_for_state_change(self, server, expected_status, max_retries=10):
-        retry_count = 0
-        while True:
-            server = self.admin_api.get_server(server['id'])
-            if server['status'] == expected_status:
-                break
-            retry_count += 1
-            if retry_count == max_retries:
-                self.fail('Wait for state change failed, '
-                          'expected_status=%s, actual_status=%s'
-                          % (expected_status, server['status']))
-            time.sleep(0.5)
-
-        return server
-
     def _boot_a_server_to_group(self, group,
                                 expected_status='ACTIVE', flavor=None):
-        server = self._build_minimal_create_server_request('some-server')
+        server = self._build_minimal_create_server_request(self.api,
+                                                           'some-server')
         if flavor:
-            server[self._flavor_ref_parameter] = ('http://fake.server/%s'
+            server['flavorRef'] = ('http://fake.server/%s'
                                                   % flavor['id'])
         post = {'server': server,
                 'os:scheduler_hints': {'group': group['id']}}
@@ -118,43 +104,48 @@ class ServerGroupTestBase(test.TestCase):
         self.assertTrue(created_server['id'])
 
         # Wait for it to finish being created
-        found_server = self._wait_for_state_change(created_server,
-                                                   expected_status)
+        found_server = self._wait_for_state_change(
+            self.admin_api, created_server, expected_status)
 
         return found_server
 
-    def _build_minimal_create_server_request(self, name):
-        server = {}
 
-        image = self.api.get_images()[0]
+class ServerGroupFakeDriver(fake.SmallFakeDriver):
+    """A specific fake driver for our tests.
 
-        if self._image_ref_parameter in image:
-            image_href = image[self._image_ref_parameter]
-        else:
-            image_href = image['id']
-            image_href = 'http://fake.server/%s' % image_href
+    Here, we only want to be RAM-bound.
+    """
 
-        # We now have a valid imageId
-        server[self._image_ref_parameter] = image_href
-
-        # Set a valid flavorId
-        flavor = self.api.get_flavors()[1]
-        server[self._flavor_ref_parameter] = ('http://fake.server/%s'
-                                              % flavor['id'])
-        server['name'] = name
-        return server
+    vcpus = 1000
+    memory_mb = 8192
+    local_gb = 100000
 
 
-class ServerGroupTestV2(ServerGroupTestBase):
-    api_major_version = 'v2'
+# A fake way to change the FakeDriver given we don't have a possibility yet to
+# modify the resources for the FakeDriver
+def _fake_load_compute_driver(virtapi, compute_driver=None):
+    return ServerGroupFakeDriver(virtapi)
+
+
+class ServerGroupTestV21(ServerGroupTestBase):
 
     def setUp(self):
-        super(ServerGroupTestV2, self).setUp()
+        super(ServerGroupTestV21, self).setUp()
 
-        self.start_service('network')
+        # TODO(sbauza): Remove that once there is a way to have a custom
+        # FakeDriver supporting different resources. Note that we can't also
+        # simply change the config option for choosing our custom fake driver
+        # as the mocked method only accepts to load drivers in the nova.virt
+        # tree.
+        self.stub_out('nova.virt.driver.load_compute_driver',
+                      _fake_load_compute_driver)
         self.compute = self.start_service('compute')
 
         # NOTE(gibi): start a second compute host to be able to test affinity
+        # NOTE(sbauza): Make sure the FakeDriver returns a different nodename
+        # for the second compute node.
+        fake.set_nodes(['host2'])
+        self.addCleanup(fake.restore_nodes)
         self.compute2 = self.start_service('compute', host='host2')
         fake_network.set_stub_network_methods(self)
 
@@ -271,7 +262,7 @@ class ServerGroupTestV2(ServerGroupTestBase):
     def test_boot_servers_with_affinity_overquota(self):
         # Tests that we check server group member quotas and cleanup created
         # resources when we fail with OverQuota.
-        self.flags(quota_server_group_members=1)
+        self.flags(server_group_members=1, group='quota')
         # make sure we start with 0 servers
         servers = self.api.get_servers(detail=False)
         self.assertEqual(0, len(servers))
@@ -335,13 +326,14 @@ class ServerGroupTestV2(ServerGroupTestBase):
         created_group = self.api.post_server_groups(group)
         servers = self._boot_servers_to_group(created_group)
 
-        post = {'rebuild': {self._image_ref_parameter:
+        post = {'rebuild': {'imageRef':
                             '76fa36fc-c930-4bf3-8c8a-ea2a2420deb6'}}
         self.api.post_server_action(servers[1]['id'], post)
 
-        rebuilt_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        rebuilt_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
-        self.assertEqual(post['rebuild'][self._image_ref_parameter],
+        self.assertEqual(post['rebuild']['imageRef'],
                          rebuilt_server.get('image')['id'])
         return [servers[0], rebuilt_server]
 
@@ -374,6 +366,7 @@ class ServerGroupTestV2(ServerGroupTestBase):
 
     def test_migrate_with_anti_affinity(self):
         # Start additional host to test migration with anti-affinity
+        fake.set_nodes(['host3'])
         self.start_service('compute', host='host3')
 
         created_group = self.api.post_server_groups(self.anti_affinity)
@@ -381,12 +374,27 @@ class ServerGroupTestV2(ServerGroupTestBase):
 
         post = {'migrate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
-        migrated_server = self._wait_for_state_change(servers[1],
-                                                      'VERIFY_RESIZE')
+        migrated_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'VERIFY_RESIZE')
 
         self.assertNotEqual(servers[0]['OS-EXT-SRV-ATTR:host'],
                             migrated_server['OS-EXT-SRV-ATTR:host'])
 
+    def test_resize_to_same_host_with_anti_affinity(self):
+        self.flags(allow_resize_to_same_host=True)
+        created_group = self.api.post_server_groups(self.anti_affinity)
+        servers = self._boot_servers_to_group(created_group,
+                                              flavor=self.api.get_flavors()[0])
+
+        post = {'resize': {'flavorRef': '2'}}
+        server1_old_host = servers[1]['OS-EXT-SRV-ATTR:host']
+        self.admin_api.post_server_action(servers[1]['id'], post)
+        migrated_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'VERIFY_RESIZE')
+
+        self.assertEqual(server1_old_host,
+                         migrated_server['OS-EXT-SRV-ATTR:host'])
+
     def _get_compute_service_by_host_name(self, host_name):
         host = None
         if self.compute.host == host_name:
@@ -417,7 +425,8 @@ class ServerGroupTestV2(ServerGroupTestBase):
 
         post = {'evacuate': {'onSharedStorage': False}}
         self.admin_api.post_server_action(servers[1]['id'], post)
-        evacuated_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        evacuated_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         self.assertNotEqual(evacuated_server['OS-EXT-SRV-ATTR:host'],
                             servers[0]['OS-EXT-SRV-ATTR:host'])
@@ -438,8 +447,8 @@ class ServerGroupTestV2(ServerGroupTestBase):
         post = {'evacuate': {'onSharedStorage': False}}
         self.admin_api.post_server_action(servers[1]['id'], post)
 
-        server_after_failed_evac = self._wait_for_state_change(servers[1],
-                                                               'ACTIVE')
+        server_after_failed_evac = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -462,8 +471,8 @@ class ServerGroupTestV2(ServerGroupTestBase):
         post = {'evacuate': {'onSharedStorage': False}}
         self.admin_api.post_server_action(servers[1]['id'], post)
 
-        server_after_failed_evac = self._wait_for_state_change(servers[1],
-                                                               'ACTIVE')
+        server_after_failed_evac = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -472,11 +481,20 @@ class ServerGroupTestV2(ServerGroupTestBase):
 
         host.start()
 
+    def test_soft_affinity_not_supported(self):
+        ex = self.assertRaises(client.OpenStackApiException,
+                               self.api.post_server_groups,
+                               {'name': 'fake-name-1',
+                                'policies': ['soft-affinity']})
+        self.assertEqual(400, ex.response.status_code)
+        self.assertIn('Invalid input', ex.response.text)
+        self.assertIn('soft-affinity', ex.response.text)
+
 
 class ServerGroupAffinityConfTest(ServerGroupTestBase):
     api_major_version = 'v2.1'
     # Load only anti-affinity filter so affinity will be missing
-    _scheduler_default_filters = 'ServerGroupAntiAffinityFilter'
+    _enabled_filters = ['ServerGroupAntiAffinityFilter']
 
     @mock.patch('nova.scheduler.utils._SUPPORTS_AFFINITY', None)
     def test_affinity_no_filter(self):
@@ -493,7 +511,7 @@ class ServerGroupAffinityConfTest(ServerGroupTestBase):
 class ServerGroupAntiAffinityConfTest(ServerGroupTestBase):
     api_major_version = 'v2.1'
     # Load only affinity filter so anti-affinity will be missing
-    _scheduler_default_filters = 'ServerGroupAffinityFilter'
+    _enabled_filters = ['ServerGroupAffinityFilter']
 
     @mock.patch('nova.scheduler.utils._SUPPORTS_ANTI_AFFINITY', None)
     def test_anti_affinity_no_filter(self):
@@ -536,10 +554,6 @@ class ServerGroupSoftAntiAffinityConfTest(ServerGroupTestBase):
     soft_anti_affinity = {'name': 'fake-name-3',
                           'policies': ['soft-anti-affinity']}
 
-    # Load only soft affinity filter so anti-affinity will be missing
-    _scheduler_weight_classes = ['nova.scheduler.weights.affinity.'
-                                 'ServerGroupSoftAffinityWeigher']
-
     def _get_weight_classes(self):
         # Load only soft affinity filter so anti-affinity will be missing
         return ['nova.scheduler.weights.affinity.'
@@ -557,20 +571,7 @@ class ServerGroupSoftAntiAffinityConfTest(ServerGroupTestBase):
         self.assertEqual(400, failed_server['fault']['code'])
 
 
-class ServerGroupTestV21(ServerGroupTestV2):
-    api_major_version = 'v2.1'
-
-    def test_soft_affinity_not_supported(self):
-        ex = self.assertRaises(client.OpenStackApiException,
-                               self.api.post_server_groups,
-                               {'name': 'fake-name-1',
-                                'policies': ['soft-affinity']})
-        self.assertEqual(400, ex.response.status_code)
-        self.assertIn('Invalid input', ex.response.text)
-        self.assertIn('soft-affinity', ex.response.text)
-
-
-class ServerGroupTestV215(ServerGroupTestV2):
+class ServerGroupTestV215(ServerGroupTestV21):
     api_major_version = 'v2.1'
     microversion = '2.15'
 
@@ -615,7 +616,8 @@ class ServerGroupTestV215(ServerGroupTestV2):
 
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
-        evacuated_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        evacuated_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         self.assertNotEqual(evacuated_server['OS-EXT-SRV-ATTR:host'],
                             servers[0]['OS-EXT-SRV-ATTR:host'])
@@ -637,8 +639,8 @@ class ServerGroupTestV215(ServerGroupTestV2):
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
 
-        server_after_failed_evac = self._wait_for_state_change(servers[1],
-                                                               'ACTIVE')
+        server_after_failed_evac = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -661,8 +663,8 @@ class ServerGroupTestV215(ServerGroupTestV2):
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
 
-        server_after_failed_evac = self._wait_for_state_change(servers[1],
-                                                               'ACTIVE')
+        server_after_failed_evac = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -773,8 +775,8 @@ class ServerGroupTestV215(ServerGroupTestV2):
 
         post = {'migrate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
-        migrated_server = self._wait_for_state_change(servers[1],
-                                                      'VERIFY_RESIZE')
+        migrated_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'VERIFY_RESIZE')
 
         return [migrated_server['OS-EXT-SRV-ATTR:host'],
                 servers[0]['OS-EXT-SRV-ATTR:host']]
@@ -802,7 +804,8 @@ class ServerGroupTestV215(ServerGroupTestV2):
 
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
-        evacuated_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        evacuated_server = self._wait_for_state_change(
+            self.admin_api, servers[1], 'ACTIVE')
 
         # Note(gibi): need to get the server again as the state of the instance
         # goes to ACTIVE first then the host of the instance changes to the
@@ -825,3 +828,6 @@ class ServerGroupTestV215(ServerGroupTestV2):
             self._evacuate_with_soft_anti_affinity_policies(
                 self.soft_anti_affinity))
         self.assertEqual(evacuated_server, other_server)
+
+    def test_soft_affinity_not_supported(self):
+        pass
