@@ -14,6 +14,7 @@ cd $BASE/new/tempest
 
 source $BASE/new/devstack/functions
 source $BASE/new/devstack/functions-common
+source $BASE/new/devstack/lib/nova
 source $WORKSPACE/devstack-gate/functions.sh
 source $BASE/new/nova/nova/tests/live_migration/hooks/utils.sh
 source $BASE/new/nova/nova/tests/live_migration/hooks/nfs.sh
@@ -23,8 +24,6 @@ SUBNODES=$(cat /etc/nodepool/sub_nodes_private)
 SERVICE_HOST=$primary_node
 STACK_USER=${STACK_USER:-stack}
 
-populate_start_script
-
 echo '1. test with all local storage (use default for volumes)'
 echo 'NOTE: test_volume_backed_live_migration is skipped due to https://bugs.launchpad.net/nova/+bug/1524898'
 run_tempest "block migration test" "^.*test_live_migration(?!.*(test_volume_backed_live_migration))"
@@ -42,12 +41,25 @@ echo '2. NFS testing is skipped due to setup failures with Ubuntu 16.04'
 #run_tempest  "NFS shared storage test" "live_migration"
 #nfs_teardown
 
-echo '3. test with Ceph for root + ephemeral disks'
-prepare_ceph
-GLANCE_API_CONF=${GLANCE_API_CONF:-/etc/glance/glance-api.conf}
-configure_and_start_glance
-configure_and_start_nova
-run_tempest "Ceph nova&glance test" "^.*test_live_migration(?!.*(test_volume_backed_live_migration))"
+# NOTE(mriedem): devstack in Pike defaults to using systemd but the old side
+# for grenade is using screen and that follows through to the new side. Since
+# the restart scripts are hard-coded to assume systemd in this job, they will
+# fail if the services weren't started under systemd. So we have to skip this
+# for grenade jobs in Pike until the bug is fixed to handle restarting services
+# running under screen or systemd, or until Queens is our master branch.
+# The GRENADE_OLD_BRANCH variable is exported from devstack-gate, not in the
+# devstack local.conf.
+if [[ "$GRENADE_OLD_BRANCH" == "stable/ocata" ]]; then
+    # TODO(mriedem): Remove this in Queens if we haven't fixed the bug yet.
+    echo '3. Grenade testing with Ceph is disabled until bug 1691769 is fixed or Queens.'
+else
+    echo '3. test with Ceph for root + ephemeral disks'
+    prepare_ceph
+    GLANCE_API_CONF=${GLANCE_API_CONF:-/etc/glance/glance-api.conf}
+    configure_and_start_glance
+    configure_and_start_nova
+    run_tempest "Ceph nova&glance test" "^.*test_live_migration(?!.*(test_volume_backed_live_migration))"
+fi
 set +e
 #echo '4. test with Ceph for volumes and root + ephemeral disk'
 
