@@ -18,8 +18,6 @@ from oslo_config import cfg
 from oslo_serialization import jsonutils
 from webob import exc
 
-from nova.api.openstack.compute import block_device_mapping_v1 \
-        as block_device_mapping
 from nova.api.openstack.compute import extension_info
 from nova.api.openstack.compute import servers as servers_v21
 from nova.compute import api as compute_api
@@ -39,17 +37,8 @@ class BlockDeviceMappingTestV21(test.TestCase):
 
     def _setup_controller(self):
         ext_info = extension_info.LoadedExtensionInfo()
-        CONF.set_override('extensions_blacklist', 'os-block-device-mapping',
-                          'osapi_v21')
         self.controller = servers_v21.ServersController(
                                         extension_info=ext_info)
-        CONF.set_override('extensions_blacklist',
-                          ['os-block-device-mapping-v1',
-                           'os-block-device-mapping'],
-                          'osapi_v21')
-        self.no_volumes_controller = servers_v21.ServersController(
-                extension_info=ext_info)
-        CONF.set_override('extensions_blacklist', '', 'osapi_v21')
 
     def setUp(self):
         super(BlockDeviceMappingTestV21, self).setUp()
@@ -82,8 +71,7 @@ class BlockDeviceMappingTestV21(test.TestCase):
         return body
 
     @mock.patch.object(compute_api.API, '_validate_bdm')
-    def _test_create(self, params, mock_validate_bdm, no_image=False,
-                     override_controller=None):
+    def _test_create(self, params, mock_validate_bdm, no_image=False):
         body = self._get_servers_body(no_image)
         body['server'].update(params)
 
@@ -93,10 +81,7 @@ class BlockDeviceMappingTestV21(test.TestCase):
 
         req.body = jsonutils.dump_as_bytes(body)
 
-        if override_controller:
-            override_controller.create(req, body=body).obj['server']
-        else:
-            self.controller.create(req, body=body).obj['server']
+        self.controller.create(req, body=body).obj['server']
         mock_validate_bdm.assert_called_once_with(
             test.MatchType(fakes.FakeRequestContext),
             test.MatchType(objects.Instance),
@@ -183,19 +168,6 @@ class BlockDeviceMappingTestV21(test.TestCase):
         self.assertRaises(exception.ValidationError,
                           self._test_create, params)
 
-    def test_create_instance_with_volumes_disabled(self):
-        bdm = [{'device_name': 'foo'}]
-        params = {'block_device_mapping': bdm}
-        old_create = compute_api.API.create
-
-        def create(*args, **kwargs):
-            self.assertNotIn(block_device_mapping, kwargs)
-            return old_create(*args, **kwargs)
-
-        self.stub_out('nova.compute.api.API.create', create)
-        self._test_create(params,
-                          override_controller=self.no_volumes_controller)
-
     @mock.patch('nova.compute.api.API._get_bdm_image_metadata')
     def test_create_instance_non_bootable_volume_fails(self, fake_bdm_meta):
         bdm = [{
@@ -356,12 +328,6 @@ class BlockDeviceMappingTestV21(test.TestCase):
                           self._test_create, params)
 
     def test_create_instance_decide_format_legacy(self):
-        ext_info = extension_info.LoadedExtensionInfo()
-        CONF.set_override('extensions_blacklist',
-                          ['os-block-device-mapping',
-                           'os-block-device-mapping-v1'],
-                          'osapi_v21')
-        controller = servers_v21.ServersController(extension_info=ext_info)
         bdm = [{'device_name': 'foo1',
                 'volume_id': fakes.FAKE_UUID,
                 'delete_on_termination': True}]
@@ -377,20 +343,15 @@ class BlockDeviceMappingTestV21(test.TestCase):
 
         self.stub_out('nova.compute.api.API.create', create)
 
-        self._test_create({}, override_controller=controller)
+        self._test_create({})
 
         params = {'block_device_mapping': bdm}
-        self._test_create(params, override_controller=controller)
+        self._test_create(params)
 
     def test_create_instance_both_bdm_formats(self):
-        ext_info = extension_info.LoadedExtensionInfo()
-        CONF.set_override('extensions_blacklist', '', 'osapi_v21')
-        both_controllers = servers_v21.ServersController(
-                extension_info=ext_info)
         bdm = [{'device_name': 'foo'}]
         bdm_v2 = [{'source_type': 'volume',
                    'uuid': 'fake_vol'}]
         params = {'block_device_mapping': bdm,
                   'block_device_mapping_v2': bdm_v2}
-        self.assertRaises(exc.HTTPBadRequest, self._test_create, params,
-                          override_controller=both_controllers)
+        self.assertRaises(exc.HTTPBadRequest, self._test_create, params)
