@@ -17,7 +17,6 @@
 import copy
 import datetime
 import math
-import uuid
 
 import iso8601
 import mock
@@ -211,7 +210,7 @@ class FakeNetworkAPI(object):
         new_id = max((net['id'] for net in self.networks))
         for index, subnet_v4 in enumerate(subnets_v4):
             new_id += 1
-            net = {'id': new_id, 'uuid': str(uuid.uuid4())}
+            net = {'id': new_id, 'uuid': uuids.fake}
 
             net['cidr'] = str(subnet_v4)
             net['netmask'] = str(subnet_v4.netmask)
@@ -298,16 +297,15 @@ class NetworkCreateExceptionsTestV21(test.TestCase):
                           self.controller.create, self.req,
                           body=self.new_network)
 
-    def test_network_create_cidr_conflict(self):
-
-        @staticmethod
-        def get_all(context):
+    @mock.patch.object(objects.NetworkList, 'get_all')
+    def test_network_create_cidr_conflict(self, mock_get_all):
+        def fake_get_all(context):
             ret = objects.NetworkList(context=context, objects=[])
             net = objects.Network(cidr='10.0.0.0/23')
             ret.objects.append(net)
             return ret
 
-        self.stubs.Set(objects.NetworkList, 'get_all', get_all)
+        mock_get_all.side_effect = fake_get_all
 
         self.new_network['network']['cidr'] = '10.0.0.0/24'
         self.assertRaises(webob.exc.HTTPConflict,
