@@ -12,6 +12,7 @@
 
 import mock
 import oslo_messaging as messaging
+import six
 
 from nova.compute import power_state
 from nova.compute import rpcapi as compute_rpcapi
@@ -62,14 +63,21 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
     def test_execute_with_destination(self):
         with test.nested(
             mock.patch.object(self.task, '_check_host_is_up'),
-            mock.patch.object(self.task, '_check_requested_destination'),
+            mock.patch.object(self.task, '_check_requested_destination',
+                              return_value=(mock.sentinel.source_node,
+                                            mock.sentinel.dest_node)),
+            mock.patch.object(scheduler_utils,
+                              'claim_resources_on_destination'),
             mock.patch.object(self.task.compute_rpcapi, 'live_migration'),
-        ) as (mock_check_up, mock_check_dest, mock_mig):
+        ) as (mock_check_up, mock_check_dest, mock_claim, mock_mig):
             mock_mig.return_value = "bob"
 
             self.assertEqual("bob", self.task.execute())
             mock_check_up.assert_called_once_with(self.instance_host)
             mock_check_dest.assert_called_once_with()
+            mock_claim.assert_called_once_with(
+                self.task.scheduler_client.reportclient, self.instance,
+                mock.sentinel.source_node, mock.sentinel.dest_node)
             mock_mig.assert_called_once_with(
                 self.context,
                 host=self.instance_host,
@@ -161,7 +169,8 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
         mock_get_info.return_value = hypervisor_details
         mock_check.return_value = "migrate_data"
 
-        self.task._check_requested_destination()
+        self.assertEqual((hypervisor_details, hypervisor_details),
+                         self.task._check_requested_destination())
         self.assertEqual("migrate_data", self.task.migrate_data)
         mock_get_host.assert_called_once_with(self.context, self.destination)
         mock_is_up.assert_called_once_with("service")
@@ -240,6 +249,33 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
                           mock.call(self.destination)],
                          mock_get_info.call_args_list)
 
+    @mock.patch.object(objects.Service, 'get_by_compute_host')
+    @mock.patch.object(live_migrate.LiveMigrationTask, '_get_compute_info')
+    @mock.patch.object(servicegroup.API, 'service_is_up')
+    @mock.patch.object(compute_rpcapi.ComputeAPI,
+                       'check_can_live_migrate_destination')
+    @mock.patch.object(objects.HostMapping, 'get_by_host',
+                       return_value=objects.HostMapping(
+                           cell_mapping=objects.CellMapping(
+                               uuid=uuids.different)))
+    def test_check_requested_destination_fails_different_cells(
+            self, mock_get_host_mapping, mock_check, mock_is_up,
+            mock_get_info, mock_get_host):
+        mock_get_host.return_value = "service"
+        mock_is_up.return_value = True
+        hypervisor_details = objects.ComputeNode(
+            hypervisor_type="a",
+            hypervisor_version=6.1,
+            free_ram_mb=513,
+            memory_mb=512,
+            ram_allocation_ratio=1.0)
+        mock_get_info.return_value = hypervisor_details
+        mock_check.return_value = "migrate_data"
+
+        ex = self.assertRaises(exception.MigrationPreCheckError,
+                               self.task._check_requested_destination)
+        self.assertIn('across cells', six.text_type(ex))
+
     def test_find_destination_works(self):
         self.mox.StubOutWithMock(utils, 'get_image_from_system_metadata')
         self.mox.StubOutWithMock(scheduler_utils, 'setup_instance_group')
@@ -253,12 +289,11 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         utils.get_image_from_system_metadata(
             self.instance.system_metadata).AndReturn("image")
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
         scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+            self.context, self.fake_spec)
         self.fake_spec.reset_forced_destinations()
         self.task.scheduler_client.select_destinations(
-            self.context, self.fake_spec).AndReturn(
+            self.context, self.fake_spec, [self.instance.uuid]).AndReturn(
                         [{'host': 'host1'}])
         self.task._check_compatible_with_source_hypervisor("host1")
         self.task._call_livem_checks_on_host("host1")
@@ -266,6 +301,10 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
         self.mox.ReplayAll()
         self.assertEqual("host1", self.task._find_destination())
 
+        # Make sure the request_spec was updated to include the cell
+        # mapping.
+        self.assertIsNotNone(self.fake_spec.requested_destination.cell)
+
     def test_find_destination_works_with_no_request_spec(self):
         task = live_migrate.LiveMigrationTask(
             self.context, self.instance, self.destination,
@@ -292,12 +331,12 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             self.assertEqual("host1", task._find_destination())
 
             get_image.assert_called_once_with(self.instance.system_metadata)
-            fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-            setup_ig.assert_called_once_with(
-                self.context, fake_props,
-                {'ignore_hosts': [self.instance_host]}
-            )
-            select_dest.assert_called_once_with(self.context, another_spec)
+            setup_ig.assert_called_once_with(self.context, another_spec)
+            select_dest.assert_called_once_with(self.context, another_spec,
+                    [self.instance.uuid])
+            # Make sure the request_spec was updated to include the cell
+            # mapping.
+            self.assertIsNotNone(another_spec.requested_destination.cell)
             check_compat.assert_called_once_with("host1")
             call_livem_checks.assert_called_once_with("host1")
         do_test()
@@ -312,11 +351,9 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
                 '_check_compatible_with_source_hypervisor')
         self.mox.StubOutWithMock(self.task, '_call_livem_checks_on_host')
 
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-        scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+        scheduler_utils.setup_instance_group(self.context, self.fake_spec)
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
+                self.fake_spec, [self.instance.uuid]).AndReturn(
                         [{'host': 'host1'}])
         self.task._check_compatible_with_source_hypervisor("host1")
         self.task._call_livem_checks_on_host("host1")
@@ -335,23 +372,25 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         utils.get_image_from_system_metadata(
             self.instance.system_metadata).AndReturn("image")
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-        scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+        scheduler_utils.setup_instance_group(self.context, self.fake_spec)
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
-                        [{'host': 'host1'}])
+                self.fake_spec, [self.instance.uuid]).AndReturn(
+                        [{'host': 'host1', 'nodename': 'node1'}])
         self.task._check_compatible_with_source_hypervisor("host1")\
                 .AndRaise(error)
 
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
+                self.fake_spec, [self.instance.uuid]).AndReturn(
                         [{'host': 'host2'}])
         self.task._check_compatible_with_source_hypervisor("host2")
         self.task._call_livem_checks_on_host("host2")
 
         self.mox.ReplayAll()
-        self.assertEqual("host2", self.task._find_destination())
+        with mock.patch.object(self.task,
+                               '_remove_host_allocations') as remove_allocs:
+            self.assertEqual("host2", self.task._find_destination())
+        # Should have removed allocations for the first host.
+        remove_allocs.assert_called_once_with('host1', 'node1')
 
     def test_find_destination_retry_with_old_hypervisor(self):
         self._test_find_destination_retry_hypervisor_raises(
@@ -373,24 +412,26 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         utils.get_image_from_system_metadata(
             self.instance.system_metadata).AndReturn("image")
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-        scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+        scheduler_utils.setup_instance_group(self.context, self.fake_spec)
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
-                        [{'host': 'host1'}])
+                self.fake_spec, [self.instance.uuid]).AndReturn(
+                        [{'host': 'host1', 'nodename': 'node1'}])
         self.task._check_compatible_with_source_hypervisor("host1")
         self.task._call_livem_checks_on_host("host1")\
                 .AndRaise(exception.Invalid)
 
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
+                self.fake_spec, [self.instance.uuid]).AndReturn(
                         [{'host': 'host2'}])
         self.task._check_compatible_with_source_hypervisor("host2")
         self.task._call_livem_checks_on_host("host2")
 
         self.mox.ReplayAll()
-        self.assertEqual("host2", self.task._find_destination())
+        with mock.patch.object(self.task,
+                               '_remove_host_allocations') as remove_allocs:
+            self.assertEqual("host2", self.task._find_destination())
+        # Should have removed allocations for the first host.
+        remove_allocs.assert_called_once_with('host1', 'node1')
 
     def test_find_destination_retry_with_failed_migration_pre_checks(self):
         self.flags(migrate_max_retries=1)
@@ -404,24 +445,26 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         utils.get_image_from_system_metadata(
             self.instance.system_metadata).AndReturn("image")
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-        scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+        scheduler_utils.setup_instance_group(self.context, self.fake_spec)
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
-                        [{'host': 'host1'}])
+                self.fake_spec, [self.instance.uuid]).AndReturn(
+                        [{'host': 'host1', 'nodename': 'node1'}])
         self.task._check_compatible_with_source_hypervisor("host1")
         self.task._call_livem_checks_on_host("host1")\
                 .AndRaise(exception.MigrationPreCheckError("reason"))
 
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
+                self.fake_spec, [self.instance.uuid]).AndReturn(
                         [{'host': 'host2'}])
         self.task._check_compatible_with_source_hypervisor("host2")
         self.task._call_livem_checks_on_host("host2")
 
         self.mox.ReplayAll()
-        self.assertEqual("host2", self.task._find_destination())
+        with mock.patch.object(self.task,
+                               '_remove_host_allocations') as remove_allocs:
+            self.assertEqual("host2", self.task._find_destination())
+        # Should have removed allocations for the first host.
+        remove_allocs.assert_called_once_with('host1', 'node1')
 
     def test_find_destination_retry_exceeds_max(self):
         self.flags(migrate_max_retries=0)
@@ -434,21 +477,26 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
 
         utils.get_image_from_system_metadata(
             self.instance.system_metadata).AndReturn("image")
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-        scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+        scheduler_utils.setup_instance_group(self.context, self.fake_spec)
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndReturn(
-                        [{'host': 'host1'}])
+                self.fake_spec, [self.instance.uuid]).AndReturn(
+                        [{'host': 'host1', 'nodename': 'node1'}])
         self.task._check_compatible_with_source_hypervisor("host1")\
                 .AndRaise(exception.DestinationHypervisorTooOld)
 
         self.mox.ReplayAll()
-        with mock.patch.object(self.task.migration, 'save') as save_mock:
+        with test.nested(
+            mock.patch.object(self.task.migration, 'save'),
+            mock.patch.object(self.task, '_remove_host_allocations')
+        ) as (
+            save_mock, remove_allocs
+        ):
             self.assertRaises(exception.MaxRetriesExceeded,
                               self.task._find_destination)
             self.assertEqual('failed', self.task.migration.status)
             save_mock.assert_called_once_with()
+            # Should have removed allocations for the first host.
+            remove_allocs.assert_called_once_with('host1', 'node1')
 
     def test_find_destination_when_runs_out_of_hosts(self):
         self.mox.StubOutWithMock(utils, 'get_image_from_system_metadata')
@@ -457,11 +505,9 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
                                  'select_destinations')
         utils.get_image_from_system_metadata(
             self.instance.system_metadata).AndReturn("image")
-        fake_props = {'instance_properties': {'uuid': self.instance_uuid}}
-        scheduler_utils.setup_instance_group(
-            self.context, fake_props, {'ignore_hosts': [self.instance_host]})
+        scheduler_utils.setup_instance_group(self.context, self.fake_spec)
         self.task.scheduler_client.select_destinations(self.context,
-                self.fake_spec).AndRaise(
+                self.fake_spec, [self.instance.uuid]).AndRaise(
                         exception.NoValidHost(reason=""))
 
         self.mox.ReplayAll()
@@ -491,3 +537,39 @@ class LiveMigrationTaskTestCase(test.NoDBTestCase):
             side_effect=messaging.MessagingTimeout):
             self.assertRaises(exception.MigrationPreCheckError,
                 self.task._call_livem_checks_on_host, {})
+
+    @mock.patch.object(objects.InstanceMapping, 'get_by_instance_uuid',
+                       side_effect=exception.InstanceMappingNotFound(
+                           uuid=uuids.instance))
+    def test_get_source_cell_mapping_not_found(self, mock_get):
+        """Negative test where InstanceMappingNotFound is raised and converted
+        to MigrationPreCheckError.
+        """
+        self.assertRaises(exception.MigrationPreCheckError,
+                          self.task._get_source_cell_mapping)
+        mock_get.assert_called_once_with(
+            self.task.context, self.task.instance.uuid)
+
+    @mock.patch.object(objects.HostMapping, 'get_by_host',
+                       side_effect=exception.HostMappingNotFound(
+                           name='destination'))
+    def test_get_destination_cell_mapping_not_found(self, mock_get):
+        """Negative test where HostMappingNotFound is raised and converted
+        to MigrationPreCheckError.
+        """
+        self.assertRaises(exception.MigrationPreCheckError,
+                          self.task._get_destination_cell_mapping)
+        mock_get.assert_called_once_with(
+            self.task.context, self.task.destination)
+
+    @mock.patch.object(objects.ComputeNode, 'get_by_host_and_nodename',
+                       side_effect=exception.ComputeHostNotFound(host='host'))
+    def test_remove_host_allocations_compute_host_not_found(self, get_cn):
+        """Tests that failing to find a ComputeNode will not blow up
+        the _remove_host_allocations method.
+        """
+        with mock.patch.object(
+                self.task.scheduler_client.reportclient,
+                'remove_provider_from_instance_allocation') as remove_provider:
+            self.task._remove_host_allocations('host', 'node')
+        remove_provider.assert_not_called()
