@@ -24,6 +24,7 @@ import sqlalchemy
 from nova import context
 from nova.db.sqlalchemy import api as db_api
 from nova.db.sqlalchemy import migration
+from nova.db.sqlalchemy import models
 from nova import exception
 from nova import objects
 from nova import test
@@ -239,100 +240,50 @@ class TestNewtonCheck(test.TestCase):
             '330_enforce_mitaka_online_migrations')
         self.engine = db_api.get_engine()
 
-    def test_all_migrated(self):
-        cn = objects.ComputeNode(context=self.context,
-                                 vcpus=1, memory_mb=512, local_gb=10,
-                                 vcpus_used=0, memory_mb_used=256,
-                                 local_gb_used=5, hypervisor_type='HyperDanVM',
-                                 hypervisor_version='34', cpu_info='foo')
-        cn.create()
-        objects.Aggregate(context=self.context,
-                          name='foo').create()
-        objects.PciDevice.create(self.context, {})
-        self.migration.upgrade(self.engine)
-
-    def test_cn_not_migrated(self):
-        cn = objects.ComputeNode(context=self.context,
-                                 vcpus=1, memory_mb=512, local_gb=10,
-                                 vcpus_used=0, memory_mb_used=256,
-                                 local_gb_used=5, hypervisor_type='HyperDanVM',
-                                 hypervisor_version='34', cpu_info='foo')
-        cn.create()
-        db_api.compute_node_update(self.context, cn.id, {'uuid': None})
-        self.assertRaises(exception.ValidationError,
-                          self.migration.upgrade, self.engine)
-
     def test_aggregate_not_migrated(self):
         agg = db_api.aggregate_create(self.context, {"name": "foobar"})
         db_api.aggregate_update(self.context, agg.id, {'uuid': None})
         self.assertRaises(exception.ValidationError,
                           self.migration.upgrade, self.engine)
 
+    def setup_pci_device(self, dev_type):
+        # NOTE(jaypipes): We cannot use db_api.pci_device_update() here because
+        # newer models of PciDevice contain fields (uuid) that are not present
+        # in the older Newton DB schema and pci_device_update() uses the
+        # SQLAlchemy ORM model_query().update() form which will produce an
+        # UPDATE SQL statement that contains those new fields, resulting in an
+        # OperationalError about table pci_devices has no such column uuid.
+        engine = db_api.get_engine()
+        tbl = models.PciDevice.__table__
+        with engine.connect() as conn:
+            ins_stmt = tbl.insert().values(
+                address='foo:bar',
+                compute_node_id=1,
+                parent_addr=None,
+                vendor_id='123',
+                product_id='456',
+                dev_type=dev_type,
+                label='foobar',
+                status='whatisthis?',
+            )
+            conn.execute(ins_stmt)
+
     def test_pci_device_type_vf_not_migrated(self):
-        db_api.pci_device_update(self.context, 1, 'foo:bar',
-                                 {'parent_addr': None,
-                                  'compute_node_id': 1,
-                                  'address': 'foo:bar',
-                                  'vendor_id': '123',
-                                  'product_id': '456',
-                                  'dev_type': 'type-VF',
-                                  'label': 'foobar',
-                                  'status': 'whatisthis?'})
+        self.setup_pci_device('type-VF')
         # type-VF devices should have a parent_addr
         self.assertRaises(exception.ValidationError,
                           self.migration.upgrade, self.engine)
 
     def test_pci_device_type_pf_not_migrated(self):
-        db_api.pci_device_update(self.context, 1, 'foo:bar',
-                                 {'parent_addr': None,
-                                  'compute_node_id': 1,
-                                  'address': 'foo:bar',
-                                  'vendor_id': '123',
-                                  'product_id': '456',
-                                  'dev_type': 'type-PF',
-                                  'label': 'foobar',
-                                  'status': 'whatisthis?'})
+        self.setup_pci_device('type-PF')
         # blocker should not block on type-PF devices
         self.migration.upgrade(self.engine)
 
     def test_pci_device_type_pci_not_migrated(self):
-        db_api.pci_device_update(self.context, 1, 'foo:bar',
-                                 {'parent_addr': None,
-                                  'compute_node_id': 1,
-                                  'address': 'foo:bar',
-                                  'vendor_id': '123',
-                                  'product_id': '456',
-                                  'dev_type': 'type-PCI',
-                                  'label': 'foobar',
-                                  'status': 'whatisthis?'})
+        self.setup_pci_device('type-PCI')
         # blocker should not block on type-PCI devices
         self.migration.upgrade(self.engine)
 
-    def test_deleted_not_migrated(self):
-        cn_values = dict(vcpus=1, memory_mb=512, local_gb=10,
-                         vcpus_used=0, memory_mb_used=256,
-                         local_gb_used=5, hypervisor_type='HyperDanVM',
-                         hypervisor_version='34', cpu_info='foo')
-        cn = db_api.compute_node_create(self.context, cn_values)
-        agg_values = dict(name='foo')
-        agg = db_api.aggregate_create(self.context, agg_values)
-        pd = db_api.pci_device_update(self.context, 1, 'foo:bar',
-                                 {'parent_addr': None,
-                                  'compute_node_id': 1,
-                                  'address': 'foo:bar',
-                                  'vendor_id': '123',
-                                  'product_id': '456',
-                                  'dev_type': 'foo',
-                                  'label': 'foobar',
-                                  'status': 'whatisthis?'})
-        db_api.compute_node_delete(self.context, cn['id'])
-        db_api.aggregate_delete(self.context, agg['id'])
-        db_api.pci_device_destroy(self.context, pd['compute_node_id'],
-                                  pd['address'])
-
-        # blocker should not block on soft-deleted records
-        self.migration.upgrade(self.engine)
-
 
 class TestOcataCheck(test.TestCase):
     def setUp(self):
