@@ -13,6 +13,7 @@
 #    under the License.
 
 from os_vif import objects as osv_objects
+from os_vif.objects import fields as os_vif_fields
 
 from nova import exception
 from nova.network import model
@@ -475,6 +476,126 @@ class OSVIFUtilTestCase(test.NoDBTestCase):
 
         self.assertObjEqual(expect, actual)
 
+    def test_nova_to_osvif_vif_agilio_ovs_fallthrough(self):
+        vif = model.VIF(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            type=model.VIF_TYPE_AGILIO_OVS,
+            address="22:52:25:62:e2:aa",
+            network=model.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                label="Demo Net",
+                subnets=[]),
+            details={
+                model.VIF_DETAILS_PORT_FILTER: True,
+            }
+        )
+
+        actual = os_vif_util.nova_to_osvif_vif(vif)
+
+        expect = osv_objects.vif.VIFOpenVSwitch(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            active=False,
+            address="22:52:25:62:e2:aa",
+            has_traffic_filtering=True,
+            plugin="ovs",
+            port_profile=osv_objects.vif.VIFPortProfileOpenVSwitch(
+                interface_id="dc065497-3c8d-4f44-8fb4-e1d33c16a536"),
+            preserve_on_delete=False,
+            vif_name="nicdc065497-3c",
+            network=osv_objects.network.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                bridge_interface=None,
+                label="Demo Net",
+                subnets=osv_objects.subnet.SubnetList(
+                    objects=[])))
+
+        self.assertObjEqual(expect, actual)
+
+    def test_nova_to_osvif_vif_agilio_ovs_direct(self):
+        vif = model.VIF(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            type=model.VIF_TYPE_AGILIO_OVS,
+            address="22:52:25:62:e2:aa",
+            profile={
+                "pci_slot": "0000:08:08.5",
+            },
+            network=model.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                label="Demo Net",
+                subnets=[]),
+            vnic_type=model.VNIC_TYPE_DIRECT,
+        )
+
+        actual = os_vif_util.nova_to_osvif_vif(vif)
+
+        expect = osv_objects.vif.VIFHostDevice(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            active=False,
+            has_traffic_filtering=False,
+            address="22:52:25:62:e2:aa",
+            dev_type=osv_objects.fields.VIFHostDeviceDevType.ETHERNET,
+            dev_address="0000:08:08.5",
+            plugin="agilio_ovs",
+            port_profile=osv_objects.vif.VIFPortProfileOVSRepresentor(
+                interface_id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+                representor_name="nicdc065497-3c",
+                representor_address="0000:08:08.5"),
+            preserve_on_delete=False,
+            vif_name="nicdc065497-3c",
+            network=osv_objects.network.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                bridge_interface=None,
+                label="Demo Net",
+                subnets=osv_objects.subnet.SubnetList(
+                    objects=[])))
+
+        self.assertObjEqual(expect, actual)
+
+    def test_nova_to_osvif_vif_agilio_ovs_forwarder(self):
+        vif = model.VIF(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            type=model.VIF_TYPE_AGILIO_OVS,
+            address="22:52:25:62:e2:aa",
+            profile={
+                "pci_slot": "0000:08:08.5",
+            },
+            network=model.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                label="Demo Net",
+                subnets=[]),
+            vnic_type=model.VNIC_TYPE_VIRTIO_FORWARDER,
+            details={
+                model.VIF_DETAILS_VHOSTUSER_MODE: 'client',
+                model.VIF_DETAILS_VHOSTUSER_OVS_PLUG: True,
+                model.VIF_DETAILS_VHOSTUSER_SOCKET: '/fake/socket',
+            }
+        )
+
+        actual = os_vif_util.nova_to_osvif_vif(vif)
+
+        expect = osv_objects.vif.VIFVHostUser(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            active=False,
+            address="22:52:25:62:e2:aa",
+            has_traffic_filtering=False,
+            plugin="agilio_ovs",
+            port_profile=osv_objects.vif.VIFPortProfileOVSRepresentor(
+                interface_id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+                representor_address="0000:08:08.5",
+                representor_name="nicdc065497-3c",),
+            preserve_on_delete=False,
+            vif_name="nicdc065497-3c",
+            path='/fake/socket',
+            mode='client',
+            network=osv_objects.network.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                bridge_interface=None,
+                label="Demo Net",
+                subnets=osv_objects.subnet.SubnetList(
+                    objects=[])))
+
+        self.assertObjEqual(expect, actual)
+
     def test_nova_to_osvif_vif_ovs_plain(self):
         vif = model.VIF(
             id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
@@ -511,6 +632,7 @@ class OSVIFUtilTestCase(test.NoDBTestCase):
         self.assertObjEqual(expect, actual)
 
     def test_nova_to_osvif_vif_ovs_hybrid(self):
+        self.flags(firewall_driver=None)
         vif = model.VIF(
             id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
             type=model.VIF_TYPE_OVS,
@@ -546,6 +668,43 @@ class OSVIFUtilTestCase(test.NoDBTestCase):
 
         self.assertObjEqual(expect, actual)
 
+    def test_nova_to_osvif_ovs_with_vnic_direct(self):
+        vif = model.VIF(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            type=model.VIF_TYPE_OVS,
+            address="22:52:25:62:e2:aa",
+            vnic_type=model.VNIC_TYPE_DIRECT,
+            network=model.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                label="Demo Net",
+                subnets=[]),
+            profile={'pci_slot': '0000:0a:00.1'}
+        )
+
+        actual = os_vif_util.nova_to_osvif_vif(vif)
+
+        expect = osv_objects.vif.VIFHostDevice(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            active=False,
+            address="22:52:25:62:e2:aa",
+            dev_address='0000:0a:00.1',
+            dev_type=os_vif_fields.VIFHostDeviceDevType.ETHERNET,
+            plugin="ovs",
+            port_profile=osv_objects.vif.VIFPortProfileOVSRepresentor(
+                interface_id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+                representor_name="nicdc065497-3c",
+                representor_address="0000:0a:00.1"),
+            has_traffic_filtering=False,
+            preserve_on_delete=False,
+            network=osv_objects.network.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                bridge_interface=None,
+                label="Demo Net",
+                subnets=osv_objects.subnet.SubnetList(
+                    objects=[])))
+
+        self.assertObjEqual(expect, actual)
+
     def test_nova_to_osvif_vhostuser_ovs(self):
         vif = model.VIF(
             id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
@@ -816,3 +975,59 @@ class OSVIFUtilTestCase(test.NoDBTestCase):
         self.assertRaises(exception.NovaException,
                           os_vif_util.nova_to_osvif_vif,
                           vif)
+
+    def test_nova_to_osvif_vhostuser_vrouter(self):
+        vif = model.VIF(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            type=model.VIF_TYPE_VHOSTUSER,
+            address="22:52:25:62:e2:aa",
+            network=model.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                label="Demo Net",
+                subnets=[]),
+            details={
+                model.VIF_DETAILS_VHOSTUSER_MODE: 'client',
+                model.VIF_DETAILS_VHOSTUSER_VROUTER_PLUG: True,
+                model.VIF_DETAILS_VHOSTUSER_SOCKET: '/fake/socket',
+            }
+        )
+
+        actual = os_vif_util.nova_to_osvif_vif(vif)
+
+        expect = osv_objects.vif.VIFVHostUser(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            active=False,
+            address="22:52:25:62:e2:aa",
+            plugin="contrail_vrouter",
+            vif_name="nicdc065497-3c",
+            path='/fake/socket',
+            mode='client',
+            has_traffic_filtering=False,
+            preserve_on_delete=False,
+            network=osv_objects.network.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                bridge_interface=None,
+                label="Demo Net",
+                subnets=osv_objects.subnet.SubnetList(
+                    objects=[])))
+
+        self.assertObjEqual(expect, actual)
+
+    def test_nova_to_osvif_vhostuser_vrouter_no_socket_path(self):
+        vif = model.VIF(
+            id="dc065497-3c8d-4f44-8fb4-e1d33c16a536",
+            type=model.VIF_TYPE_VHOSTUSER,
+            address="22:52:25:62:e2:aa",
+            network=model.Network(
+                id="b82c1929-051e-481d-8110-4669916c7915",
+                label="Demo Net",
+                subnets=[]),
+            details={
+                model.VIF_DETAILS_VHOSTUSER_MODE: 'client',
+                model.VIF_DETAILS_VHOSTUSER_VROUTER_PLUG: True,
+            }
+        )
+
+        self.assertRaises(exception.VifDetailsMissingVhostuserSockPath,
+                          os_vif_util.nova_to_osvif_vif,
+                          vif)
