@@ -70,6 +70,18 @@ class _TestLiveMigrateData(object):
             migrate_data.LiveMigrateData.detect_implementation(legacy),
             migrate_data.XenapiLiveMigrateData)
 
+    def test_obj_make_compatible(self):
+        props = {
+            'serial_listen_addr': '127.0.0.1',
+            'serial_listen_ports': [1000, 10001, 10002, 10003],
+        }
+
+        obj = migrate_data.LibvirtLiveMigrateData(**props)
+        primitive = obj.obj_to_primitive()
+        self.assertIn('serial_listen_ports', primitive['nova_object.data'])
+        obj.obj_make_compatible(primitive['nova_object.data'], '1.1')
+        self.assertNotIn('serial_listen_ports', primitive['nova_object.data'])
+
 
 class TestLiveMigrateData(test_objects._LocalTest,
                           _TestLiveMigrateData):
@@ -228,7 +240,8 @@ class _TestXenapiLiveMigrateData(object):
             block_migration=False,
             destination_sr_ref='foo',
             migrate_send_data={'key': 'val'},
-            sr_uuid_map={'apple': 'banana'})
+            sr_uuid_map={'apple': 'banana'},
+            vif_uuid_map={'orange': 'lemon'})
         expected = {
             'is_volume_backed': False,
             'block_migration': False,
@@ -245,7 +258,8 @@ class _TestXenapiLiveMigrateData(object):
             block_migration=False,
             destination_sr_ref='foo',
             migrate_send_data={'key': 'val'},
-            sr_uuid_map={'apple': 'banana'})
+            sr_uuid_map={'apple': 'banana'},
+            vif_uuid_map={'orange': 'lemon'})
         legacy = obj.to_legacy_dict()
         legacy['ignore_this_thing'] = True
         obj2 = migrate_data.XenapiLiveMigrateData()
@@ -256,7 +270,8 @@ class _TestXenapiLiveMigrateData(object):
         obj = migrate_data.XenapiLiveMigrateData(
             is_volume_backed=False,
             destination_sr_ref='foo',
-            sr_uuid_map={'apple': 'banana'})
+            sr_uuid_map={'apple': 'banana'},
+            vif_uuid_map={'orange': 'lemon'})
         expected = {
             'is_volume_backed': False,
         }
@@ -268,6 +283,7 @@ class _TestXenapiLiveMigrateData(object):
             'is_volume_backed': False,
             'pre_live_migration_result': {
                 'sr_uuid_map': {},
+                'vif_uuid_map': {},
             },
         }
         self.assertEqual(expected, obj.to_legacy_dict(True))
@@ -276,25 +292,47 @@ class _TestXenapiLiveMigrateData(object):
         obj = migrate_data.XenapiLiveMigrateData(
             is_volume_backed=False,
             destination_sr_ref='foo',
-            sr_uuid_map={'apple': 'banana'})
+            sr_uuid_map={'apple': 'banana'},
+            vif_uuid_map={'orange': 'lemon'})
         legacy = obj.to_legacy_dict()
         obj2 = migrate_data.XenapiLiveMigrateData()
         obj2.from_legacy_dict(legacy)
         self.assertFalse(obj2.block_migration)
         self.assertNotIn('migrate_send_data', obj2)
         self.assertNotIn('sr_uuid_map', obj2)
+        self.assertNotIn('vif_uuid_map', obj2)
 
     def test_to_legacy_with_pre_result(self):
         obj = migrate_data.XenapiLiveMigrateData(
-            sr_uuid_map={'a': 'b'})
+            sr_uuid_map={'a': 'b'},
+            vif_uuid_map={'c': 'd'})
         self.assertNotIn('sr_uuid_map', obj.to_legacy_dict())
+        self.assertNotIn('vi_uuid_map', obj.to_legacy_dict())
         legacy = obj.to_legacy_dict(True)
         self.assertEqual(
             {'a': 'b'},
             legacy['pre_live_migration_result']['sr_uuid_map'])
+        self.assertEqual(
+            {'c': 'd'},
+            legacy['pre_live_migration_result']['vif_uuid_map']
+        )
         obj2 = migrate_data.XenapiLiveMigrateData()
         obj2.from_legacy_dict(legacy)
         self.assertEqual({'a': 'b'}, obj2.sr_uuid_map)
+        self.assertEqual({'c': 'd'}, obj2.vif_uuid_map)
+
+    def test_obj_make_compatible(self):
+        obj = migrate_data.XenapiLiveMigrateData(
+            is_volume_backed=False,
+            block_migration=False,
+            destination_sr_ref='foo',
+            migrate_send_data={'key': 'val'},
+            sr_uuid_map={'apple': 'banana'},
+            vif_uuid_map={'orange': 'lemon'})
+        primitive = obj.obj_to_primitive('1.0')
+        self.assertNotIn('vif_uuid_map', primitive['nova_object.data'])
+        primitive2 = obj.obj_to_primitive('1.1')
+        self.assertIn('vif_uuid_map', primitive2['nova_object.data'])
 
 
 class TestXenapiLiveMigrateData(test_objects._LocalTest,
@@ -305,3 +343,97 @@ class TestXenapiLiveMigrateData(test_objects._LocalTest,
 class TestRemoteXenapiLiveMigrateData(test_objects._RemoteTest,
                                       _TestXenapiLiveMigrateData):
     pass
+
+
+class _TestHyperVLiveMigrateData(object):
+    def test_obj_make_compatible(self):
+        obj = migrate_data.HyperVLiveMigrateData(
+            is_shared_instance_path=True)
+        primitive = obj.obj_to_primitive(target_version='1.0')
+        self.assertNotIn('is_shared_instance_path', primitive)
+
+    def test_to_legacy_dict(self):
+        obj = migrate_data.HyperVLiveMigrateData(
+            is_shared_instance_path=False)
+        expected = {
+            'is_shared_instance_path': False,
+        }
+        self.assertEqual(expected, obj.to_legacy_dict())
+
+    def test_from_legacy_dict(self):
+        obj = migrate_data.HyperVLiveMigrateData(
+            is_shared_instance_path=False)
+        legacy = obj.to_legacy_dict()
+        obj2 = migrate_data.HyperVLiveMigrateData()
+        obj2.from_legacy_dict(legacy)
+        self.assertEqual(obj.is_shared_instance_path,
+                         obj2.is_shared_instance_path)
+
+
+class TestHyperVLiveMigrateData(test_objects._LocalTest,
+                                _TestHyperVLiveMigrateData):
+    pass
+
+
+class TestRemoteHyperVLiveMigrateData(test_objects._RemoteTest,
+                                      _TestHyperVLiveMigrateData):
+    pass
+
+
+class _TestPowerVMLiveMigrateData(object):
+    @staticmethod
+    def _mk_obj():
+        return migrate_data.PowerVMLiveMigrateData(
+            host_mig_data=dict(one=2),
+            dest_ip='1.2.3.4',
+            dest_user_id='a_user',
+            dest_sys_name='a_sys',
+            public_key='a_key',
+            dest_proc_compat='POWER7',
+            vol_data=dict(three=4),
+            vea_vlan_mappings=dict(five=6))
+
+    @staticmethod
+    def _mk_leg():
+        return {
+            'host_mig_data': {'one': '2'},
+            'dest_ip': '1.2.3.4',
+            'dest_user_id': 'a_user',
+            'dest_sys_name': 'a_sys',
+            'public_key': 'a_key',
+            'dest_proc_compat': 'POWER7',
+            'vol_data': {'three': '4'},
+            'vea_vlan_mappings': {'five': '6'},
+        }
+
+    def test_migrate_data(self):
+        obj = self._mk_obj()
+        self.assertEqual('a_key', obj.public_key)
+        obj.public_key = 'key2'
+        self.assertEqual('key2', obj.public_key)
+
+    def test_obj_make_compatible(self):
+        obj = self._mk_obj()
+        primitive = obj.obj_to_primitive(target_version='1.0')
+        self.assertNotIn('vea_vlan_mappings', primitive)
+
+    def test_to_legacy_dict(self):
+        self.assertEqual(self._mk_leg(), self._mk_obj().to_legacy_dict())
+
+    def test_from_legacy_dict(self):
+        obj = self._mk_obj()
+        leg = self._mk_leg()
+        obj2 = migrate_data.PowerVMLiveMigrateData()
+        obj2.from_legacy_dict(leg)
+        for field in leg:
+            self.assertEqual(getattr(obj, field), getattr(obj2, field))
+
+
+class TestPowerVMLiveMigrateData(test_objects._LocalTest,
+                                 _TestPowerVMLiveMigrateData):
+    pass
+
+
+class TestRemotePowerVMLiveMigrateData(test_objects._RemoteTest,
+                                      _TestPowerVMLiveMigrateData):
+    pass
