@@ -16,10 +16,12 @@
 """Tests for common notifications."""
 
 import copy
+import datetime
 
 import mock
 from oslo_context import context as o_context
 from oslo_context import fixture as o_fixture
+from oslo_utils import timeutils
 
 from nova.compute import flavors
 from nova.compute import task_states
@@ -66,6 +68,9 @@ class NotificationsTestCase(test.TestCase):
         self.project_id = 'fake'
         self.context = context.RequestContext(self.user_id, self.project_id)
 
+        self.fake_time = datetime.datetime(2017, 2, 2, 16, 45, 0)
+        timeutils.set_time_override(self.fake_time)
+
         self.instance = self._wrapped_create()
 
         self.decorated_function_called = False
@@ -339,6 +344,11 @@ class NotificationsTestCase(test.TestCase):
         self.assertEqual(payload["display_name"], display_name)
         self.assertEqual(payload["hostname"], hostname)
         self.assertEqual(payload["node"], node)
+        self.assertEqual("2017-02-01T00:00:00.000000",
+                         payload["audit_period_beginning"])
+        self.assertEqual("2017-02-02T16:45:00.000000",
+                         payload["audit_period_ending"])
+
         payload = fake_notifier.VERSIONED_NOTIFICATIONS[0][
             'payload']['nova_object.data']
         state_update = payload['state_update']['nova_object.data']
@@ -416,7 +426,7 @@ class NotificationsTestCase(test.TestCase):
         self.assertEqual('testservice.testhost', notif.publisher_id)
 
         notif = fake_notifier.VERSIONED_NOTIFICATIONS[0]
-        self.assertEqual('testservice:testhost', notif['publisher_id'])
+        self.assertEqual('nova-testservice:testhost', notif['publisher_id'])
 
     def test_update_with_host_name(self):
         notifications.send_update_with_states(self.context, self.instance,
@@ -491,6 +501,20 @@ class NotificationsTestCase(test.TestCase):
         self.assertEqual(40, info['ephemeral_gb'])
         self.assertEqual(60, info['disk_gb'])
 
+    def test_payload_has_timestamp_fields(self):
+        time = datetime.datetime(2017, 2, 2, 16, 45, 0)
+        # do not define deleted_at to test that missing value is handled
+        # properly
+        self.instance.terminated_at = time
+        self.instance.launched_at = time
+
+        info = notifications.info_from_instance(self.context, self.instance,
+                                                self.net_info, None)
+
+        self.assertEqual('2017-02-02T16:45:00.000000', info['terminated_at'])
+        self.assertEqual('2017-02-02T16:45:00.000000', info['launched_at'])
+        self.assertEqual('', info['deleted_at'])
+
     def test_send_access_ip_update(self):
         notifications.send_update(self.context, self.instance, self.instance)
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
@@ -520,13 +544,23 @@ class NotificationsTestCase(test.TestCase):
             self.assertEqual(payload["old_display_name"], old_display_name)
             self.assertEqual(payload["display_name"], new_display_name)
 
+    def test_send_versioned_tags_update(self):
+        objects.TagList.create(self.context,
+                               self.instance.uuid, ['tag1', 'tag2'])
+        notifications.send_update(self.context, self.instance, self.instance)
+        self.assertEqual(1, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+
+        self.assertEqual(['tag1', 'tag2'],
+                         fake_notifier.VERSIONED_NOTIFICATIONS[0]
+                         ['payload']['nova_object.data']['tags'])
+
     def test_send_no_state_change(self):
         called = [False]
 
         def sending_no_state_change(context, instance, **kwargs):
             called[0] = True
         self.stub_out('nova.notifications.base.'
-                      '_send_instance_update_notification',
+                      'send_instance_update_notification',
                        sending_no_state_change)
         notifications.send_update(self.context, self.instance, self.instance)
         self.assertTrue(called[0])
@@ -535,7 +569,7 @@ class NotificationsTestCase(test.TestCase):
         def fail_sending(context, instance, **kwargs):
             raise Exception('failed to notify')
         self.stub_out('nova.notifications.base.'
-                      '_send_instance_update_notification',
+                      'send_instance_update_notification',
                        fail_sending)
 
         notifications.send_update(self.context, self.instance, self.instance)
@@ -547,7 +581,7 @@ class NotificationsTestCase(test.TestCase):
         # not logged as an error.
         notfound = exception.InstanceNotFound(instance_id=self.instance.uuid)
         with mock.patch.object(notifications,
-                               '_send_instance_update_notification',
+                               'send_instance_update_notification',
                                side_effect=notfound):
             notifications.send_update(
                 self.context, self.instance, self.instance)
@@ -561,7 +595,7 @@ class NotificationsTestCase(test.TestCase):
         # not logged as an error.
         notfound = exception.InstanceNotFound(instance_id=self.instance.uuid)
         with mock.patch.object(notifications,
-                               '_send_instance_update_notification',
+                               'send_instance_update_notification',
                                side_effect=notfound):
             notifications.send_update_with_states(
                 self.context, self.instance,
