@@ -18,6 +18,8 @@ import mock
 from nova import exception
 from nova import test
 from nova.tests.unit.virt.libvirt import fakelibvirt
+from nova.virt import fake
+from nova.virt.libvirt import driver
 from nova.virt.libvirt import host
 from nova.virt.libvirt.volume import volume
 
@@ -47,6 +49,17 @@ class FakeSecret(object):
         return 0
 
 
+class LibvirtBaseVolumeDriverSubclassSignatureTestCase(
+        test.SubclassSignatureTestCase):
+    def _get_base_class(self):
+        # We do this because it has the side-effect of loading all the
+        # volume drivers
+        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        driver.LibvirtDriver(fake.FakeVirtAPI(), False)
+
+        return volume.LibvirtBaseVolumeDriver
+
+
 class LibvirtVolumeBaseTestCase(test.NoDBTestCase):
     """Contains common setup and helper methods for libvirt volume tests."""
 
@@ -290,22 +303,3 @@ class LibvirtVolumeTestCase(LibvirtISCSIVolumeBaseTestCase):
         conf = libvirt_driver.get_config(connection_info, self.disk_info)
         tree = conf.format_dom()
         self.assertIsNone(tree.find("driver[@discard]"))
-
-    @mock.patch('nova.virt.libvirt.host.Host.has_min_version')
-    def test_libvirt_volume_driver_discard_true_bad_version(
-            self, mock_has_min_version):
-        # Check the discard attrib is not present in driver section
-        mock_has_min_version.return_value = False
-        libvirt_driver = volume.LibvirtVolumeDriver(self.fake_host)
-        connection_info = {
-            'driver_volume_type': 'fake',
-            'data': {
-                'device_path': '/foo',
-                'discard': True,
-            },
-            'serial': 'fake_serial',
-        }
-
-        conf = libvirt_driver.get_config(connection_info, self.disk_info)
-        tree = conf.format_dom()
-        self.assertIsNone(tree.find("driver[@discard]"))
