@@ -28,7 +28,6 @@ from oslo_utils import units
 from oslo_utils import uuidutils
 from oslo_vmware import exceptions as vexc
 from oslo_vmware.objects import datastore as ds_obj
-import six
 
 from nova import exception
 from nova.virt.vmwareapi import constants
@@ -232,7 +231,7 @@ class ManagedObject(object):
 
     def delete(self, attr):
         """Deletes an attribute."""
-        self.propSet = filter(lambda elem: elem.name != attr, self.propSet)
+        self.propSet = [elem for elem in self.propSet if elem.name != attr]
 
     def __setattr__(self, attr, val):
         # TODO(hartsocks): this is adds unnecessary complexity to the class
@@ -1008,7 +1007,7 @@ def create_vm(uuid=None, name=None,
     if vm_path.rel_path == '':
         vm_path = vm_path.join(name, name + '.vmx')
 
-    for key, value in six.iteritems(_db_content["Datastore"]):
+    for key, value in _db_content["Datastore"].items():
         if value.get('summary.name') == vm_path.datastore:
             ds = key
             break
@@ -1242,7 +1241,7 @@ class FakeVim(object):
 
         about_info = DataObject()
         about_info.name = "VMware vCenter Server"
-        about_info.version = "5.1.0"
+        about_info.version = constants.MIN_VC_VERSION
         about_info.instanceUuid = _FAKE_VCENTER_UUID
 
         service_content.about = about_info
