@@ -21,7 +21,7 @@ import time
 from oslo_log import log as logging
 
 import nova.conf
-from nova.i18n import _, _LE, _LI, _LW
+from nova.i18n import _
 from nova import utils
 from nova.virt.disk.mount import api
 
@@ -46,14 +46,14 @@ class NbdMount(api.Mount):
                 if not os.path.exists('/var/lock/qemu-nbd-%s' % device):
                     return device
                 else:
-                    LOG.error(_LE('NBD error - previous umount did not '
-                                  'cleanup /var/lock/qemu-nbd-%s.'), device)
-        LOG.warning(_LW('No free nbd devices'))
+                    LOG.error('NBD error - previous umount did not '
+                              'cleanup /var/lock/qemu-nbd-%s.', device)
+        LOG.warning('No free nbd devices')
         return None
 
     def _allocate_nbd(self):
         if not os.path.exists('/sys/block/nbd0'):
-            LOG.error(_LE('nbd module not loaded'))
+            LOG.error('nbd module not loaded')
             self.error = _('nbd unavailable: module not loaded')
             return None
 
@@ -81,7 +81,7 @@ class NbdMount(api.Mount):
                                  run_as_root=True)
         if err:
             self.error = _('qemu-nbd error: %s') % err
-            LOG.info(_LI('NBD mount error: %s'), self.error)
+            LOG.info('NBD mount error: %s', self.error)
             return False
 
         # NOTE(vish): this forks into another process, so give it a chance
@@ -94,14 +94,14 @@ class NbdMount(api.Mount):
             time.sleep(1)
         else:
             self.error = _('nbd device %s did not show up') % device
-            LOG.info(_LI('NBD mount error: %s'), self.error)
+            LOG.info('NBD mount error: %s', self.error)
 
             # Cleanup
             _out, err = utils.trycmd('qemu-nbd', '-d', device,
                                      run_as_root=True)
             if err:
-                LOG.warning(_LW('Detaching from erroneous nbd device returned '
-                                'error: %s'), err)
+                LOG.warning('Detaching from erroneous nbd device returned '
+                            'error: %s', err)
             return False
 
         self.error = ''
