@@ -27,6 +27,8 @@ higher level APIs around the raw libvirt API. These APIs are
 then used by all the other libvirt related classes
 """
 
+import time
+
 from lxml import etree
 from oslo_log import log as logging
 from oslo_service import loopingcall
@@ -34,13 +36,10 @@ from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
 import six
-import time
 
 from nova.compute import power_state
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LE
-from nova.i18n import _LW
 from nova import utils
 from nova.virt import hardware
 from nova.virt.libvirt import compat
@@ -126,7 +125,7 @@ class Guest(object):
             guest = host.write_instance_config(xml)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error defining a guest with XML: %s'),
+                LOG.error('Error defining a guest with XML: %s',
                           encodeutils.safe_decode(xml))
         return guest
 
@@ -140,8 +139,8 @@ class Guest(object):
             return self._domain.createWithFlags(flags)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error launching a defined domain '
-                              'with XML: %s'),
+                LOG.error('Error launching a defined domain '
+                          'with XML: %s',
                           self._encoded_xml, errors='ignore')
 
     def poweroff(self):
@@ -176,7 +175,7 @@ class Guest(object):
                 LOG.debug('Failed to set time: agent not configured',
                           instance_uuid=self.uuid)
             else:
-                LOG.warning(_LW('Failed to set time: %(reason)s'),
+                LOG.warning('Failed to set time: %(reason)s',
                             {'reason': e}, instance_uuid=self.uuid)
         except Exception as ex:
             # The highest priority is not to let this method crash and thus
@@ -209,7 +208,7 @@ class Guest(object):
                     check_exit_code=[0, 1])
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error enabling hairpin mode with XML: %s'),
+                LOG.error('Error enabling hairpin mode with XML: %s',
                           self._encoded_xml, errors='ignore')
 
     def get_interfaces(self):
@@ -308,6 +307,15 @@ class Guest(object):
         LOG.debug("attach device xml: %s", device_xml)
         self._domain.attachDeviceFlags(device_xml, flags=flags)
 
+    def get_config(self):
+        """Returns the config instance for a guest
+
+        :returns: LibvirtConfigGuest instance
+        """
+        config = vconfig.LibvirtConfigGuest()
+        config.parse_str(self._domain.XMLDesc(0))
+        return config
+
     def get_disk(self, device):
         """Returns the disk mounted at device
 
@@ -641,23 +649,10 @@ class Guest(object):
                 destination, flags=flags, bandwidth=bandwidth)
         else:
             if params:
-                # Due to a quirk in the libvirt python bindings,
-                # VIR_MIGRATE_NON_SHARED_INC with an empty migrate_disks is
-                # interpreted as "block migrate all writable disks" rather than
-                # "don't block migrate any disks". This includes attached
-                # volumes, which will potentially corrupt data on those
-                # volumes. Consequently we need to explicitly unset
-                # VIR_MIGRATE_NON_SHARED_INC if there are no disks to be block
-                # migrated.
-                if (flags & libvirt.VIR_MIGRATE_NON_SHARED_INC != 0 and
-                        not params.get('migrate_disks')):
-                    flags &= ~libvirt.VIR_MIGRATE_NON_SHARED_INC
-
-                # In migrateToURI3 these parameters are extracted from the
-                # `params` dict
                 if migrate_uri:
+                    # In migrateToURI3 this parameter is searched in
+                    # the `params` dict
                     params['migrate_uri'] = migrate_uri
-                params['bandwidth'] = bandwidth
                 self._domain.migrateToURI3(
                     destination, params=params, flags=flags)
             else:
